import { KommentarE2EComponent } from '../../components/kommentar/kommentar-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

describe('Kommentar', () => {
	const mainPage: MainPage = new MainPage();

	const vorgangPage: VorgangPage = new VorgangPage();
	const kommentarContainer: KommentarE2EComponent = vorgangPage.getKommentarContainer();

	let vorgangWithKommentar: VorgangE2E;

	const kommentarText: string = 'Test text to test the test text test';

	before(() => {
		cy.fixture(DataCollections.VORGANG_KOMMENTAR).as('data').then(data => {
			vorgangWithKommentar = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(mainPage.getVorgangList().getRoot());
	})

	describe('create', () => {

		it('Open Vorgang-Detail-Page', () => {
			mainPage.getVorgangList().getListItem(vorgangWithKommentar.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should only show hinzufuegen button initial', () => {
			exist(kommentarContainer.getHinzufuegenButton());
			notExist(kommentarContainer.getFormular());
		})

		describe('click on hinzufuegen button', () => {

			it('should show formular', () => {
				kommentarContainer.getHinzufuegenButton().click();

				exist(kommentarContainer.getFormular());
			})

			it('should hide hinzufuegen button', () => {
				notExist(kommentarContainer.getHinzufuegenButton());
			})

			it('should show formular buttons', () => {
				exist(kommentarContainer.getFormularAbbrechenButton());
				exist(kommentarContainer.getFormularSpeichernButton());
			})
		})

		describe('click on hinzufuegen button in formular', () => {

			it('shoul show error on invalid input', () => {
				kommentarContainer.getFormularSpeichernButton().click();

				exist(kommentarContainer.getTextError());
			})

			it('should show only hinzufuegen button on valid input', () => {
				kommentarContainer.getTextInput().clear().type(kommentarText);

				kommentarContainer.getFormularSpeichernButton().click();

				notExist(kommentarContainer.getTextError());
				notExist(kommentarContainer.getFormular());
				exist(kommentarContainer.getHinzufuegenButton());
			})

			it('should show entry in list', () => {
				exist(kommentarContainer.getKommentar(kommentarText).getRoot());
			})

			it('should show user profile at kommentar', () => {
				haveText(kommentarContainer.getKommentar(kommentarText).getUserProfileIcon().getIconContainer().getAssignedIcon(), 'TT');
				contains(kommentarContainer.getKommentar(kommentarText).getUserProfileName(), 'Theo Test');
				exist(kommentarContainer.getKommentar(kommentarText).getUserProfileCreatedAt());
			})
		})

		describe('click on abbrechen button in formular', () => {

			it('shoul hide formular', () => {
				kommentarContainer.getHinzufuegenButton().click();
				kommentarContainer.getFormularAbbrechenButton().click();

				notExist(kommentarContainer.getFormular());
				exist(kommentarContainer.getHinzufuegenButton());
			})
		})
	})

	describe('edit', () => {

		it('should only show hinzufuegen button initial', () => {
			exist(kommentarContainer.getHinzufuegenButton());
			notExist(kommentarContainer.getFormular());
		})

		describe('click on existing kommentar', () => {

			it('should show formular with value', () => {
				exist(kommentarContainer.getHinzufuegenButton());

				kommentarContainer.getKommentar(kommentarText).getRoot().click();

				exist(kommentarContainer.getFormularAbbrechenButton());
				exist(kommentarContainer.getFormularSpeichernButton());
			})

			it('should show formular buttons', () => {
				exist(kommentarContainer.getFormularAbbrechenButton());
				exist(kommentarContainer.getFormularSpeichernButton());
			})
		})

		describe('click on "speichern" button in formular', () => {

			const kommentarText: string = 'Test text to test the test text test';

			it('should show only hinzufuegen button on valid input', () => {
				kommentarContainer.getTextInput().clear().type(kommentarText);

				kommentarContainer.getFormularSpeichernButton().click();

				notExist(kommentarContainer.getTextError());
				notExist(kommentarContainer.getFormular());
				exist(kommentarContainer.getHinzufuegenButton());
			})

			it('should show entry in list', () => {
				exist(kommentarContainer.getKommentar(kommentarText).getRoot());
			})
		})

		describe('click on abbrechen button in formular', () => {

			it('shoul hide formular', () => {
				kommentarContainer.getKommentar(kommentarText).getRoot().click();
				kommentarContainer.getFormularAbbrechenButton().click();

				notExist(kommentarContainer.getFormular());
				exist(kommentarContainer.getHinzufuegenButton());
			})
		})
	})
});
