import { UserProfileE2EComponent } from '../../components/user-profile/user-profile.component.e2e';
import { VorgangListItemE2EComponent } from '../../components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, enterWith, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

describe('User Profile im ausgewählten Vorgang', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const userProfileContainer: UserProfileE2EComponent = vorgangPage.getVorgangDetailHeader().getUserContainer();

	let vorgang: VorgangE2E;

	const userBeate = {
		name: 'Beate Burger',
		dataTestId: 'Beate_Burger',
		initials: 'BB'
	}

	const userTheo = {
		name: 'Theo Test',
		dataTestId: 'Theo_Test',
		initials: 'TT'
	}

	before(() => {
		cy.fixture(DataCollections.VORGANG_USER_PROFILE).as('data').then(data => {
			vorgang = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.BEATE).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('icon in list', () => {

		it('should have icon', () => {
			const item: VorgangListItemE2EComponent = vorgangList.getListItem(vorgang.name);

			exist(item.getUserProfileItem().getRoot());
		})
	})

	describe('icon in vorgang detail', () => {

		it('open vorgang detail', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show icon', () => {
			exist(userProfileContainer.getIconContainer().getRoot());
		})

		it('should show formular on icon click', () => {
			userProfileContainer.getIconContainer().getRoot().click();

			exist(userProfileContainer.getSearchContainer().getRoot());
		})

		describe('fill formular', () => {

			it('should show error on enter empty field', () => {
				enterWith(userProfileContainer.getSearchContainer().getInput(), 'Clark Kent');

				exist(userProfileContainer.getSearchContainer().getError());
				exist(userProfileContainer.getIconContainer().getUnassignedIcon());
			})

			it('should show error on enter wrong input', () => {
				enterWith(userProfileContainer.getSearchContainer().getInput(), '');

				exist(userProfileContainer.getSearchContainer().getError());
				exist(userProfileContainer.getIconContainer().getUnassignedIcon());
			})

			describe('on valid formular', () => {

				it('should hide formular', () => {
					userProfileContainer.getSearchContainer().getInput().clear().type(userBeate.name);
					exist(userProfileContainer.getSearchContainer().getSearchOption(userBeate.dataTestId));
					userProfileContainer.getSearchContainer().getSearchOption(userBeate.dataTestId).click();

					notExist(userProfileContainer.getSearchContainer().getError());
				})

				it('should show assign icon', () => {
					exist(userProfileContainer.getIconContainer().getAssignedIcon());
					haveText(userProfileContainer.getIconContainer().getAssignedIcon(), userBeate.initials)
				})
			})
		})

		describe('navigate to list', () => {

			it('should show assigned icon', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());

				exist(vorgangList.getListItem(vorgang.name).getUserProfileItem().getRoot());
				haveText(vorgangList.getListItem(vorgang.name).getUserProfileItem().getIconContainer().getAssignedIcon(), userBeate.initials);
			})
		})

		describe('open vorgang detail', () => {

			it('open vorgang detail', () => {
				vorgangList.getListItem(vorgang.name).getRoot().click();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})
		})

		describe('click on subnavigation icon button', () => {

			it('should show formular on icon click', () => {
				vorgangPage.getSubnavigation().clickAssignUserProfileIconButton();

				exist(userProfileContainer.getSearchContainer().getRoot());
			})

			describe('on valid formular', () => {

				it('should hide formular and show assign icon', () => {
					userProfileContainer.getSearchContainer().getInput().clear().type(userTheo.name);
					exist(userProfileContainer.getSearchContainer().getSearchOption(userTheo.dataTestId));
					userProfileContainer.getSearchContainer().getSearchOption(userTheo.dataTestId).click();

					notExist(userProfileContainer.getSearchContainer().getError());
				})

				it('should show assign icon', () => {
					exist(userProfileContainer.getIconContainer().getAssignedIcon());
					haveText(userProfileContainer.getIconContainer().getAssignedIcon(), userTheo.initials)
				})
			})
		})

		describe('navigate to list', () => {

			it('should show assigned icon', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());

				exist(vorgangList.getListItem(vorgang.name).getUserProfileItem().getRoot());
				haveText(vorgangList.getListItem(vorgang.name).getUserProfileItem().getIconContainer().getAssignedIcon(), userTheo.initials);
			})
		})
	})
})