import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, fileDownloaded, MongoCollections, notExist } from '../../support/cypress.util';

describe('Vorgang Anhang', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangKeinAnhang: VorgangE2E;
	let vorgangAnhangVorhanden: VorgangE2E;
	const fileName = 'XML-Daten.xml';

	before(() => {
		cy.fixture(DataCollections.ANHANG).as('data').then(data => {
			vorgangKeinAnhang = data[0];
			vorgangAnhangVorhanden = data[1];

			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('Kein Anhang vorhanden', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangKeinAnhang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('Should show keine Anhänge vorhanden', () => {
			exist(vorgangPage.getAnhangContainer().getEmptyAnhaenge());
		})

		it('click on navigate back', () => {
			vorgangPage.getSubnavigation().clickBackButton();

			vorgangList.waitForSpinnerToDisappear();

			exist(vorgangList.getRoot());
		})
	})

	describe('Anhang herunterladen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangAnhangVorhanden.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('Should download Anhang', () => {
			notExist(vorgangPage.getSpinner());

			exist(vorgangPage.getAnhangContainer().getAnhaenge());

			vorgangPage.getAnhangContainer().getAnhang(fileName).click();

			vorgangPage.getAnhangContainer().waitForDownloadSpinnerToDisappear(fileName);

			fileDownloaded(fileName);
		})

		it('click on navigate back', () => {
			vorgangPage.getSubnavigation().clickBackButton();

			exist(vorgangList.getRoot());
		})
	})
});