import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Verwerfen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangVerwerfen: VorgangE2E;
	let vorgangVerwerfenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangVerwerfen = data[3];
			vorgangVerwerfenRevoke = data[4];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST_EA).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('verwerfen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangVerwerfen.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangVerwerfen.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().clickVerwerfenByButton();

				contains(snackbar.getMessage(), VorgangMessagesE2E.VERWORFEN);
			})

			it('should show snackBar close button', () => {
				exist(snackbar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackbar.getCloseButton().click();

				notExist(snackbar.getMessage());
			})

			it('should have status Verworfen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN]);
			})

			it('should have button zurückholen', () => {
				exist(vorgangPage.getSubnavigation().getZurueckholenIconButton());
				exist(vorgangPage.getFormularContainer().getZurueckholenButton());
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Verworfen', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangVerwerfen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN]);
			})
		})
	})

	describe('verwerfen und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangVerwerfenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangVerwerfenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().clickVerwerfenByIconButton();

				contains(snackbar.getMessage(), VorgangMessagesE2E.VERWORFEN);
			})

			it('should show snackBar close button', () => {
				exist(snackbar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackbar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})

			it('should not have button zurückholen', () => {
				notExist(vorgangPage.getSubnavigation().getZurueckholenIconButton());
				notExist(vorgangPage.getFormularContainer().getZurueckholenButton());
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Neu', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangVerwerfenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})
		})
	})
});
