import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Zurueckholen for User with Role "EINHEITLICHER_ANSPRECHPARTNER"', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangZurueckholen: VorgangE2E;
	let vorgangZurueckholenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangZurueckholen = data[5];
			vorgangZurueckholenRevoke = data[6];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST_EA).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('Zurueckholen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangZurueckholen.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Verworfen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangZurueckholen.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().getZurueckholenButton().click();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ZURUECKGEHOLT);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})

			it('should have button annehmen und verwerfen', () => {
				exist(vorgangPage.getFormularContainer().getVerwerfenButton());
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Neu', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangZurueckholen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})
		})
	})

	describe('Zurueckholen und rueckgaengig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangZurueckholenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangZurueckholenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().getZurueckholenIconButton().click();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ZURUECKGEHOLT);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackBar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status Verworfen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN]);
			})

			it('should have button zurueckholen', () => {
				exist(vorgangPage.getSubnavigation().getZurueckholenIconButton());
				exist(vorgangPage.getFormularContainer().getZurueckholenButton());
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Verworfen', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangZurueckholenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN]);
			})
		})
	})
});
