import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

describe('Vorgang abschliessen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangAbschliessen: VorgangE2E;
	let vorgangAbschliessenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangAbschliessen = data[13];
			vorgangAbschliessenRevoke = data[14];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('abschliessen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.scrollToBottom();

			vorgangList.getListItem(vorgangAbschliessen.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Beschieden', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbschliessen.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().clickAbschliessenByButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ABGESCHLOSSEN)
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status Abgeschlossen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Abgeschlossen', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangAbschliessen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
			})
		})
	})

	describe('abschliessen und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangAbschliessenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Beschieden', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbschliessenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().clickAbschliessenByIconButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ABGESCHLOSSEN);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackBar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status Beschieden', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.BESCHIEDEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Beschieden', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangAbschliessenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.BESCHIEDEN])
			})
		})
	})
});
