import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Annehmen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangAnnehmen: VorgangE2E;
	let vorgangAnnehmenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangAnnehmen = data[1];
			vorgangAnnehmenRevoke = data[2];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})


	describe('annehmen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangAnnehmen.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAnnehmen.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().clickAnnehmenByButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ANGENOMMEN);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Angenommen', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangAnnehmen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})
		})
	})

	describe('annehmen und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangAnnehmenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAnnehmenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().clickAnnehmenByIconButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ANGENOMMEN);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackBar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status Neu', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Neu', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangAnnehmenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.NEU]);
			})
		})
	})
});
