import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

describe('Vorgang bearbeiten', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangBearbeiten: VorgangE2E;
	let vorgangBearbeitenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangBearbeiten = data[7];
			vorgangBearbeitenRevoke = data[8];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data: data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('bearbeiten', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangBearbeiten.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBearbeiten.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().clickBearbeitenByButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.BEARBEITET);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status In Bearbeitung', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status In Bearbeitung', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangBearbeiten.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG])
			})
		})
	})

	describe('bearbeiten und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangBearbeitenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBearbeitenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().clickBearbeitenByIconButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.BEARBEITET);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackBar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Angenommen', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangBearbeitenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN])
			})
		})
	})
});
