import { formatDate, registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { ExpansionPanelE2Eomponent } from '../../components/ui/expansion-panel.e2e.component';
import { VorgangDetailHeaderE2EComponent } from '../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, shouldFirstContains, shouldHaveAttributeBeGreaterThan, shouldHaveAttributeBeLowerThan } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Detailansicht', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const vorgangHeader: VorgangDetailHeaderE2EComponent = new VorgangDetailHeaderE2EComponent();
	const expansionPanelContainer: ExpansionPanelE2Eomponent = vorgangPage.getFormularContainer().getExpansionPanelContainer();

	let vorgang: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgang = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	it('Open Vorgang-Detail-Page', () => {
		vorgangList.getListItem(vorgang.name).getRoot().click();

		exist(vorgangHeader.getRoot());
	})

	it('Check Values', () => {
		testHeaderDaten(vorgang);
	})

	it('Reload Page', () => {
		cy.reload();

		exist(vorgangHeader.getRoot());
	})

	it('Check Values again', () => {
		testHeaderDaten(vorgang);
	})

	it('Check Expanion Panel', () => {
		shouldFirstContains(expansionPanelContainer.getExpansionPanelTitle(), 'Formulardaten');
	})

	it('Check Expansion Panel is open', () => {
		shouldHaveAttributeBeGreaterThan(expansionPanelContainer.getExpansionPanel(), 'outerHeight', 50);
	})

	it('Check if Expansion Panel is closeable', () => {
		expansionPanelContainer.getExpansionPanelTitle().first().click();

		shouldHaveAttributeBeLowerThan(expansionPanelContainer.getExpansionPanel(), 'outerHeight', 50);
	})

	function testHeaderDaten(vorgang: VorgangE2E) {
		const keinAz = 'kein Aktenzeichen zugewiesen';

		haveText(vorgangHeader.getStatus(), vorgangStatusLabelE2E[vorgang.status]);
		haveText(vorgangHeader.getAktenzeichen(), vorgang.aktenzeichen ? vorgang.aktenzeichen : keinAz);
		haveText(vorgangHeader.getName(), vorgang.name);
		haveText(vorgangHeader.getCreatedAt(), formatDate(vorgang.createdAt.$date, 'EEEE, d. MMMM y, HH:mm', 'de'));
	}
});