import { VorgangForwardingE2EComponent } from '../../components/vorgang/vorgang-forward.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';
import { FORWARDING_INVALID_EMAIL } from '../../support/data.util';

describe('Vorgang forwarding fail', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const forwardingContainer: VorgangForwardingE2EComponent = vorgangPage.getForwardingContainer();

	let vorgang: VorgangE2E;

	before(() => {
		cy.task(CypressTasks.DROP_COLLECTION, { collection: MongoCollections.COMMANDS });
		cy.fixture(DataCollections.VORGANG_FORWARDING).as('vorgangData').then(vorgangData => {
			vorgang = vorgangData[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data: vorgangData })
		});

		cy.fixture(DatabaseUser.TEST_EA).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('after click on vorgang', () => {

		it('should show vorgang header', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show forwarding', () => {
			exist(forwardingContainer.getRoot());
		})
	})

	describe('fill forward formular', () => {

		it('should show spinner with text on valid formular', () => {
			const forwardForumlar = forwardingContainer.getFormular();
			forwardForumlar.getZustaendigeStelle().clear().type(FORWARDING_INVALID_EMAIL);
			forwardForumlar.getPasswort().clear().type('validespasswort');

			forwardForumlar.forward();

			exist(forwardingContainer.getSpinner());
		})
	})

	describe('fail forwarding', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG], () => {
			vorgangPage.waitForSpinnerToDisappear();

			haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG])
		})

		it('should show forward fail text', () => {
			exist(forwardingContainer.getForwardingErrorMessage());
		})

		it('should NOT show "mark as success" button', () => {
			notExist(forwardingContainer.getMarkAsSuccessButton());
		})

		it('should show formular', () => {
			exist(forwardingContainer.getFormular().getRoot());
		})
	})
})
