import { VorgangForwardingE2EComponent } from '../../components/vorgang/vorgang-forward.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveText, haveValue, MongoCollections, notExist } from '../../support/cypress.util';
import { FORWARDING_TEST_EMAIL } from '../../support/data.util';

describe('Vorgang forwarding', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const forwardingContainer: VorgangForwardingE2EComponent = vorgangPage.getForwardingContainer();

	let vorgang: VorgangE2E;
	let vorgangEmpty: VorgangE2E;

	before(() => {
		cy.task(CypressTasks.DROP_COLLECTION, { collection: MongoCollections.COMMANDS });
		cy.fixture(DataCollections.VORGANG_FORWARDING).as('data').then(data => {
			vorgang = data[0];
			vorgangEmpty = data[1];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST_EA).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('after click on vorgang', () => {

		it('should show vorgang header', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show forwarding', () => {
			exist(forwardingContainer.getRoot());
		})

		it('should be prefilled', () => {
			haveValue(forwardingContainer.getFormular().getZustaendigeStelle(), 'test@test.de');
		})
	})

	describe('fill forward formular', () => {

		it('should show validation message for invalid email', () => {
			const forwardFormular = forwardingContainer.getFormular();
			forwardFormular.getPasswort().clear().type('zukurz');

			forwardFormular.forward();

			exist(forwardFormular.getPasswortError());
		})

		it('should show validation message for invalid password', () => {
			const forwardFormular = forwardingContainer.getFormular();
			forwardFormular.getZustaendigeStelle().clear().type('invalidemail.com');

			forwardFormular.forward();

			exist(forwardFormular.getZustaendigeStelleError());
		})

		it('should show spinner with text on valid formular', () => {
			const forwardFormular = forwardingContainer.getFormular();
			forwardFormular.getZustaendigeStelle().clear().type(FORWARDING_TEST_EMAIL);
			forwardFormular.getPasswort().clear().type('validespasswort');

			forwardFormular.forward();

			exist(forwardingContainer.getSpinner());
		})
	})

	describe('success forwarding', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET], () => {
			vorgangPage.waitForSpinnerToDisappear();

			haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET])
		})

		it('should show forward success text', () => {
			exist(forwardingContainer.getForwardingSuccessMessage());
		})

		it('should show "mark as success" button', () => {
			exist(forwardingContainer.getMarkAsSuccessButton());
		})

		it('should NOT show formular', () => {
			notExist(forwardingContainer.getFormular().getRoot());
		})

		it('should show "zustellung success message" after click on button', () => {
			forwardingContainer.getMarkAsSuccessButton().click();

			exist(forwardingContainer.getForwardingSuccessMessage());
		})
	})

	describe('switch to list view', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET] + ' in list', () => {
			vorgangPage.getSubnavigation().clickBackButton();

			vorgangList.waitForSpinnerToDisappear();

			haveText(vorgangList.getListItem(vorgang.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET]);
		})
	})

	describe('change to non forwarding vorgang', () => {

		it('should formular after switching vorgang without pending commands', () => {
			vorgangList.getListItem(vorgangEmpty.name).getRoot().click();

			exist(forwardingContainer.getRoot());
		})

		it('should show forward formular', () => {
			exist(forwardingContainer.getFormular().getRoot());
		})

		it('sould not show forwarding success text', () => {
			notExist(forwardingContainer.getForwardingSuccessMessage());
		})
	})
})
