import { VorgangForwardingE2EComponent } from '../../components/vorgang/vorgang-forward.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';
import { FORWARDING_TEST_EMAIL } from '../../support/data.util';

describe('Vorgang forwarding failed', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const forwardingContainer: VorgangForwardingE2EComponent = vorgangPage.getForwardingContainer();

	let vorgang: VorgangE2E;

	before(() => {
		cy.task(CypressTasks.DROP_COLLECTION, { collection: MongoCollections.COMMANDS });
		cy.fixture(DataCollections.VORGANG_FORWARDING).as('data').then(data => {
			vorgang = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST_EA).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('after click on vorgang', () => {

		it('should show vorgang header', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show forwarding', () => {
			exist(forwardingContainer.getRoot());
		})
	})

	describe('fill forward formular', () => {

		it('should show spinner with text on valid formular', () => {
			const formular = forwardingContainer.getFormular();
			formular.getZustaendigeStelle().clear().type(FORWARDING_TEST_EMAIL);
			formular.getPasswort().clear().type('validespasswort');

			formular.forward();

			vorgangPage.waitForSpinnerToDisappear();
		})
	})

	describe('mark forwarding as fail', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET], () => {
			haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET])
		})

		it('should show forward success text', () => {
			exist(forwardingContainer.getForwardingSuccessMessage());
		})

		it('should show "mark as fail" button', () => {
			exist(forwardingContainer.getMarkAsFailButton());
		})

		it('should NOT show formular', () => {
			notExist(forwardingContainer.getFormular().getRoot());
		})

		it('should show "zustellung failed message" after click on button', () => {
			forwardingContainer.getMarkAsFailButton().click();

			exist(forwardingContainer.getFailedMessage());
		})

		it('should show formular', () => {
			exist(forwardingContainer.getFormular().getRoot());
		})

		it('should hide mark as fail button', () => {
			notExist(forwardingContainer.getMarkAsFailButton());
		})

		it('should hide mark as success button', () => {
			notExist(forwardingContainer.getMarkAsSuccessButton());
		})
	})

	describe('switch to list view', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG] + ' in list', () => {
			vorgangPage.getSubnavigation().clickBackButton();

			vorgangList.waitForSpinnerToDisappear();

			haveText(vorgangList.getListItem(vorgang.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
		})
	})
})