import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

describe('Vorgang wiedereroeffnen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangBeschiedenWiedereroeffnen: VorgangE2E;
	let vorgangBeschiedenWiedereroeffnenRevoke: VorgangE2E;
	let vorgangAbgeschlossenWiedereroeffnen: VorgangE2E;
	let vorgangAbgeschlossenWiedereroeffnenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG_WIEDEREROEFFNEN).as('vorgangData').then(vorgangData => {
			vorgangBeschiedenWiedereroeffnen = vorgangData[0];
			vorgangBeschiedenWiedereroeffnenRevoke = vorgangData[1];

			vorgangAbgeschlossenWiedereroeffnen = vorgangData[2];
			vorgangAbgeschlossenWiedereroeffnenRevoke = vorgangData[3];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data: vorgangData })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('vorgang in status beschieden', () => {

		describe('wiedereroeffnen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangBeschiedenWiedereroeffnen.name).getRoot().click();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by button', () => {

				it('should have status Beschieden', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBeschiedenWiedereroeffnen.status]);
				})

				it('should show snackBar message', () => {
					vorgangPage.getFormularContainer().clickWiedereroeffnenByButton();

					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should show snackBar close button', () => {
					exist(snackbar.getCloseButton());
				})

				it('should close snackBar', () => {
					snackbar.getCloseButton().click();

					notExist(snackbar.getMessage());
				})

				it('should have status In Bearbeitung', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().clickBackButton();

					exist(vorgangList.getRoot());
				})

				it('should have status In Bearbeitung', () => {
					vorgangList.waitForSpinnerToDisappear();

					haveText(vorgangList.getListItem(vorgangBeschiedenWiedereroeffnen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})
			})
		})

		describe('wiedereroeffnen und rückgängig machen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangBeschiedenWiedereroeffnenRevoke.name).getRoot().click();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by icon-button', () => {
				it('should have status Beschieden', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBeschiedenWiedereroeffnenRevoke.status]);
				})

				it('should show snackBar message', () => {
					vorgangPage.getSubnavigation().clickWiedereroeffnenByIconButton();

					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should show snackBar close button', () => {
					exist(snackbar.getCloseButton());
				})

				it('should show progressbar', () => {
					snackbar.getRevokeButton().click();

					exist(vorgangPage.getProgressBar());
				})

				it('should show status Beschieden', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.BESCHIEDEN]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().clickBackButton();

					exist(vorgangList.getRoot());
				})

				it('should have status Beschieden', () => {
					vorgangList.waitForSpinnerToDisappear();

					haveText(vorgangList.getListItem(vorgangBeschiedenWiedereroeffnenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.BESCHIEDEN]);
				})
			})
		})
	})

	describe('vorgang in status abgeschlossen', () => {

		describe('wiedereroeffnen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnen.name).getRoot().click();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by button', () => {

				it('should have status Abgeschlossen', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbgeschlossenWiedereroeffnen.status]);
				})

				it('should show snackBar message', () => {
					vorgangPage.getFormularContainer().clickWiedereroeffnenByButton();

					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should show snackBar close button', () => {
					exist(snackbar.getCloseButton());
				})

				it('should close snackBar', () => {
					snackbar.getCloseButton().click();

					notExist(snackbar.getMessage());
				})

				it('should have status In Bearbeitung', () => {
					vorgangPage.getVorgangDetailHeader().getStatus().should('exist').should('have.text', vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().clickBackButton();

					exist(vorgangList.getRoot());
				})

				it('should have status In Bearbeitung', () => {
					vorgangList.waitForSpinnerToDisappear();

					haveText(vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
				})
			})
		})

		describe('wiedereroeffnen und rückgängig machen', () => {

			it('Open Vorgang-Detail-Page', () => {
				vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnenRevoke.name).getRoot().click();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			describe('by icon-button', () => {

				it('should have status Abgeschlossen', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangAbgeschlossenWiedereroeffnenRevoke.status]);
				})

				it('should show snackBar message', () => {
					vorgangPage.getSubnavigation().clickWiedereroeffnenByIconButton();

					contains(snackbar.getMessage(), VorgangMessagesE2E.WIEDEREROEFFNET);
				})

				it('should show snackBar close button', () => {
					exist(snackbar.getCloseButton());
				})

				it('should show progressbar', () => {
					snackbar.getRevokeButton().click();

					exist(vorgangPage.getProgressBar());
				})

				it('should show status Abgeschlossen', () => {
					haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
				})

				it('back to vorgang list', () => {
					vorgangPage.getSubnavigation().clickBackButton();

					exist(vorgangList.getRoot());
				})

				it('should have status Abgeschlossen', () => {
					vorgangList.waitForSpinnerToDisappear();

					haveText(vorgangList.getListItem(vorgangAbgeschlossenWiedereroeffnenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN]);
				})
			})
		})
	})
});
