import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { contains, CypressTasks, DatabaseUser, DataCollections, exist, haveText, MongoCollections, notExist } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Zurueckstellen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangZurueckstellen: VorgangE2E;
	let vorgangZurueckstellenRevoke: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG).as('data').then(data => {
			vorgangZurueckstellen = data[11];
			vorgangZurueckstellenRevoke = data[12];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('zurueckstellen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangZurueckstellen.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status In Bearbeitung', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangZurueckstellen.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getFormularContainer().clickZurueckstellenByButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ZURUECKGESTELLT);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status Zurückgestellt', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangZurueckstellen.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})
		})
	})

	describe('zurückstellen und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangZurueckstellenRevoke.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status In Bearbeitung', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangZurueckstellenRevoke.status]);
			})

			it('should show snackBar message', () => {
				vorgangPage.getSubnavigation().clickZurueckstellenByIconButton();

				contains(snackBar.getMessage(), VorgangMessagesE2E.ZURUECKGESTELLT);
			})

			it('should show snackBar close button', () => {
				exist(snackBar.getCloseButton());
			})

			it('should show progressbar', () => {
				snackBar.getRevokeButton().click();

				exist(vorgangPage.getProgressBar());
			})

			it('should show status In Bearbeitung', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().clickBackButton();

				exist(vorgangList.getRoot());
			})

			it('should have status In Bearbeitung', () => {
				vorgangList.waitForSpinnerToDisappear();

				haveText(vorgangList.getListItem(vorgangZurueckstellenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
			})
		})
	})
});
