import { VorgangListItemE2EComponent } from '../../components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { containClass, CypressTasks, DatabaseUser, DataCollections, exist, MongoCollections, mouseEnter, notContainClass, notExist } from '../../support/cypress.util';

describe('VorgangList Wiedervorlagen Next Frist', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgang: VorgangE2E;
	let vorgangWithWiedervorlageInFuture: VorgangE2E;
	let vorgangWithWiedervorlageInThePast: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.WIEDERVORLAGEN).as('data').then(data => {
			vorgang = data[0];
			vorgangWithWiedervorlageInFuture = data[1];
			vorgangWithWiedervorlageInThePast = data[2];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
		vorgangPage.waitForSpinnerToDisappear();
	})

	describe('on vorgang with no wiedervorlagen', () => {

		it('should not show next frist icon', () => {
			const item: VorgangListItemE2EComponent = vorgangList.getListItem(vorgang.name);

			notExist(item.getWiedervorlageNextFrist());
		})
	})

	describe('on vorgang with existing wiedervorlagen', () => {

		it('should show next frist in future and list', () => {
			const vorgang: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangWithWiedervorlageInFuture.name);
			notContainClass(vorgang.getWiedervorlageNextFrist(), 'red');

			mouseEnter(vorgang.getWiedervorlageNextFrist());

			exist(vorgang.getWiedervorlageNextFrist());

			vorgang.getWiedervorlageNextFrist().trigger('mouseleave'); // fix for next test
		})

		it('should show next frist in the past and list', () => {
			const vorgang: VorgangListItemE2EComponent = vorgangList.getListItem(vorgangWithWiedervorlageInThePast.name);
			containClass(vorgang.getWiedervorlageNextFrist(), 'red');

			mouseEnter(vorgang.getWiedervorlageNextFrist());

			exist(vorgang.getWiedervorlageNextFrist());
		})
	})
});
