import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, MongoCollections, notExist } from '../../support/cypress.util';

describe('VorgangList Suche for EA User', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();

	let vorgangToStay: VorgangE2E;
	let vorgangToDisappear: VorgangE2E;

	const searchBy: string = '20201118365670866102';

	describe('on user with role EINHEITLICHER_ANSPRECHPARTNER', () => {

		before(() => {
			cy.fixture(DataCollections.VORGANG_SEARCH).as('data').then(data => {
				vorgangToStay = data[0];
				vorgangToDisappear = data[1];
				cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
			});

			cy.fixture(DatabaseUser.TEST_EA).then(user => {
				cy.login(user.name, user.password);
			})

			exist(vorgangList.getRoot());
			vorgangList.waitForSpinnerToDisappear();
		})

		it('should filter list', () => {
			doSearch();

			exist(vorgangList.getListItem(vorgangToStay.name).getRoot());
			notExist(vorgangList.getListItem(vorgangToDisappear.name).getRoot());
		})

		it('should filter list and keep criteria on reload', () => {
			doSearch();

			cy.reload();

			vorgangList.waitForSpinnerToDisappear();

			exist(vorgangList.getRoot());
			exist(vorgangList.getListItem(vorgangToStay.name).getRoot());
			notExist(vorgangList.getListItem(vorgangToDisappear.name).getRoot());
		})

		it('should show unfiltered list after navigate to vorgang detail and back', () => {
			doSearch();

			exist(vorgangList.getRoot());
			vorgangList.getListItem(vorgangToStay.name).getRoot().click();
			exist(vorgangPage.getVorgangDetailHeader().getRoot());
			vorgangPage.getSubnavigation().clickBackButton();
			vorgangList.waitForSpinnerToDisappear();

			exist(vorgangList.getRoot());
			exist(vorgangList.getListItem(vorgangToStay.name).getRoot());
			exist(vorgangList.getListItem(vorgangToDisappear.name).getRoot());
		})

		function doSearch(): void {
			mainPage.getVorgangSearch().getInput().clear().type(searchBy)
			mainPage.getVorgangSearch().search();
		}
	})
});