import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, MongoCollections } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Detailansicht Wiedervorlage routing', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

	let vorgang: VorgangE2E;

	before(() => {
		cy.fixture(DataCollections.WIEDERVORLAGEN).as('data').then(data => {
			vorgang = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	it('Open Vorgang-Detail-Page', () => {
		vorgangList.getListItem(vorgang.name).getRoot().click();

		exist(vorgangPage.getVorgangDetailHeader().getRoot());
	})

	it('click on create wiedervorlage icon button should open page', () => {
		vorgangPage.getSubnavigation().clickWiedervorlageIconButton();

		exist(wiedervorlagePage.getSubnavigation().getRoot());
	})

	it('click on navigate back', () => {
		wiedervorlagePage.getSubnavigation().navigateBack();

		exist(vorgangPage.getVorgangDetailHeader().getRoot());
	})

	it('click on create wiedervorlage button should open page', () => {
		vorgangPage.getWiedervorlagenContainer().getCreateWiedervorlageButton().click();

		exist(wiedervorlagePage.getSubnavigation().getRoot());
	})

	it('click on navigate back', () => {
		wiedervorlagePage.getSubnavigation().navigateBack();

		exist(vorgangPage.getVorgangDetailHeader().getRoot());
	})
});
