import { formatDate, registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageInVorgangE2EComponent } from '../../components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { WiedervorlagenInVorgangE2EComponent } from '../../components/wiedervorlage/wiedervorlagen-in-vorgang.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { WiedervorlageE2E } from '../../model/wiedervorlage';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { containClass, CypressTasks, DatabaseUser, DataCollections, exist, MongoCollections, notContainClass, notExist } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Wiedervorlagen List', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervoralgeContainer: WiedervorlagenInVorgangE2EComponent = vorgangPage.getWiedervorlagenContainer();

	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

	let vorgang: VorgangE2E;

	let wiedervorlageFristToday;
	let wiedervorlageFristInThePast: WiedervorlageE2E;
	let wiedervorlageFristInTheFuture: WiedervorlageE2E;
	let wiedervorlageFristIsDone: WiedervorlageE2E;
	let wiedervorlageWithoutDetails: WiedervorlageE2E;

	before(() => {
		cy.fixture(DataCollections.VORGANG_WIEDERVORLAGEN).as('data').then(data => {
			vorgang = data[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
		vorgangList.waitForSpinnerToDisappear();
	})

	it('navigate to vorgang detail', () => {
		vorgangList.getListItem(vorgang.name).getRoot().click();

		exist(vorgangPage.getVorgangDetailHeader().getRoot());
	})

	describe('create valid Wiedervorlage with frist today', () => {

		it('click on create wiedervorlage should open page', () => {
			vorgangPage.getWiedervorlagenContainer().getCreateWiedervorlageButton().click();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('input valid data', () => {
			wiedervorlageFristToday = new WiedervorlageE2E();
			wiedervorlageFristToday.betreff = 'Test Betreff';
			wiedervorlageFristToday.beschreibung = 'Test Beschreibung';
			wiedervorlagePage.getWiedervorlageContainer().getBetreff().clear().type('Test Betreff');
			wiedervorlagePage.getWiedervorlageContainer().getBeschreibung().clear().type('Test Beschreibung');

			const today = formatDate(new Date(), 'dd.M.yyyy', 'de');
			wiedervorlagePage.getWiedervorlageContainer().getFrist().clear().type(today);
			wiedervorlageFristToday.frist = today;

			wiedervorlagePage.getWiedervorlageContainer().getSpeichernButton().click();
		})

		it('should open Vorgang-Detail-Page', () => {
			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})
	})

	describe('check status on wiedervorlage', () => {

		it('should show status yellow on frist today', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			containClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status red on frist in the past', () => {
			wiedervorlageFristInThePast = vorgang.wiedervorlages[0];

			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristInThePast.betreff);

			exist(wiedervorlage.getRoot());
			containClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status grey on frist in the future', () => {
			wiedervorlageFristInTheFuture = vorgang.wiedervorlages[1];

			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristInTheFuture.betreff);

			exist(wiedervorlage.getRoot());
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status white with check on frist is done', () => {
			wiedervorlageFristIsDone = vorgang.wiedervorlages[2];

			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristIsDone.betreff);

			exist(wiedervorlage.getRoot());
			containClass(wiedervorlage.getStatus().getStatusDot(), 'erledigt');

			exist(wiedervorlage.getExpandButton());
		})
	})

	describe('on wiedervorlage with detailinformation', () => {

		it('should show expand button', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			exist(wiedervorlage.getExpandButton());
		})

		it('should show detail information', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervoralgeContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			exist(wiedervorlage.getExpandButton());

			wiedervorlage.expandItem();

			exist(wiedervorlage.getExpandable());
		})
	})

	describe('on wiedervorlage without detailinformation', () => {

		it('should not show expand button for details', () => {
			wiedervorlageWithoutDetails = vorgang.wiedervorlages[3];

			const wiedervorlage: WiedervorlageInVorgangE2EComponent = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(wiedervorlageWithoutDetails.betreff);

			exist(wiedervorlage.getRoot());
			notExist(wiedervorlage.getExpandButton());
		})
	})
});