import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageAttachmentListE2EComponent } from '../../components/wiedervorlage/wiedervorlage-attachment-list.component';
import { WiedervorlageAttachmentsE2EComponent } from '../../components/wiedervorlage/wiedervorlage-attachments.e2e.component';
import { WiedervorlageE2EComponent } from '../../components/wiedervorlage/wiedervorlage-page.e2e.component';
import { VorgangE2E } from '../../model/vorgang';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, fileDownloaded, haveLength, MongoCollections } from '../../support/cypress.util';

describe('Wiedervorlage attachments download', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlageContainerInVorgang = vorgangPage.getWiedervorlagenContainer();

	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();
	const wiedervorlageContainer: WiedervorlageE2EComponent = wiedervorlagePage.getWiedervorlageContainer();
	const attachmentContainer: WiedervorlageAttachmentsE2EComponent = wiedervorlageContainer.getAttachmentContainer();
	const attachmentList: WiedervorlageAttachmentListE2EComponent = attachmentContainer.getAttachmentList();

	let vorgang: VorgangE2E;
	let wiedervorlage;
	let binaryFileWithContent;
	let binaryFileWithoutContent;

	before(() => {
		cy.fixture(DataCollections.BINARY_FILES).then(data => {
			binaryFileWithContent = data[0];
			binaryFileWithoutContent = data[1];
			cy.task(CypressTasks.INIT_BINARY_FILE_DATA, { collection: MongoCollections.BINARY_FILE, data })
		});

		cy.fixture(DataCollections.WIEDERVORLAGE_ATTACHMENT).then(data => {
			vorgang = data[0];
			wiedervorlage = vorgang.wiedervorlages[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('Download attachment from wiedervorlage page', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('Open Wiedervorlage-Page', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).getLink().click();

			haveLength(attachmentList.getAttachments(), 2);
		})

		it('click on file should start download', () => {
			attachmentList.getAttachment(binaryFileWithContent.file.name).download();

			fileDownloaded(binaryFileWithContent.file.name);
		})
	})

	describe('Download attachment from wiedervorlagelist in vorgang', () => {

		it('Open Vorgang-Detail-Page', () => {
			wiedervorlagePage.getSubnavigation().navigateBack();

			wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).expandItem();
			wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).getAttachmentList().getAttachment(binaryFileWithoutContent.file.name).download();

			fileDownloaded(binaryFileWithoutContent.file.name);
		})
	})
})