import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageAttachmentListE2EComponent } from '../../components/wiedervorlage/wiedervorlage-attachment-list.component';
import { WiedervorlageAttachmentsE2EComponent } from '../../components/wiedervorlage/wiedervorlage-attachments.e2e.component';
import { WiedervorlageE2EComponent } from '../../components/wiedervorlage/wiedervorlage-page.e2e.component';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveLength, MongoCollections, notExist } from '../../support/cypress.util';
import { TEST_FILE, TEST_FILE_WITH_CONTENT } from '../../support/data.util';
import { uploadEmptyFile, uploadFile } from '../../support/file-upload';

describe('Wiedervorlage attachments', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlageContainerInVorgang = vorgangPage.getWiedervorlagenContainer();

	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();
	const wiedervorlageContainer: WiedervorlageE2EComponent = wiedervorlagePage.getWiedervorlageContainer();
	const attachmentContainer: WiedervorlageAttachmentsE2EComponent = wiedervorlageContainer.getAttachmentContainer();
	const attachmentList: WiedervorlageAttachmentListE2EComponent = attachmentContainer.getAttachmentList();

	const WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF: string = 'WiedervorlageWithAttachments';

	let vorgangWithoutWiedervorlage;

	before(() => {
		cy.fixture(DataCollections.WIEDERVORLAGEN).then(data => {
			vorgangWithoutWiedervorlage = data[2];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(vorgangList.getRoot());
	})

	describe('Upload attachment', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangWithoutWiedervorlage.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('Open Wiedervorlage-Page', () => {
			wiedervorlageContainerInVorgang.getCreateWiedervorlageButton().click();

			exist(attachmentList.getRoot());
		})

		it('upload empty attachment', () => {
			uploadEmptyFile(attachmentContainer.getUploadInput(), TEST_FILE)

			exist(attachmentList.getAttachment(TEST_FILE).getRoot());
		})

		it('upload attachment with content', () => {
			uploadFile(attachmentContainer.getUploadInput(), TEST_FILE_WITH_CONTENT);

			exist(attachmentList.getAttachment(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('save wiedervorlage', () => {
			wiedervorlageContainer.getBetreff().clear().type(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF);

			wiedervorlageContainer.getSpeichernButton().click();
		})

		it('wiedervorlage in vorgang-detail should show attachments', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).expandItem();

			exist(wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getAttachmentList().getAttachment(TEST_FILE).getRoot());
		})
	})

	describe('Delete attachment', () => {

		it('open wiedervorlage page', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getLink().click();

			exist(attachmentList.getAttachment(TEST_FILE).getRoot());
			exist(attachmentList.getAttachment(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('delete', () => {
			attachmentList.getAttachment(TEST_FILE).delete();

			notExist(attachmentList.getAttachment(TEST_FILE).getRoot());
			exist(attachmentList.getAttachment(TEST_FILE_WITH_CONTENT).getRoot());

			wiedervorlageContainer.getSpeichernButton().click();
		})

		it('check attachments in wiedervorlage list', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getExpandButton();
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).expandItem();

			notExist(wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getAttachmentList().getAttachment(TEST_FILE).getRoot());
			exist(wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getAttachmentList().getAttachment(TEST_FILE_WITH_CONTENT).getRoot());
		})
	})

	describe('switch to wiedervorlage without attachments', () => {

		const WIEDERVORLAGE_WITHOUT_ATTACHMENT_BETREFF: string = 'WiedervorlageZumErledigen';

		it('should not show any attachments', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITHOUT_ATTACHMENT_BETREFF).getLink().click();

			haveLength(attachmentList.getAttachments(), 0);

			wiedervorlagePage.getSubnavigation().navigateBack();
		})
	})
})