import { formatDate, registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangListE2EComponent } from '../../components/vorgang/vorgang-list.e2e.component';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { CypressTasks, DatabaseUser, DataCollections, exist, haveText, haveValue, MongoCollections, notExist } from '../../support/cypress.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Wiedervorlage', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

	let vorgang;
	let validWiedervorlage;
	let invalidWiedervorlage;
	let editWiedervorlage;
	let editWiedervorlageNew;

	let vorganWithWiedervorlageToEdit;

	before(() => {
		cy.fixture(DataCollections.WIEDERVORLAGEN).then(vorgangData => {
			vorgang = vorgangData[0];
			vorganWithWiedervorlageToEdit = vorgangData[1];
			editWiedervorlage = vorganWithWiedervorlageToEdit.wiedervorlages[0];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data: vorgangData })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		cy.fixture('wiedervorlage.json').then(wiedervorlageData => {
			const date = new Date();
			const formatedDate = formatDate(date, 'dd.MM.yyyy', 'de');
			validWiedervorlage = wiedervorlageData['valid'];
			validWiedervorlage.frist = formatedDate;
			invalidWiedervorlage = wiedervorlageData['invalid'];

			const wiedervorlageEditFristDateFormatted = formatDate(new Date(), 'dd.M.yyyy', 'de');
			editWiedervorlage.frist = formatDate(editWiedervorlage.frist.$date, 'dd.M.yyyy', 'de');

			editWiedervorlageNew = wiedervorlageData['edit'].new;
			editWiedervorlageNew.frist = wiedervorlageEditFristDateFormatted;
		})

		exist(vorgangList.getRoot());
	})

	describe('Open Vorgang-Detail-Page', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})
	})

	describe('Create valid Wiedervorlage', () => {

		it('click on create wiedervorlage should open page', () => {
			vorgangPage.getWiedervorlagenContainer().getCreateWiedervorlageButton().click();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('input valid data', () => {
			fillInputFields(validWiedervorlage);

			wiedervorlagePage.getWiedervorlageContainer().getSpeichernButton().click();
		})

		it('should open Vorgang-Detail-Page', () => {
			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show snackbar', () => {
			haveText(mainPage.getSnackBar().getMessage(), `Die Wiedervorlage "${validWiedervorlage.betreff}" wurde angelegt`);
		})

		it('Wiedervorlage should exist', () => {
			let wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(validWiedervorlage.betreff);
			exist(wiedervorlage.getRoot());

			haveText(wiedervorlage.getFrist(), validWiedervorlage.fristAusgeschrieben);
			haveText(wiedervorlage.getBetreff(), validWiedervorlage.betreff);
		})
	})

	describe('Create invalid Wiedervorlage', () => {

		it('click on create wiedervorlage should open page', () => {
			vorgangPage.getSubnavigation().clickWiedervorlageIconButton();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('input invalid data', () => {
			fillInputFields(invalidWiedervorlage);

			wiedervorlagePage.getWiedervorlageContainer().getSpeichernButton().click();
		})

		it('should have validation error', () => {
			exist(wiedervorlagePage.getWiedervorlageContainer().getBetreffError());
			exist(wiedervorlagePage.getWiedervorlageContainer().getDatumError());
		})

		it('back to Vorgang-Detail-Page', () => {
			wiedervorlagePage.getSubnavigation().navigateBack();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('Wiedervorlage should not exist', () => {
			let wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(invalidWiedervorlage.betreff);

			notExist(wiedervorlage.getRoot());
		})

		it('navigate to VorganglistPage', () => {
			vorgangPage.getSubnavigation().clickBackButton();

			exist(vorgangList.getRoot());
		})
	})

	describe('edit Wiedervorlage', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorganWithWiedervorlageToEdit.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('click on wiedervorlage should open Wiedervorlage-Page', () => {
			let wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(editWiedervorlage.betreff);

			wiedervorlage.getLink().click();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('should have input values', () => {
			haveValue(wiedervorlagePage.getWiedervorlageContainer().getBeschreibung(), editWiedervorlage.beschreibung);
			haveValue(wiedervorlagePage.getWiedervorlageContainer().getBetreff(), editWiedervorlage.betreff);
			haveValue(wiedervorlagePage.getWiedervorlageContainer().getFrist(), editWiedervorlage.frist);
		})

		it('input new data', () => {
			fillInputFields(editWiedervorlageNew);

			wiedervorlagePage.getWiedervorlageContainer().getSpeichernButton().click();
		})

		it('should open Vorgang-Detail-Page', () => {
			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show snackbar', () => {
			haveText(mainPage.getSnackBar().getMessage(), `Die Wiedervorlage "${editWiedervorlageNew.betreff}" wurde gespeichert`);
		})

		it('Wiedervorlage should exist', () => {
			let wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(editWiedervorlageNew.betreff);

			exist(wiedervorlage.getRoot());

			haveText(wiedervorlage.getFrist(), editWiedervorlageNew.fristAusgeschrieben);
			haveText(wiedervorlage.getBetreff(), editWiedervorlageNew.betreff);
		})

		it('old wiedervorlage should not exit', () => {
			let wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(editWiedervorlage.betreff);

			notExist(wiedervorlage.getRoot());
		})
	})

	function fillInputFields(input) {
		wiedervorlagePage.getWiedervorlageContainer().getBetreff().clear().type(input.betreff);
		wiedervorlagePage.getWiedervorlageContainer().getBeschreibung().clear().type(input.beschreibung);
		input.frist ? wiedervorlagePage.getWiedervorlageContainer().getFrist().clear().type(input.frist) : '';
	}
});