import { WiedervorlageInVorgangE2EComponent } from '../../components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { MainPage } from '../../page-objects/main.po';
import { VorgangPage } from '../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../page-objects/wiedervorlage.po';
import { containClass, CypressTasks, DatabaseUser, DataCollections, exist, MongoCollections, notContainClass } from '../../support/cypress.util';

describe('Wiedervorlage erledigen/wiedereroeffnen', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

	let vorgang;
	let wiedervorlageZumErledigen;
	let wiedervorlageZumWiedereroeffnen;

	before(() => {
		cy.fixture(DataCollections.WIEDERVORLAGEN).then(vorgangData => {
			vorgang = vorgangData[2];
			wiedervorlageZumErledigen = vorgang.wiedervorlages[0];
			wiedervorlageZumWiedereroeffnen = vorgang.wiedervorlages[1];
			cy.task(CypressTasks.INIT_DATA, { collection: MongoCollections.VORGANG, data: vorgangData })
		});

		cy.fixture(DatabaseUser.TEST).then(user => {
			cy.login(user.name, user.password);
		})

		exist(mainPage.getVorgangList().getRoot());
	})

	describe('Open Vorgang-Detail-Page', () => {

		it('Open Vorgang-Detail-Page', () => {
			mainPage.getVorgangList().getListItem(vorgang.name).getRoot().click();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})
	})

	describe('erledigen', () => {

		let wiedervorlage: WiedervorlageInVorgangE2EComponent;

		before(() => {
			wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(wiedervorlageZumErledigen.betreff);
		})

		it('should open wiedervorlage on click', () => {
			wiedervorlage.getLink().click();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('Als erledigt markieren', () => {
			wiedervorlagePage.getSubnavigation().erledigen();
		})

		it('should have erledigt status', () => {
			containClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})

		it('should open vorgang detail', () => {
			wiedervorlagePage.getSubnavigation().navigateBack();
		})
	})

	describe('wiedereroeffnen', () => {

		let wiedervorlage: WiedervorlageInVorgangE2EComponent;

		before(() => {
			wiedervorlage = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(wiedervorlageZumWiedereroeffnen.betreff);
		})

		it('should open wiedervorlage on click', () => {
			wiedervorlage.getLink().click();

			exist(wiedervorlagePage.getSubnavigation().getRoot());
		})

		it('Als offen markieren', () => {
			wiedervorlagePage.getSubnavigation().wiedereroeffnen();
		})

		it('should have wiedereroeffnet status', () => {
			notContainClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})
	})

	describe('kinda tiny pentest for erledigen/wiedereroeffnen', () => {

		it('should mark as erledigt', () => {
			wiedervorlagePage.getSubnavigation().erledigen();

			containClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})

		it('should mark as open', () => {
			wiedervorlagePage.getSubnavigation().wiedereroeffnen();

			notContainClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})

		it('should mark as erledigt', () => {
			wiedervorlagePage.getSubnavigation().erledigen();

			containClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})

		it('should mark as open', () => {
			wiedervorlagePage.getSubnavigation().wiedereroeffnen();

			notContainClass(wiedervorlagePage.getWiedervorlageContainer().getStatusDot(), 'erledigt');
		})
	})
})