declare namespace Cypress {
	interface Chainable<Subject> {
		getTestElementWithOid(oid: string, ...args);
		getTestElement(selector: string, ...args);
		getTestElementWithClass(selector: string, ...args);
		findTestElementWithClass(selector: string, ...args);
		login(user, password);
		logout();
		getUserInfo();
	}
}

Cypress.Commands.add('getTestElement', (selector, ...args) => {
	return cy.get(`[data-test-id~="${selector}"]`, ...args)
})

Cypress.Commands.add('getTestElementWithClass', (selector, ...args) => {
	return cy.get(`[data-test-class="${selector}"]`, ...args)
})

Cypress.Commands.add('getTestElementWithOid', (oid, ...args) => {
	return cy.getTestElement(oid, ...args)
})

Cypress.Commands.add('findTestElementWithClass', { prevSubject: true }, (subject, selector) => {
	return subject.find(`[data-test-class="${selector}"]`)
})

Cypress.Commands.add('login', (user, password) => {
	cy.logout();

	cy.request({
		method: 'POST',
		followRedirect: false,
		url: `${Cypress.env("keycloakUrl")}auth/realms/${Cypress.env("keycloakRealm")}/protocol/openid-connect/token`,
		headers: {
			"Content-Type": "application/x-www-form-urlencoded"
		},
		body: {
			client_id: Cypress.env("keycloakClient"),
			username: user,
			password: password,
			grant_type: "password",
			redirect_uri: Cypress.config("baseUrl"),
			response_mode: 'fragment',
			response_type: 'code',
			scope: 'openid'
		}
	}).then(response => {
		const token = response.body.access_token;
		const authorization: any = `bearer ${token}`;

		cy.visit('', authorization);

		window.sessionStorage.setItem('access_token', token);
		window.sessionStorage.setItem('id_token', token);
	});
})

Cypress.Commands.add('getUserInfo', () => {
	return cy.request({
		method: 'GET',
		url: `${Cypress.env("keycloakUrl")}auth/realms/${Cypress.env("keycloakRealm")}/protocol/openid-connect/userinfo`,
		headers: {
			Authorization: `bearer ${window.sessionStorage.getItem('access_token')}`
		}
	});
})

Cypress.Commands.add('logout', () => {
	cy.request({
		method: 'GET',
		url: `${Cypress.env("keycloakUrl")}auth/realms/${Cypress.env("keycloakRealm")}/protocol/openid-connect/logout`,
		headers: {
			"Content-Type": "application/x-www-form-urlencoded"
		},
		body: {
			refresh_token: window.sessionStorage.getItem('id_token')
		},
		failOnStatusCode: false
	}).then(() => {
		window.sessionStorage.clear();

		cy.visit('');
	});
})
