import { HttpClient } from '@angular/common/http';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { ApiRootResource, ApiRootService } from '@goofy-client/api-root-shared';
import { AppService } from '@goofy-client/app-shared';
import { ENVIRONMENT_CONFIG } from '@goofy-client/environment-shared';
import { NavigationModule } from '@goofy-client/navigation';
import { createEmptyStateResource, createStateResource, NavigationService, StateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { UiModule } from '@goofy-client/ui';
import { OAuthService } from 'angular-oauth2-oidc';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { BehaviorSubject, of } from 'rxjs';
import { AppComponent } from './app.component';

describe('AppComponent', () => {
	let component: AppComponent;
	let fixture: ComponentFixture<AppComponent>;

	const apiRootSubj: BehaviorSubject<StateResource<ApiRootResource>> = new BehaviorSubject(null);
	const darkModeSubj: BehaviorSubject<boolean> = new BehaviorSubject(false);
	const apiRootService = { ...mock(ApiRootService), getApiRoot: () => apiRootSubj };

	const appService = { ...mock(AppService), getDarkMode: () => darkModeSubj };
	const navigationService = { ...mock(NavigationService), init: () => { }, urlChanged: () => of({}) };
	const authService = { ...mock(OAuthService), loadDiscoveryDocumentAndLogin: () => new Promise((resolve, reject) => resolve(null)) };
	const envConfig = { authServer: 'http://' };
	const httpClient = mock(HttpClient);

	const buildInfo: string = 'goofy-client-build-info';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				UiModule,
				NavigationModule,
				RouterTestingModule
			],
			declarations: [AppComponent],
			providers: [
				{
					provide: ApiRootService,
					useValue: apiRootService
				},
				{
					provide: AppService,
					useValue: appService
				},
				{
					provide: NavigationService,
					useValue: navigationService
				}, {
					provide: OAuthService,
					useValue: authService
				},
				{
					provide: ENVIRONMENT_CONFIG,
					useValue: envConfig
				},
				{
					provide: HttpClient,
					useValue: httpClient
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(AppComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create the app', () => {
		expect(component).toBeTruthy();
	});

	it(`should have as title 'goofy'`, () => {
		expect(component.title).toEqual('goofy');
	});

	describe('buildInfo', () => {

		it('should be rendered', () => {
			apiRootSubj.next(createStateResource(createApiRootResource()));

			fixture.detectChanges();

			const header = fixture.nativeElement.querySelector(buildInfo);
			expect(header).toBeInstanceOf(HTMLElement);
		})

		it('should NOT be rendered if api root not loaded', () => {
			apiRootSubj.next(createEmptyStateResource());

			fixture.detectChanges();

			const header = fixture.nativeElement.querySelector(buildInfo);
			expect(header).not.toBeInstanceOf(HTMLElement);
		})
	})
});
