import { Component, Inject, OnInit } from '@angular/core';
import { MAT_MOMENT_DATE_ADAPTER_OPTIONS, MAT_MOMENT_DATE_FORMATS, MomentDateAdapter } from '@angular/material-moment-adapter';
import { DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE } from '@angular/material/core';
import { ApiRootResource, ApiRootService } from '@goofy-client/api-root-shared';
import { ENVIRONMENT_CONFIG } from '@goofy-client/environment-shared';
import { StateResource } from '@goofy-client/tech-shared';
import { IconService } from '@goofy-client/ui';
import { AuthConfig, OAuthService } from 'angular-oauth2-oidc';
import { JwksValidationHandler } from 'angular-oauth2-oidc-jwks';
import { Environment } from 'libs/environment-shared/src/lib/environment.model';
import { Observable } from 'rxjs';

@Component({
	selector: 'goofy-client-root',
	templateUrl: './app.component.html',
	styleUrls: ['./app.component.scss'],
	providers: [
		{ provide: MAT_DATE_LOCALE, useValue: 'de-DE' },
		{ provide: DateAdapter, useClass: MomentDateAdapter, deps: [MAT_DATE_LOCALE] },
		{ provide: MAT_DATE_FORMATS, useValue: MAT_MOMENT_DATE_FORMATS },
		{ provide: MAT_MOMENT_DATE_ADAPTER_OPTIONS, useValue: { useUtc: true } }
	]
})
export class AppComponent implements OnInit {
	title = 'goofy';

	apiRoot$: Observable<StateResource<ApiRootResource>>;

	constructor(
		private apiRootService: ApiRootService,
		private iconService: IconService,
		private oAuthService: OAuthService,
		@Inject(ENVIRONMENT_CONFIG) private envConfig: Environment,
	) {
		this.iconService.registerIcons();
	}

	ngOnInit(): void {
		this.configureWithNewConfigApi();
	}

	private configureWithNewConfigApi() {
		this.oAuthService.configure(this.buildConfiguration());
		this.oAuthService.setupAutomaticSilentRefresh();
		this.oAuthService.tokenValidationHandler = new JwksValidationHandler();
		this.oAuthService.loadDiscoveryDocumentAndLogin().then(() => {
			this.apiRoot$ = this.apiRootService.getApiRoot();
		});
	}

	private buildConfiguration(): AuthConfig {
		return {
			// Url of the Identity Provider
			issuer: this.envConfig.authServer + '/realms/' + this.envConfig.realm,

			// URL of the SPA to redirect the user to after login
			redirectUri: window.location.origin + window.location.pathname,

			// URL of the SPA to redirect the user after silent refresh
			silentRefreshRedirectUri: window.location.origin + '/silent-refresh.html',
			useSilentRefresh: true,
			timeoutFactor: 0.5,

			// The SPA's id. The SPA is registerd with this id at the auth-server
			clientId: 'goofy',

			// set the scope for the permissions the client should request
			scope: 'openid profile email',
			requireHttps: false
		};
	}
}

