import { registerLocaleData } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import localeDe from '@angular/common/locales/de';
import { LOCALE_ID, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterModule, Routes } from '@angular/router';
import { ApiRootModule } from '@goofy-client/api-root-shared';
import { AppSharedModule } from '@goofy-client/app-shared';
import { EnvironmentModule } from '@goofy-client/environment-shared';
import { NavigationModule } from '@goofy-client/navigation';
import { UiModule } from '@goofy-client/ui';
import { OAuthModule } from 'angular-oauth2-oidc';
import { AppComponent } from './app.component';
import { MatTooltipDefaultOptions, MAT_TOOLTIP_DEFAULT_OPTIONS } from '@angular/material/tooltip';
import { MatDialogConfig } from '@angular/material/dialog/dialog-config';
import { MAT_DIALOG_DEFAULT_OPTIONS } from '@angular/material/dialog';

registerLocaleData(localeDe);

const routes: Routes = [
	{
		path: '',
		loadChildren: () => import('@goofy-client/vorgang').then(m => m.VorgangModule)
	}
];

const tooltipDefaults: MatTooltipDefaultOptions = {
	showDelay: 400,
	hideDelay: 0,
	touchendHideDelay: 0,
};

const dialogDefaults: MatDialogConfig = {
	position: { bottom: '0', right: '30px' },
	hasBackdrop: false,
	width: '480px',
	panelClass: 'fixed-dialog',
	disableClose: true
}

@NgModule({
	declarations: [
		AppComponent,
	],
	imports: [
		BrowserModule,
		HttpClientModule,
		BrowserAnimationsModule,
		RouterModule.forRoot(routes, { relativeLinkResolution: 'corrected' }),
		UiModule,
		EnvironmentModule,
		OAuthModule.forRoot({
			resourceServer: {
				sendAccessToken: true
			}
		}),
		ApiRootModule,
		NavigationModule,
		AppSharedModule
	],
	providers: [
		{ provide: LOCALE_ID, useValue: 'de' },
		{ provide: MAT_TOOLTIP_DEFAULT_OPTIONS, useValue: tooltipDefaults },
		{ provide: MAT_DIALOG_DEFAULT_OPTIONS, useValue: dialogDefaults }
	],
	bootstrap: [AppComponent]
})



export class AppModule { }
