import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { of } from 'rxjs';
import { createApiRoot } from '../../test/api-root';
import { ApiRoot } from './api-root.model';
import { ApiRootRepository } from './api-root.repository';
import { ApiRootService } from './api-root.service';

describe('ApiRootService', () => {

	var repository;
	var service;

	beforeEach(() => {
		repository = mock(ApiRootRepository);
		service = new ApiRootService(repository);
	})

	describe('calling getApiRoot', () => {

		beforeEach(() => {
			repository.loadApiRoot.mockReturnValue(of(null));
		})

		it('should call repository', () => {
			service.getApiRoot();

			expect(repository.loadApiRoot).toHaveBeenCalled();
		});

		it('should not call repository if already loading', () => {
			service.apiRoot$.next(createEmptyStateResource(true));

			service.getApiRoot();

			expect(repository.loadApiRoot).not.toHaveBeenCalled();
		});

		it('should set loading flag', () => {
			service.getApiRoot();

			expect(service.apiRoot$.value.loading).toBe(true);
		});

		describe('on successful load', () => {
			const apiRoot: ApiRoot = createApiRoot();

			beforeEach(() => {
				repository.loadApiRoot.mockReturnValue(of(apiRoot));
			})

			it('should set loaded', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.loaded).toBe(true);
			});

			it('should set resource', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.resource).toBe(apiRoot);
			});

			it('should unset loading', () => {
				service.getApiRoot();

				expect(service.apiRoot$.value.loading).toBe(false);
			});
		})
	})
})