import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { createEmptyStateResource, createStateResource, doIfLoadingRequired, StateResource } from '@goofy-client/tech-shared';
import { ApiRootRepository } from './api-root.repository';
import { ApiRootResource } from './api-root.model';

@Injectable()
export class ApiRootService {

	private readonly apiRoot$: BehaviorSubject<StateResource<ApiRootResource>> = new BehaviorSubject<StateResource<ApiRootResource>>(createEmptyStateResource());

	constructor(private repository: ApiRootRepository) { }

	public getApiRoot(): Observable<StateResource<ApiRootResource>> {
		doIfLoadingRequired(this.apiRoot$.value, () => this.loadApiRoot())
		return this.apiRoot$;
	}

	private loadApiRoot(): void {
		this.apiRoot$.next({ ...this.apiRoot$.value, loading: true });

		this.repository.loadApiRoot().subscribe(apiRoot => {
			if (apiRoot !== null) {
				this.apiRoot$.next(createStateResource(apiRoot));
			}
		})
	}
}
