import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { getDarkModeFromLocalStorage, getNavigationCollapsedFromLocalStorage, setDarkModeIntoStorage, setNavigationCollapsedIntoLocalStorage } from '../storage/storage';

@Injectable()
export class AppService {

	private readonly darkMode: BehaviorSubject<boolean> = new BehaviorSubject(Boolean(JSON.parse(getDarkModeFromLocalStorage())));
	private readonly navigationCollapse: BehaviorSubject<boolean> = new BehaviorSubject(Boolean(JSON.parse(getNavigationCollapsedFromLocalStorage())));

	public setDarkMode(darkMode: boolean): void {
		setDarkModeIntoStorage(String(darkMode));

		this.darkMode.next(darkMode);
	}

	public setNavigationCollapse(navigationCollapse: boolean): void {
		setNavigationCollapsedIntoLocalStorage(String(navigationCollapse));

		this.navigationCollapse.next(navigationCollapse);
	}

	public getDarkMode(): Observable<boolean> {
		return this.darkMode;
	}

	public getNavigationCollapse(): Observable<boolean> {
		return this.navigationCollapse;
	}
}