import { HttpClient } from '@angular/common/http';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { getUrl, Resource } from '@ngxp/rest';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { of } from 'rxjs';
import { BinaryFileRepository } from './binary-file.repository';

describe('BinaryFileRepository', () => {
	let repository: BinaryFileRepository;

	const httpClient = mock(HttpClient);

	beforeEach(() => {
		repository = new BinaryFileRepository(useFromMock(httpClient));
	})

	it('should be created', () => {
		expect(repository).toBeTruthy();
	})

	describe('uploadFile', () => {

		const linkRel: string = DummyLinkRel.UPLOAD_FILE;
		const dummyResource: Resource = createDummyResource([linkRel]);
		const blob: Blob = new Blob(['test text'], { type: 'text/plain' });

		beforeEach(() => {
			httpClient.post.mockReturnValue(of({}));
		})

		it('should call POST on httpClient with uri and data', () => {
			const formData: FormData = new FormData();
			formData.append('file', blob, 'filename');

			repository.uploadFile(dummyResource, linkRel, blob);

			expect(httpClient.post).toHaveBeenCalledWith(getUrl(dummyResource, linkRel), formData, { observe: 'response' });
		})

		it('and return result', () => {
			let result = repository.uploadFile(dummyResource, linkRel, blob);

			expect(result).not.toBeNull();
		})
	})
})