import { Injectable } from '@angular/core';
import { Resource, ResourceFactory } from '@ngxp/rest';
import { Observable } from 'rxjs';
import { CommandLinkRel } from './command.linkrel';
import { CommandListResource, CommandResource, CommandStatus, CreateCommand } from './command.model';

@Injectable({ providedIn: 'root' })
export class CommandRepository {

	constructor(private resourceFactory: ResourceFactory) { }

	public createCommand(resource: Resource, linkrel: string, command: CreateCommand): Observable<CommandResource> {
		return this.resourceFactory.from(resource).post(linkrel, command);
	}

	public getCommand(resource: CommandResource): Observable<CommandResource> {
		return this.resourceFactory.from(resource).get(CommandLinkRel.SELF);
	}

	public getPendingCommands(resource: Resource, linkRel: string): Observable<CommandListResource> {
		return this.resourceFactory.from(resource).get(linkRel)
	}

	public revokeCommand(resource: CommandResource): Observable<CommandResource> {
		return this.resourceFactory.from(resource).patch(CommandLinkRel.REVOKE, { status: CommandStatus.REVOKED });
	}

	public getEffectedResource(command: CommandResource): Observable<any> {
		return this.resourceFactory.from(command).get(CommandLinkRel.EFFECTED_RESOURCE);
	}
}