import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createForwardingListResource } from 'libs/forwarding-shared/test/forwarding';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { ForwardingListResource } from './forwarding.model';
import { ForwardingRepository } from './forwarding.repository';

describe('ForwardingRepository', () => {
	let repository: ForwardingRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		repository = new ForwardingRepository(useFromMock(resourceFactory));

		resourceFactory.fromId.mockReturnValue(resourceWrapper);
		resourceFactory.from.mockReturnValue(resourceWrapper);
	})

	describe('getForwardings', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARDING]);
		const commandList: ForwardingListResource = createForwardingListResource();

		beforeEach(() => {
			resourceWrapper.get.mockReturnValue(hot('a', { a: commandList }));
		})

		it('should call resourceFactory', () => {
			repository.getForwardings(vorgang);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
		})

		it('should call resourceWrapper', () => {
			repository.getForwardings(vorgang);

			expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangWithEingangLinkRel.FORWARDING);
		})

		it('should return result', () => {
			let result = repository.getForwardings(vorgang);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(cold('a', { a: commandList }));
		})
	})
})