import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIcon } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, TextEditorComponent } from '@goofy-client/ui';
import { VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardFormComponent } from './vorgang-forward-form.component';
import { VorgangForwardFormService } from './vorgang-forward.formservice';

describe('VorgangForwardFormComponent', () => {
	let component: VorgangForwardFormComponent;
	let fixture: ComponentFixture<VorgangForwardFormComponent>;

	const form: FormGroup = new FormGroup({
		[VorgangForwardFormService.FIELD_PASSWORD]: new FormControl(null),
		[VorgangForwardFormService.FIELD_EMAIL]: new FormControl(null)
	});

	const formService = { ...mock(VorgangForwardFormService), form };

	const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangForwardFormComponent,
				MatIcon,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(TextEditorComponent)
			],
			imports: [
				MatFormFieldModule,
				MatInputModule,
				ReactiveFormsModule,
				BrowserAnimationsModule
			],
			providers: [
				{
					provide: VorgangForwardFormService,
					useValue: formService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangForwardFormComponent);
		component = fixture.componentInstance;
		component.vorgang = vorgangWithEingang;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('zustaendige stelle email exists', () => {

		it('should return false if vorgang is undefined', () => {
			component.vorgang = undefined;

			var result: boolean = component.zustaendigeStelleEmailExists();

			expect(result).toBeFalsy();
		})

		it('should return true if exists', () => {
			component.vorgang = vorgangWithEingang;

			var result: boolean = component.zustaendigeStelleEmailExists();

			expect(result).toBeTruthy();
		})

		it('should return false if NOT exists', () => {
			component.vorgang = { ...vorgangWithEingang, eingang: { ...vorgangWithEingang.eingang, zustaendigeStelle: { email: null } } };

			var result: boolean = component.zustaendigeStelleEmailExists();

			expect(result).toBeFalsy();
		})
	})

	describe('patch email field', () => {

		it('should set email', () => {
			component.vorgang = vorgangWithEingang;

			component.patchEmail();

			expect(formService.patchField).toHaveBeenCalledWith('email', vorgangWithEingang.eingang.zustaendigeStelle.email);
		})
	})
});
