import { Component, EventEmitter, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { CommandResource } from '@goofy-client/command-shared';
import { StateResource } from '@goofy-client/tech-shared';
import { VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { isNil } from 'lodash-es';
import { VorgangForwardFormService } from './vorgang-forward.formservice';

@Component({
	selector: 'goofy-client-vorgang-forward-form',
	templateUrl: './vorgang-forward-form.component.html',
	styleUrls: ['./vorgang-forward-form.component.scss']
})
export class VorgangForwardFormComponent implements OnChanges {

	@Input() forwardStateResource: StateResource<CommandResource>;
	@Input() vorgang: VorgangWithEingangResource;

	@Output('doSubmit') submit: EventEmitter<void> = new EventEmitter();

	readonly formServiceClass = VorgangForwardFormService;

	constructor(private formService: VorgangForwardFormService) { }

	ngOnChanges(changes: SimpleChanges) {
		if (changes.vorgang && this.zustaendigeStelleEmailExists()) {
			this.patchEmail();
		}
	}

	zustaendigeStelleEmailExists(): boolean {
		return this.vorgang && this.vorgang.eingang && this.vorgang.eingang.zustaendigeStelle && !isNil(this.vorgang.eingang.zustaendigeStelle.email);
	}

	patchEmail(): void {
		this.formService.patchField('email', this.vorgang.eingang.zustaendigeStelle.email)
	}

	get form(): FormGroup {
		return this.formService.form;
	}
}
