import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { ForwardingService } from '@goofy-client/forwarding-shared';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { VorgangForwardFormService } from './vorgang-forward.formservice';

describe('VorgangForwardFormService', () => {
	let formService: VorgangForwardFormService;
	let service;

	const formBuilder: FormBuilder = new FormBuilder();

	beforeEach(() => {
		service = mock(ForwardingService);

		formService = new VorgangForwardFormService(formBuilder, useFromMock(service));
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	describe('submit', () => {

		const returnValue = {};

		beforeEach(() => {
			service.forward.mockReturnValue(of(returnValue));
			service.doSubmit = jest.fn();
		})

		it('should call service', () => {
			formService.vorgang = createVorgangResource();

			formService.submit();

			expect(service.forward).toHaveBeenCalledWith(formService.vorgang, formService.form.value);
		})
	})

	describe('patch field', () => {

		const controlName: string = 'testField';
		const value: string = 'testField';

		const form: FormGroup = <any>{
			controls: {
				[controlName]: new FormControl()
			}
		};

		it('should set control value', () => {
			formService.form = form;

			formService.patchField(controlName, value);

			expect(formService.form.controls[controlName].value).toEqual(value);
		})
	})
})