import { Injectable } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { CommandResource } from '@goofy-client/command-shared';
import { ForwardingService } from '@goofy-client/forwarding-shared';
import { AbstractFormService, StateResource } from '@goofy-client/tech-shared';
import { VorgangResource } from '@goofy-client/vorgang-shared';
import { Observable } from 'rxjs';

@Injectable()
export class VorgangForwardFormService extends AbstractFormService {

	static readonly FIELD_PATH_PREFIX = 'command.redirectRequest';

	static readonly FIELD_EMAIL = 'email';
	static readonly FIELD_PASSWORD = 'password';

	vorgang: VorgangResource;

	constructor(formBuilder: FormBuilder, private forwardingService: ForwardingService) {
		super(formBuilder);
	}

	protected initForm(): FormGroup {
		return this.formBuilder.group({
			[VorgangForwardFormService.FIELD_EMAIL]: new FormControl(),
			[VorgangForwardFormService.FIELD_PASSWORD]: new FormControl()
		})
	}

	protected getPathPrefix(): string {
		return VorgangForwardFormService.FIELD_PATH_PREFIX;
	}

	public setVorgang(vorgang: VorgangResource): void {
		this.vorgang = vorgang;
	}

	protected doSubmit(): Observable<StateResource<CommandResource>> {
		return this.forwardingService.forward(this.vorgang, this.getFormValue());
	}

	patchField(fieldName: string, value: string) {
		this.form.controls[fieldName].patchValue(value);
	}
}
