import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder } from '@angular/forms';
import { ForwardingService } from '@goofy-client/forwarding-shared';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ExpansionPanelComponent, SpinnerComponent } from '@goofy-client/ui';
import { VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangForwardRequest, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangForwardFormService } from './vorgang-forward-formular/vorgang-forward-form/vorgang-forward.formservice';
import { VorgangForwardFormularComponent } from './vorgang-forward-formular/vorgang-forward-formular.component';
import { VorgangForwardingContainerComponent } from './vorgang-forwarding-container.component';
import { VorgangForwardingInfoListComponent } from './vorgang-forwarding-info-list/vorgang-forwarding-info-list.component';

describe('VorgangForwardingContainerComponent', () => {
	let component: VorgangForwardingContainerComponent;
	let fixture: ComponentFixture<VorgangForwardingContainerComponent>;

	const formService = mock(VorgangForwardFormService);
	const service = mock(ForwardingService);

	const forwarding: string = '[data-test-id="forwarding"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangForwardingContainerComponent,
				HasLinkPipe,
				MockComponent(SpinnerComponent),
				MockComponent(VorgangForwardingInfoListComponent),
				MockComponent(VorgangForwardFormularComponent),
				MockComponent(ExpansionPanelComponent)
			],
			providers: [
				FormBuilder,
				{
					provide: VorgangForwardFormService,
					useValue: formService
				},
				{
					provide: ForwardingService,
					useValue: service
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangForwardingContainerComponent);
		component = fixture.componentInstance;
		component.pendingForwardStateResource$ = of(createStateResource({ ...createCommandResource(), redirectRequest: createVorgangForwardRequest() }));
		component.vorgang = createVorgangWithEingangResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnChanges', () => {

		it('should call update vorgang', () => {
			component.updateVorgang = jest.fn();

			component.ngOnChanges();

			expect(component.updateVorgang).toHaveBeenCalled();
		})

		it('should call get forwardings', () => {
			component.getForwardings = jest.fn();

			component.ngOnChanges();

			expect(component.getForwardings).toHaveBeenCalled();
		})

		it('should call get pending forward command', () => {
			component.getPendingForwardCommand = jest.fn();

			component.ngOnChanges();

			expect(component.getPendingForwardCommand).toHaveBeenCalled();
		})
	})

	describe('getForwardings', () => {

		it('should do nothing if link not exists', () => {
			component.vorgang = createVorgangWithEingangResource();
			fixture.detectChanges();

			component.getForwardings();

			expect(service.getForwardings).not.toHaveBeenCalled();
		})

		it('should call service get forward commands', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARDING]);
			fixture.detectChanges();

			component.getForwardings();

			expect(service.getForwardings).toHaveBeenCalled();
		})
	})

	describe('getPendingForwardCommand', () => {

		it('should call service get pending forward command', () => {
			component.vorgang = createVorgangWithEingangResource();
			fixture.detectChanges();

			component.getPendingForwardCommand();

			expect(service.getPendingForwardCommand).toHaveBeenCalled();
		})
	})

	describe('forwarding', () => {

		describe('on forwarding link', () => {

			it('should show if exists', () => {
				component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARDING]);
				fixture.detectChanges();

				const element = fixture.nativeElement.querySelector(forwarding);

				expect(element).toBeInstanceOf(HTMLElement);
			})

			it('should hide is not exists', () => {
				component.vorgang = createVorgangWithEingangResource();
				fixture.detectChanges();

				const element = fixture.nativeElement.querySelector(forwarding);

				expect(element).not.toBeInstanceOf(HTMLElement);
			})
		})

		describe('on forward link', () => {

			it('should show if exists', () => {
				component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARD]);
				fixture.detectChanges();

				const element = fixture.nativeElement.querySelector(forwarding);

				expect(element).toBeInstanceOf(HTMLElement);
			})

			it('should hide is not exists', () => {
				component.vorgang = createVorgangWithEingangResource();
				fixture.detectChanges();

				const element = fixture.nativeElement.querySelector(forwarding);

				expect(element).not.toBeInstanceOf(HTMLElement);
			})
		})
	})
})
