import { Component, Input, OnChanges } from '@angular/core';
import { CommandResource } from '@goofy-client/command-shared';
import { ForwardingListResource, ForwardingService } from '@goofy-client/forwarding-shared';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { hasLink } from '@ngxp/rest';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { Observable, of } from 'rxjs';
import { VorgangForwardFormService } from './vorgang-forward-formular/vorgang-forward-form/vorgang-forward.formservice';

@Component({
	selector: 'goofy-client-vorgang-forwarding-container',
	templateUrl: './vorgang-forwarding-container.component.html',
	styleUrls: ['./vorgang-forwarding-container.component.scss'],
	providers: [VorgangForwardFormService]
})
export class VorgangForwardingContainerComponent implements OnChanges {

	@Input() public vorgang: VorgangWithEingangResource;

	forwardingStateResources$: Observable<StateResource<ForwardingListResource>> = of(createEmptyStateResource());
	pendingForwardStateResource$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource());

	readonly linkRel = CommandLinkRel;
	readonly vorgangLinkRel = VorgangWithEingangLinkRel;

	constructor(public formService: VorgangForwardFormService, private forwardingService: ForwardingService) { }

	ngOnChanges(): void {
		this.updateVorgang();

		this.getForwardings();
		this.getPendingForwardCommand();
	}

	updateVorgang(): void {
		this.formService.setVorgang(this.vorgang);
	}

	getForwardings(): void {
		if (hasLink(this.vorgang, VorgangWithEingangLinkRel.FORWARDING)) {
			this.forwardingStateResources$ = this.forwardingService.getForwardings(this.vorgang);
		}
	}

	getPendingForwardCommand(): void {
		this.pendingForwardStateResource$ = this.forwardingService.getPendingForwardCommand();
	}

	submit(): void {
		this.pendingForwardStateResource$ = this.formService.submit();
	}
}