import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { ForwardingLinkRel } from '@goofy-client/forwarding-shared';
import { FormatDateWithTimePipe, HasLinkPipe } from '@goofy-client/tech-shared';
import { createForwardingResource } from 'libs/forwarding-shared/test/forwarding';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardZustellenContainerComponent } from '../../../vorgang-forwarding-zustellen-container/vorgang-forward-zustellen-container.component';
import { VorgangForwardingInfoComponent } from './vorgang-forwarding-info.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangForwardingInfoComponent', () => {
	let component: VorgangForwardingInfoComponent;
	let fixture: ComponentFixture<VorgangForwardingInfoComponent>;

	const errorMessage: string = '[data-test-id="forwarding-error-message"]';
	const successMessage: string = '[data-test-id="forwarding-success-message"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangForwardingInfoComponent,
				FormatDateWithTimePipe,
				MatIcon,
				HasLinkPipe,
				MockComponent(VorgangForwardZustellenContainerComponent)
			],
			providers: [
				{ provide: LOCALE_ID, useValue: 'de' },
				{ provide: MAT_DATE_LOCALE, useValue: 'de-DE' }
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangForwardingInfoComponent);
		component = fixture.componentInstance;
		component.forwarding = createForwardingResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('on forwarding error', () => {

		it('should show error message', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.ERROR]);
			fixture.detectChanges();

			const message = fixture.nativeElement.querySelector(errorMessage);

			expect(message).toBeInstanceOf(HTMLElement);
		})

		it('should hide successfull message', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.ERROR]);
			fixture.detectChanges();

			const message = fixture.nativeElement.querySelector(successMessage);

			expect(message).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('on success forwarding', () => {

		it('should hide error message', () => {
			component.forwarding = createForwardingResource();
			fixture.detectChanges();

			const message = fixture.nativeElement.querySelector(errorMessage);

			expect(message).not.toBeInstanceOf(HTMLElement);
		})

		it('should show success message', () => {
			component.forwarding = createForwardingResource();
			fixture.detectChanges();

			const message = fixture.nativeElement.querySelector(successMessage);

			expect(message).toBeInstanceOf(HTMLElement);
		})
	})
});
