import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ForwardingLinkRel, ForwardingResource, ForwardingService } from '@goofy-client/forwarding-shared';
import { HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent } from '@goofy-client/ui';
import { createForwardingResource } from 'libs/forwarding-shared/test/forwarding';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardZustellenContainerComponent } from './vorgang-forward-zustellen-container.component';

describe('VorgangForwardZustellenContainerComponent', () => {
	let component: VorgangForwardZustellenContainerComponent;
	let fixture: ComponentFixture<VorgangForwardZustellenContainerComponent>;

	const forwarding: ForwardingResource = createForwardingResource();

	const markAsSuccess: string = '[data-test-id="mark-as-success-button"]';
	const markAsFail: string = '[data-test-id="mark-as-fail-button"]';

	const failedMessage: string = '[data-test-id="failed-message"]';
	const successfullMessage: string = '[data-test-id="successfull-message"]';

	const forwardingService = mock(ForwardingService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				HasLinkPipe,
				VorgangForwardZustellenContainerComponent,
				MatIcon,
				MockComponent(ButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: ForwardingService,
					useValue: forwardingService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangForwardZustellenContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should call service mark as success', () => {
		component.forwarding = forwarding;
		fixture.detectChanges();

		component.markAsSuccess();

		expect(forwardingService.markAsSuccess).toHaveBeenCalledWith(forwarding);
	})

	it('should call service mark as fail', () => {
		component.forwarding = forwarding;
		fixture.detectChanges();

		component.markAsFail();

		expect(forwardingService.markAsFail).toHaveBeenCalledWith(forwarding);
	})

	describe('mark as success button', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.MARK_AS_SUCCESS]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsSuccess);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsSuccess);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('mark as fail button', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.MARK_AS_FAIL]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsFail);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsFail);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('failed message', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.FAILED]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(failedMessage);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(failedMessage);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('successfull message', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.SUCCESSFUL]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(successfullMessage);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(successfullMessage);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})
});
