import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { createKommentarListResource } from '../../test/kommentar';
import { KommentarListLinkRel } from './kommentar.linkrel';
import { KommentarListResource } from './kommentar.model';
import { KommentarRepository } from './kommentar.repository';

describe('KommentarRepository', () => {
	let repository: KommentarRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		repository = new KommentarRepository(useFromMock(resourceFactory));
	})

	describe('findKommentare', () => {
		const kommentarListResource: KommentarListResource = createKommentarListResource();
		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([KommentarListLinkRel.KOMMENTAR_LIST]);

		beforeEach(() => {
			resourceFactory.from.mockReturnValue(resourceWrapper);
			resourceWrapper.get.mockReturnValue(cold('a', { a: kommentarListResource }));
		})

		it('should call resourceFactory with resource', () => {
			repository.findKommentare(vorgang);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
		})

		it('should call resourceWrapper with link', () => {
			repository.findKommentare(vorgang);

			expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangHeaderLinkRel.KOMMENTARE);
		})

		it('should return result', () => {
			let result = repository.findKommentare(vorgang);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(hot('a', { a: kommentarListResource }));
		})
	})
})