import { Component, EventEmitter, Input, OnChanges, Output } from '@angular/core';
import { CommandResource } from '@goofy-client/command-shared';
import { KommentarResource } from '@goofy-client/kommentar-shared';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { isNil } from 'lodash-es';
import { Observable, of } from 'rxjs';
import { KommentarFormService } from './kommentar.formservice';

@Component({
	selector: 'goofy-client-kommentar-form',
	templateUrl: './kommentar-form.component.html',
	styleUrls: ['./kommentar-form.component.scss'],
	providers: [KommentarFormService]
})
export class KommentarFormComponent implements OnChanges {

	@Input() kommentar: KommentarResource;

	@Output() cancel: EventEmitter<void> = new EventEmitter();

	submitInProgress$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource());

	readonly formServiceClass = KommentarFormService;

	constructor(public formService: KommentarFormService) { }

	ngOnChanges(): void {
		if (!isNil(this.kommentar)) {
			this.patch();
		}
	}

	patch(): void {
		this.formService.patch(this.kommentar);
	}

	submit(): void {
		this.submitInProgress$ = this.formService.submit();
	}
}