import { DOCUMENT } from '@angular/common';
import { Component, Inject, Renderer2 } from '@angular/core';
import { Observable } from 'rxjs';
import { AppService, localStorageDark } from '@goofy-client/app-shared';
import { OAuthService } from 'angular-oauth2-oidc';

@Component({
  selector: 'goofy-client-header-container',
  templateUrl: './header-container.component.html',
  styleUrls: ['./header-container.component.scss']
})
export class HeaderContainerComponent {

	navigationCollapse$: Observable<boolean>;
	darkMode$: Observable<boolean>;

	constructor(
		private appService: AppService,
		private renderer: Renderer2,
		private authService: OAuthService,
		@Inject(DOCUMENT) private document: Document,
	) {
		this.navigationCollapse$ = this.appService.getNavigationCollapse();
		this.darkMode$ = this.appService.getDarkMode();

		this.subscribeToDarkMode();
	}

 	private subscribeToDarkMode(): void {
		this.darkMode$.subscribe(darkMode => darkMode ? this.addClass(localStorageDark) : this.removeClass(localStorageDark));
	}

	private addClass(styleClass: string): void {
		this.renderer.addClass(this.document.body, styleClass);
	}

	private removeClass(styleClass: string): void {
		this.renderer.removeClass(this.document.body, styleClass);
	}

	toggleNavigation(isToggle: boolean): void {
		this.appService.setNavigationCollapse(isToggle);
	}

	changeColorMode(darkMode: boolean): void {
		this.appService.setDarkMode(darkMode);
	}

	logout(): void {
		this.authService.logOut();
	}
}
