import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { getUrl, Resource, ResourceFactory, ResourceUri } from '@ngxp/rest';
import { Observable } from 'rxjs';
import { OzgFileLinkRel } from './ozg-file.linkrel';
import { OzgFileListResource, OzgFileResource } from './ozg-file.model';

@Injectable({ providedIn: 'root' })
export class OzgFileRepository {
	constructor(private httpClient: HttpClient, private resourceFactory: ResourceFactory) { }

	public download(fileResource: OzgFileResource): Observable<Blob> {
		return this.httpClient.get<Blob>(getUrl(fileResource, OzgFileLinkRel.DOWNLOAD), this.buildRequestOptions());
	}

	buildRequestOptions(): GetRequestOptions {
		let headers = new HttpHeaders();
		headers = headers.set('Accept', ['application/*', 'images/*']);
		return { headers, responseType: 'blob' as 'json' };
	}

	public getFiles(resource: Resource, linkRel: string): Observable<OzgFileListResource> {
		return this.resourceFactory.from(resource).get(linkRel);
	}

	public getFile(fileUri: ResourceUri): Observable<OzgFileResource> {
		return this.resourceFactory.fromId(fileUri).get();
	}
}

export interface GetRequestOptions {
	headers?: HttpHeaders | {
		[header: string]: string | string[];
	};
	responseType?: 'json';
}