import { createEmptyStateResource, createStateResource, ListResource, StateResource } from '@goofy-client/tech-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { Resource } from '@ngxp/rest';
import * as faker from 'faker';
import { cold, hot } from 'jest-marbles';
import { createOzgFileResource } from 'libs/ozg-file-shared/test/ozg-file';
import { createDummyListResource, createDummyResource } from 'libs/tech-shared/test/resource';
import { OzgFileService } from '../lib/ozg-file.service';
import { OzgFileResource } from './ozg-file.model';
import { OzgFileRepository } from './ozg-file.repository';

describe('OzgFileService', () => {
	let service: OzgFileService;
	let repository: Mock<OzgFileRepository>;

	const ozgFileResource: OzgFileResource = createOzgFileResource();

	beforeEach(() => {
		repository = mock(OzgFileRepository);

		service = new OzgFileService(useFromMock(repository));
	})

	describe('download file', () => {

		const blob = {};

		beforeEach(() => {
			service.save = jest.fn();
			repository.download.mockReturnValue(hot('a', { a: blob }));
		})

		it('should call repository', () => {
			service.downloadFile(ozgFileResource)

			expect(repository.download).toHaveBeenCalledWith(ozgFileResource);
		})

		describe('save file', () => {

			describe('on existing reponse', () => {

				it('should save file', () => {
					service.saveOzgFile(blob, ozgFileResource);

					expect(service.save).toHaveBeenCalled();
				})

				it('should return loaded stateResource', () => {
					const result: StateResource<Blob> = service.saveOzgFile(blob, ozgFileResource);

					expect(result).toEqual(createStateResource(blob));
				})
			})

			describe('on non existing reponse', () => {

				it('should return loading stateresource', () => {
					const result: StateResource<Blob> = service.saveOzgFile(undefined, ozgFileResource);

					expect(result).toEqual(createEmptyStateResource(true));
				})
			})
		})
	})

	describe('get files', () => {

		const listResource: ListResource = createDummyListResource();
		const resource: Resource = createDummyResource();
		const linkRel: string = faker.internet.url();

		beforeEach(() => {
			repository.getFiles.mockReturnValue(hot('-a', { a: listResource }));
		})

		it('should call repository', () => {
			service.getFileList(resource, linkRel);

			expect(repository.getFiles).toHaveBeenCalledWith(resource, linkRel);
		})

		it('should return value', () => {
			const fileList = service.getFileList(resource, linkRel);

			expect(fileList).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(listResource) }));
		})
	})
})