import { Injectable } from '@angular/core';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { Resource, ResourceUri } from '@ngxp/rest';
import { saveAs } from 'file-saver';
import { isNil } from 'lodash-es';
import { Observable } from 'rxjs';
import { map, startWith } from 'rxjs/operators';
import { OzgFileListResource, OzgFileResource } from './ozg-file.model';
import { OzgFileRepository } from './ozg-file.repository';

@Injectable({ providedIn: 'root' })
export class OzgFileService {

	constructor(private repository: OzgFileRepository) { }

	public downloadFile(file: OzgFileResource): Observable<StateResource<any>> {
		return this.repository.download(file).pipe(
			map(data => this.saveOzgFile(data, file)),
			startWith(createEmptyStateResource(true)));
	}

	saveOzgFile(data: any, file: OzgFileResource): StateResource<Blob> {
		if (!isNil(data)) {
			this.save(data, file.name);
			return createStateResource(data);
		}
		return createEmptyStateResource(true);
	}

	save(data: any, fileName: string): void {
		saveAs(data, fileName)
	}

	public getFileList(resource: Resource, linkRel: string): Observable<StateResource<OzgFileListResource>> {
		return this.repository.getFiles(resource, linkRel).pipe(
			map(fileList => createStateResource(fileList)),
			startWith(createEmptyStateResource<OzgFileListResource>(true)));
	}

	public getFile(uri: ResourceUri): Observable<StateResource<OzgFileResource>> {
		return this.repository.getFile(uri).pipe(
			map(fileList => createStateResource(fileList)),
			startWith(createEmptyStateResource<OzgFileResource>(true)));
	}
}