import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createEmptyStateResource, HasLinkPipe, ToEmbeddedResourcesPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ExpansionPanelComponent } from '@goofy-client/ui';
import { VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { AnhaengeComponent } from './anhaenge/anhaenge.component';
import { AnhangListContainerComponent } from './anhang-list-container.component';

describe('AnhangListContainerComponent', () => {
	let component: AnhangListContainerComponent;
	let fixture: ComponentFixture<AnhangListContainerComponent>;

	const vorgangWithAttachments: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ATTACHMENTS]);
	const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

	const vorgangService = mock(VorgangService);

	const emptyAnhaenge: string = '[data-test-id="empty-anhaenge"]';
	const anhaenge: string = '[data-test-id="anhaenge"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				AnhangListContainerComponent,
				HasLinkPipe,
				ToEmbeddedResourcesPipe,
				MockComponent(AnhaengeComponent),
				MockComponent(ExpansionPanelComponent)
			],
			providers: [
				{
					provide: VorgangService,
					useValue: vorgangService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(AnhangListContainerComponent);
		component = fixture.componentInstance;
		component.attachmentStateResource$ = of(createEmptyStateResource());
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('on vorgang with "attachment" link', () => {

		beforeEach(() => {
			component.vorgangWithEingang = vorgangWithAttachments;
			fixture.detectChanges();
		})

		it('should attachments visible', () => {
			const element = fixture.nativeElement.querySelector(anhaenge);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should empty attachments not visible', () => {
			const element = fixture.nativeElement.querySelector(emptyAnhaenge);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('on vorgang without "attachment" link', () => {

		beforeEach(() => {
			component.vorgangWithEingang = vorgang;
			fixture.detectChanges();
		})

		it('should attachments not visible', () => {
			const element = fixture.nativeElement.querySelector(anhaenge);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})

		it('should empty attachments visible', () => {
			const element = fixture.nativeElement.querySelector(emptyAnhaenge);

			expect(element).toBeInstanceOf(HTMLElement);
		})
	})

	describe('ng on changes', () => {

		it('should call vorgang service getAttachments', () => {
			component.ngOnChanges();

			expect(vorgangService.getAttachments).toHaveBeenCalled();
		})
	})
});