import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatTooltip } from '@angular/material/tooltip';
import { OzgFileLinkRel, OzgFileResource } from '@goofy-client/ozg-file-shared';
import { createEmptyStateResource, FileSizePipe, ToResourceUriPipe } from '@goofy-client/tech-shared';
import { IconButtonWithSpinnerComponent, SpinnerComponent } from '@goofy-client/ui';
import { createOzgFileResource } from 'libs/ozg-file-shared/test/ozg-file';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { OzgFileComponent } from './ozg-file.component';

describe('OzgFileComponent', () => {
	let component: OzgFileComponent;
	let fixture: ComponentFixture<OzgFileComponent>;

	const file: OzgFileResource = createOzgFileResource([OzgFileLinkRel.DOWNLOAD]);
	const buttonSelector = '[data-test-class="file-download-button"]'

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				OzgFileComponent,
				MatIcon,
				FileSizePipe,
				ToResourceUriPipe,
				MockComponent(MatTooltip),
				MockComponent(SpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(OzgFileComponent);
		component = fixture.componentInstance;
		component.file = file;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Button', () => {
		it('should NOT disabled initial', () => {
			component.stateResource = createEmptyStateResource();
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(buttonSelector);

			expect(button).not.toBeDisabled()
		});

		it('should disabled if loading', () => {
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(buttonSelector);

			expect(button).toBeDisabled();
		});
	})

	describe('click on download button', () => {

		it('should emit download if link exists', () => {
			jest.spyOn(component.startDownload, 'emit');
			component.file = createOzgFileResource([OzgFileLinkRel.DOWNLOAD]);

			component.downloadFile();

			expect(component.startDownload.emit).toHaveBeenCalledWith(component.file);
		})

		it('should not emit download', () => {
			jest.spyOn(component.startDownload, 'emit');
			component.file = createOzgFileResource();

			component.downloadFile();

			expect(component.startDownload.emit).not.toHaveBeenCalled();
		})
	})

	describe('click on delete button', () => {

		it('should emit delete', () => {
			jest.spyOn(component.startDelete, 'emit');
			component.file = createOzgFileResource();

			component.deleteFile();

			expect(component.startDelete.emit).toHaveBeenCalledWith(component.file);
		})
	})
});
