import { Component, EventEmitter, Input, Output } from '@angular/core';
import { OzgFileLinkRel, OzgFileResource } from '@goofy-client/ozg-file-shared';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { hasLink } from '@ngxp/rest';
import { isNil } from 'lodash-es';

@Component({
	selector: 'goofy-client-ozg-file',
	templateUrl: './ozg-file.component.html',
	styleUrls: ['./ozg-file.component.scss']
})
export class OzgFileComponent {

	@Input() file: OzgFileResource;
	@Input() stateResource: StateResource<any>;
	@Input() deletable: boolean = false;

	readonly fileLinkRel = OzgFileLinkRel;

	@Output() public startDownload: EventEmitter<OzgFileResource> = new EventEmitter<OzgFileResource>();
	@Output() public startDelete: EventEmitter<OzgFileResource> = new EventEmitter<OzgFileResource>();

	get isDisabled(): boolean {
		return this.getStateResource().loading;
	}

	getStateResource(): StateResource<any> {
		return isNil(this.stateResource) ? createEmptyStateResource() : this.stateResource;
	}

	downloadFile(): void {
		if (hasLink(this.file, OzgFileLinkRel.DOWNLOAD)) this.startDownload.emit(this.file);
	}

	deleteFile(): void {
		this.startDelete.emit(this.file);
	}
}