import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { FormatListDatePipe } from './format-list-date.pipe';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('FormatListDatePipe', () => {
	let pipe: FormatListDatePipe = new FormatListDatePipe();

	it('should create', () => {
		expect(pipe).toBeTruthy();
	});

	it('should return "Heute"', () => {
		const date: Date = new Date();

		const result: string = pipe.transform(date);

		expect(result).toBe('Heute');
	})

	describe('format date in the past', () => {

		it('should return "Gestern"', () => {
			const date: Date = new Date();
			date.setDate(date.getDate() - 1);

			const result: string = pipe.transform(date);

			expect(result).toBe('Gestern');
		})
		it('should return "Vorgestern"', () => {
			const date: Date = new Date();
			date.setDate(date.getDate() - 2);

			const result: string = pipe.transform(date);

			expect(result).toBe('Vorgestern');
		})
	})

	describe('format date in the future', () => {
		it('should return "Morgen"', () => {
			const date: Date = new Date();
			date.setDate(date.getDate() + 1);

			const result: string = pipe.transform(date);

			expect(result).toBe('Morgen');
		})

		it('should return "Übermorgen"', () => {
			const date: Date = new Date();
			date.setDate(date.getDate() + 2);

			const result: string = pipe.transform(date);

			expect(result).toBe('Übermorgen');
		})
	})

	it('should return formatted for this year', () => {
		const date: Date = new Date();
		date.setDate(date.getDate() + 3);

		const result: string = pipe.transform(date);

		expect(result.length).toBeLessThan(8);
	})

	it('should format in "dd.MM.yyyy"', () => {
		const date: Date = new Date();
		date.setFullYear(2050, 4, 10);

		const result: string = pipe.transform(date);

		expect(result).toBe('10.05.2050');
	})
});
