import { getUrl, Resource } from '@ngxp/rest';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { ToResourceUriPipe } from './to-resource-uri.pipe';

describe('ToResourceUriPipe', () => {
	const base64ResultWithSlash: string = 'subjects?_d=1';
	const pipe = new ToResourceUriPipe();

	it('create an instance', () => {
		expect(pipe).toBeTruthy();
	});

	describe('pass resource in pipe', () => {
		const dummyResource: Resource = createDummyResource();

		it('output as expected', () => {
			const pipeResult: string = pipe.transform(dummyResource);

			const result: string = decode(pipeResult);

			expect(result).toBe(getUrl(dummyResource));
		});
		it('replace "/" with "_" if necessary', () => {
			const pipeResult: string = pipe.transform({ _links: { self: { href: base64ResultWithSlash } } })

			const result: string = decode(pipeResult);

			expect(result).toBe(base64ResultWithSlash);
		});
	})

	describe('pass resource and linkrel in pipe', () => {
		const linkRel: string = 'neuerLink';
		const dummyResourceWithLink: Resource = createDummyResource([linkRel]);

		it('output as expected', () => {
			const pipeResult: string = pipe.transform(dummyResourceWithLink, linkRel);

			const result: string = decode(pipeResult);

			expect(result).toBe(getUrl(dummyResourceWithLink, linkRel));
		});
	})

	function decode(encodedResult: string): string {
		return atob(encodedResult.replace('_', '/'));
	}
});