import { Resource } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { ApiError } from '../tech.model';

export interface ListResource extends Resource {
	_embedded
}

export interface StateResource<T> {
	resource: T;
	reload: boolean;
	loading: boolean;
	loaded: boolean;
	error?: ApiError;
}

export function createEmptyStateResource<T>(loading: boolean = false): StateResource<T> {
	return createStateResource(<T>null, loading);
}

export function createStateResource<T>(resource: T, loading: boolean = false): StateResource<T> {
	return { loading, loaded: !isNil(resource), reload: false, resource }
}

export function createErrorStateResource<T>(error: ApiError): StateResource<any> {
	return { ...createEmptyStateResource(), error };
}

export function doIfLoadingRequired(stateResource: StateResource<any>, runable: () => void): boolean {
	if (isLoadingRequired(stateResource)) {
		runable();
		return true
	}
	return false;
}

export function isLoadingRequired(stateResource: StateResource<any>): boolean {
	return !stateResource.loading && ((!stateResource.loaded) || stateResource.reload);
}

export function hasError(stateResource: StateResource<any>): boolean {
	return !isNil(stateResource.error);
}