import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { CommandResource } from '@goofy-client/command-shared';
import { ApiError, createEmptyStateResource, createErrorStateResource, StateResource } from '@goofy-client/tech-shared';
import { createApiError } from 'libs/tech-shared/test/error';
import { Observable } from 'rxjs';
import { AbstractFormService } from './formservice.abstract';

describe('AbstractFormService', () => {
	let formService: AbstractFormService;

	beforeEach(() => {
		formService = new DummyFormService(new FormBuilder());
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	describe('handleResponse', () => {
		const apiError: ApiError = createApiError();
		const stateResource: StateResource<CommandResource> = createErrorStateResource(apiError);

		beforeEach(() => {
			formService.setError = jest.fn();
		})

		it('should setError on validation error', () => {
			formService.handleResponse({ ...stateResource, loading: false });

			expect(formService.setError).toHaveBeenCalledWith(apiError);
		})

		it('should return stateresource while loading', () => {
			const commandStateResource: StateResource<CommandResource> = createEmptyStateResource(true);

			const result: StateResource<CommandResource> = formService.handleResponse(commandStateResource);

			expect(result).toBe(commandStateResource);
		})
	})

	describe('patch', () => {

		it('should set form value', () => {
			const formValue = { [DummyFormService.FIELD]: 'huhu' };

			formService.patch(formValue);

			expect(formService.form.value).toEqual(formValue);
		})
	})
})


export class DummyFormService extends AbstractFormService {

	static FIELD = 'field';

	constructor(formBuilder: FormBuilder) {
		super(formBuilder);
	}

	initForm(): FormGroup {
		return this.formBuilder.group({
			[DummyFormService.FIELD]: new FormControl(null)
		});
	}

	doSubmit(): Observable<StateResource<any>> {
		throw new Error('Method not implemented.');
	}

	getPathPrefix(): string {
		throw new Error('Method not implemented.');
	}
}