import { FormBuilder, FormGroup } from '@angular/forms';
import { ApiError, hasError, setValidationError, StateResource } from '@goofy-client/tech-shared';
import { isNil } from 'lodash-es';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

export abstract class AbstractFormService {
	form: FormGroup;
	pathPrefix: string;
	source: any;

	constructor(public formBuilder: FormBuilder) {
		this.form = this.initForm();
	}

	protected abstract initForm(): FormGroup;

	public submit(): Observable<StateResource<any>> {
		return this.doSubmit().pipe(map(result => this.handleResponse(result)));
	}

	protected abstract doSubmit(): Observable<StateResource<any>>;

	handleResponse(result: StateResource<any>): StateResource<any> {
		if (result.loading) return result;
		if (hasError(result)) {
			this.setError(result.error);
		}
		return result;
	}

	patch(valueToPatch: any): void {
		this.form.reset();
		this.form.patchValue(valueToPatch);

		this.source = valueToPatch;

		this.doAfterPatch(valueToPatch);
	}

	doAfterPatch(source: any): void {
		//No Implementation here for abstract class
	}

	setError(apiError: ApiError): void {
		apiError.issues.forEach(issue => setValidationError(this.form, issue, this.getPathPrefix()))
	}

	protected abstract getPathPrefix(): string;

	getFormValue(): any {
		return this.form.value;
	}

	protected getSourceValue(): any {
		return this.source;
	}

	public isPatch(): boolean {
		return !isNil(this.source);
	}
}