import { replaceAllWhitespaces, replacePlaceholder, replacePlaceholders } from './tech.util';

describe('TechUtil', () => {

	describe('replaceAllWhitespaces', () => {

		it('should replace whitespaces in value', () => {
			const value: string = 'value to replace whitespaces';

			const result: string = replaceAllWhitespaces(value, '_');

			expect(result).toBe('value_to_replace_whitespaces');
		})
	})

	describe('replace placeholder', () => {
		it('should replace with value', () => {
			const text = 'replace {this}';
			const placeholder = 'this';
			const value = 'with this';

			const replaced = replacePlaceholder(text, placeholder, value);

			expect(replaced).toBe('replace with this');
		});

		it('should replace all', () => {
			const text = 'replace {here} and {here}';
			const placeholder = 'here';
			const value = 'there';

			const replaced = replacePlaceholder(text, placeholder, value);

			expect(replaced).toBe('replace there and there');
		});

		it('should ignore other placeholder', () => {
			const text = 'replace {here}';
			const placeholder = 'there';
			const value = 'this';

			const replaced = replacePlaceholder(text, placeholder, value);

			expect(replaced).toBe('replace {here}');
		})
	});

	describe('replace placeholders', () => {
		it('should replace all placeholder', () => {
			const text = 'replace {here} and {there}'
			const placeholders = { here: 'this', there: 'that' };

			const replaced = replacePlaceholders(text, placeholders);

			expect(replaced).toBe('replace this and that');
		})
	})
})