import { AbstractControl, FormGroup } from '@angular/forms';
import { isNil } from 'lodash-es';
import { Issue } from '../tech.model';
import { replacePlaceholder } from '../tech.util';
import { VALIDATION_MESSAGES } from './tech.validaton.messages';

export function isValidationError(issue: Issue): boolean {
	return issue.messageCode.includes('javax.validation.constraints')
}

export function setValidationError(form: FormGroup, issue: Issue, pathPrefix?: string) {
	const control = getControlForIssue(form, issue, pathPrefix);

	control.setErrors({ [issue.messageCode]: issue });
	control.markAsTouched();
}

export function getControlForIssue(form: FormGroup, issue: Issue, pathPrefix?: string): AbstractControl {
	const fieldPath = pathPrefix ? issue.field.substr(pathPrefix.length + 1) : issue.field;

	let curControl: AbstractControl = form;
	fieldPath.split('.').forEach(field => curControl = (<FormGroup>curControl).controls[field]);

	return curControl;
}

export function getMessageForIssue(label: string, issue: Issue) {
	let msg = VALIDATION_MESSAGES[issue.messageCode];

	if (isNil(msg)) {
		console.warn('No message for code ' + issue.messageCode + ' found.');
		return issue.messageCode;
	}

	msg = replacePlaceholder(msg, 'field', label);
	issue.parameters.forEach(param =>
		msg = replacePlaceholder(msg, param.name, param.value)
	);
	return msg;
}

