import { Resource } from '@ngxp/rest';
import * as faker from 'faker';
import { isUndefined, times, zipObject } from 'lodash-es';
import { ListResource } from '../src/lib/resource/resource.util';
import { createDummy, DummyListLinkRel } from './dummy';

// @ts-ignore
export function toResource<R = T & Resource, T = any>(
	model: T,
	linkRelations: string[] = [],
	embeddedResources?: { [key: string]: Resource | Resource[] }
): R {
	const resource: Resource = {
		...(model),
		_links: {
			self: {
				href: faker.internet.url()
			},
			...zipObject(
				linkRelations,
				times(linkRelations.length, () => ({
					href: faker.internet.url()
				}))
			)
		}
	};

	if (!isUndefined(embeddedResources)) {
		resource._embedded = embeddedResources;
	}

	return <any>resource;
}

export function createDummyResource(linkRelations: string[] = []): Resource {
	return toResource(createDummy(), linkRelations);
}

export function createDummyListResource(linkRel: string[] = []): ListResource {
	return toResource({}, [...linkRel], {
		[DummyListLinkRel.LIST]: createDummyResource()
	})
}