import { Component, EventEmitter, Type } from '@angular/core';

export type Mock<T> = { [K in keyof T]: jest.Mock; };

export function mock<T>(service: Type<T>): Mock<T> {
	return <Mock<T>>Object.getOwnPropertyNames(service.prototype).reduce((mock, key) => ({
		...mock,
		[key]: jest.fn()
	}), (<any>{}));
}

export function useFromMock<T>(mock: Mock<T>): T {
	return <T><any>mock;
}

export function mockComponent(options: Component): Component {

	const metadata: Component = {
		selector: options.selector,
		template: options.template || '',
		inputs: options.inputs,
		outputs: options.outputs || [],
		exportAs: options.exportAs || ''
	};

	class Mock { }

	metadata.outputs.forEach(method => {
		Mock.prototype[method] = new EventEmitter<any>();
	});

	return Component(metadata)(Mock as any);
}