import { MatIconRegistry } from '@angular/material/icon';
import { IconService } from './icon.service';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { Icons } from './icon.model';

describe('IconService', () => {
	let service: IconService;
	let matIconRegistry: Mock<MatIconRegistry>;
	let domSanitizer: Mock<any>;

	const iconBasePath: string = '/assets/icons/';
	const iconsObjectLength = Object.keys(Icons).length

	beforeEach(() => {
		matIconRegistry = mock(MatIconRegistry);
		domSanitizer = { bypassSecurityTrustResourceUrl: jest.fn() };

		service = new IconService(useFromMock(matIconRegistry), useFromMock(domSanitizer));
	})

	it('should create', () => {
		expect(service).toBeDefined();
	})

	describe('add Icon to registry', () => {
		it('should call registry to add SvgIgon', () => {
			service.registerIcons();

			expect(matIconRegistry.addSvgIcon).toHaveBeenCalled();
			expect(matIconRegistry.addSvgIcon).toHaveBeenCalledTimes(iconsObjectLength);
		})

		it('should call domSanitizer', () => {
			service.registerIcons();

			expect(domSanitizer.bypassSecurityTrustResourceUrl).toHaveBeenCalled();
			expect(domSanitizer.bypassSecurityTrustResourceUrl).toHaveBeenCalledTimes(iconsObjectLength);
		})

		it('should call domSanitizer for `incoming` icon', () => {
			service.registerIcons();

			expect(domSanitizer.bypassSecurityTrustResourceUrl).toHaveBeenCalledWith(iconBasePath + 'incoming.svg');
		})
	})
})
