import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIconModule } from '@angular/material/icon';
import { MatSnackBarModule, MatSnackBarRef, MAT_SNACK_BAR_DATA } from '@angular/material/snack-bar';
import { mock } from '@goofy-client/test-utils';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { configureTestSuite } from 'ng-bullet';
import { SnackbarComponent } from './snackbar.component';
import { SnackBarData } from './snackbar.service';

describe('SnackbarComponent', () => {
	let component: SnackbarComponent;
	let fixture: ComponentFixture<SnackbarComponent>;

	const snackBarData: SnackBarData = { message: 'Message', commandResource: createCommandResource() };
	const snackBarRef = { ...mock(MatSnackBarRef), dissmis: jest.fn(), dismissWithAction: jest.fn() };

	const revokeButton: string = '[data-test-id="revoke-button"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				MatSnackBarModule,
				MatIconModule
			],
			declarations: [SnackbarComponent],
			providers: [
				{
					provide: MAT_SNACK_BAR_DATA,
					useValue: { data: snackBarData }
				},
				{
					provide: MatSnackBarRef,
					useValue: snackBarRef
				}
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(SnackbarComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('clear button', () => {
		it('should close the snackbar', () => {
			component.close();

			expect(snackBarRef.dismiss).toHaveBeenCalled();
		})
	})

	describe('revoke button', () => {
		it('should be visible', () => {
			component.data = { message: 'Message', commandResource: createCommandResource([CommandLinkRel.REVOKE]) };

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(revokeButton);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.data = { message: 'Message', commandResource: createCommandResource() };

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(revokeButton);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})

		it('should close the snackbar with action', () => {
			component.revoke();

			expect(snackBarRef.dismissWithAction).toHaveBeenCalled();
		})
	})
});
