import { MatSnackBar } from '@angular/material/snack-bar';
import { CommandResource } from '@goofy-client/command-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { SnackbarComponent } from './snackbar.component';
import { SnackBarService } from './snackbar.service';

describe('SnackBarService', () => {
	let service: SnackBarService;
	let snackBar: Mock<MatSnackBar>;

	const commandResource: CommandResource = createCommandResource();
	const message: string = 'Snackbar Message';

	beforeEach(() => {
		snackBar = mock(MatSnackBar);

		service = new SnackBarService(useFromMock(snackBar));
	})

	it('should create', () => {
		expect(service).toBeDefined();
	})

	describe('show snackbar', () => {

		beforeEach(() => {
			service.listenOnAction = jest.fn();
			service.listenOnAfterDismissed = jest.fn();
		})

		it('should open from component', () => {
			service.showSnackbar(commandResource, message);

			expect(snackBar.openFromComponent).toHaveBeenCalledWith(SnackbarComponent, {
				data: { message, commandResource }, duration: 10000,
			});
		})

		it('should listen to snackbar action', () => {
			service.showSnackbar(commandResource, message, null);

			expect(service.listenOnAction).toHaveBeenCalledWith(null);
		})

		it('should listen to snackbar after dismissed', () => {
			service.showSnackbar(commandResource, message, null);

			expect(service.listenOnAfterDismissed).toHaveBeenCalled();
		})
	})
})