import { Injectable } from '@angular/core';
import { MatSnackBar, MatSnackBarRef } from '@angular/material/snack-bar';
import { CommandResource } from '@goofy-client/command-shared';
import { isNil } from 'lodash-es';
import { Subscription } from 'rxjs';
import { SnackbarComponent } from './snackbar.component';

@Injectable({ providedIn: 'root' })
export class SnackBarService {

	private subscription: Subscription;
	private snackBarRef: MatSnackBarRef<SnackbarComponent>;
	private durationTime: number = 10000;

	constructor(private snackBar: MatSnackBar) { }

	public showSnackbar(commandResource: CommandResource, message: string, revokeAction?: () => void) {
		this.openSnackBar(commandResource, message);
		this.listenToActions(revokeAction);
	}

	private openSnackBar(commandResource: CommandResource, message: string): void {
		this.snackBarRef = this.snackBar.openFromComponent(SnackbarComponent, {
			data: { message, commandResource }, duration: this.durationTime,
		});
	}

	private listenToActions(revokeAction: () => void): void {
		this.listenOnAction(revokeAction);
		this.listenOnAfterDismissed();
	}

	listenOnAction(revokeAction: () => void): void {
		this.subscription = this.snackBarRef.onAction().subscribe(() => {
			revokeAction();
			this.closeSnackBar();
		});
	}

	listenOnAfterDismissed() {
		this.subscription.add(this.snackBarRef.afterDismissed().subscribe(() => this.closeSnackBar()));
	}

	private closeSnackBar() {
		this.snackBarRef.dismiss();
		this.unsubscribe();
	}

	private unsubscribe(): void {
		if (!isNil(this.subscription)) this.subscription.unsubscribe();
	}
}

export interface SnackBarData {
	message: string,
	commandResource: CommandResource
}
