import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatButton } from '@angular/material/button';
import { MatRipple } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatTooltipModule } from '@angular/material/tooltip';
import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { MockComponent } from 'ng-mocks';
import { SpinnerComponent } from '../spinner/spinner.component';
import { ButtonWithSpinnerComponent } from './button-with-spinner.component';

describe('ButtonWithSpinnerComponent', () => {
	let component: ButtonWithSpinnerComponent;
	let fixture: ComponentFixture<ButtonWithSpinnerComponent>;

	const buttonSelector = '[data-test-class="icon-button"]'
	const iconSelector = '[data-test-class="icon"]'

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				MatButton,
				MatRipple,
				MatIcon,
				ButtonWithSpinnerComponent,
				MockComponent(SpinnerComponent)
			],
			imports: [
				MatTooltipModule
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(ButtonWithSpinnerComponent);
		component = fixture.componentInstance;
		component.icon = 'add';
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Button', () => {
		it('should NOT disabled initial', () => {
			const button = fixture.nativeElement.querySelector(buttonSelector);
			component.stateResource = createEmptyStateResource();
			fixture.detectChanges();

			expect(button).not.toBeDisabled()
		});

		it('should disabled if loading', () => {
			const button = fixture.nativeElement.querySelector(buttonSelector);
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			expect(button).toBeDisabled();
		});
	})

	describe('Icon', () => {
		it('should visible initial', () => {
			const icon = fixture.nativeElement.querySelector(iconSelector);
			component.stateResource = createEmptyStateResource();
			fixture.detectChanges();

			expect(icon).toBeVisible();
		});

		it('should not visible if loading', () => {
			const icon = fixture.nativeElement.querySelector(iconSelector);
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			expect(icon).not.toBeVisible();
		});
	})
});
