import { Component, Optional, Self } from '@angular/core';
import { ControlValueAccessor, FormControl, NgControl } from '@angular/forms';
import { Issue } from '@goofy-client/tech-shared';
import { isEmpty } from 'lodash-es';
import { Subscription } from 'rxjs';

@Component({
	template: 'NO UI'
})
export abstract class FormControlEditorAbstractComponent implements ControlValueAccessor {

	readonly fieldControl: FormControl = new FormControl();
	private onChange = (text: string) => { };
	public onTouched = () => { };

	private changesSubscr: Subscription;
	private statusSubscr: Subscription;

	disabled: boolean = false;

	constructor(@Self() @Optional() public control: NgControl) {
		if (this.control) this.control.valueAccessor = this;

		this.changesSubscr = this.fieldControl.valueChanges.subscribe(val => {
			val = isEmpty(val) ? null : val;
			this.onChange(val);
			this.setErrors();
		});
	}

	ngOnInit(): void {
		if (!this.statusSubscr && this.control) this.statusSubscr = this.control.statusChanges.subscribe(() => {
			this.setErrors();
		});
	}

	touch(): void {
		this.onTouched();
	}

	writeValue(text: string): void {
		this.fieldControl.setValue(text);
		this.setErrors();
	}
	registerOnChange(fn: (text: string) => {}): void {
		this.onChange = fn;
	}
	registerOnTouched(fn: () => {}): void {
		this.onTouched = fn;
	}
	setDisabledState?(isDisabled: boolean): void {
		this.disabled = isDisabled;
	}

	ngOnDestroy(): void {
		if (this.changesSubscr) this.changesSubscr.unsubscribe();
		if (this.statusSubscr) this.statusSubscr.unsubscribe();
	}

	private setErrors() {
		if (this.control) {
			this.fieldControl.setErrors(this.control.errors);
			if (this.control.invalid) this.fieldControl.markAsTouched();
		}
	}

	get issues(): Issue[] {
		return this.fieldControl.errors ? Object.keys(this.fieldControl.errors).map(key => <Issue>this.fieldControl.errors[key]) : [];
	}
}