import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, NgControl, ReactiveFormsModule } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ConvertForDataTestPipe } from '@goofy-client/tech-shared';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { ValidationErrorComponent } from '../../validation-error/validation-error.component';
import { TextEditorComponent } from './text-editor.component';

describe('TextEditorComponent', () => {
	let component: TextEditorComponent;
	let fixture: ComponentFixture<TextEditorComponent>;

	const input: string = '[data-test-id="Ein_Label-text-input"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				TextEditorComponent,
				ConvertForDataTestPipe,
				MockComponent(ValidationErrorComponent)
			],
			imports: [
				MatFormFieldModule,
				MatInputModule,
				ReactiveFormsModule,
				BrowserAnimationsModule
			],
			providers: [
				{
					provide: NgControl,
					useValue: new FormControl()
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(TextEditorComponent);
		component = fixture.componentInstance;
		component.label = 'Ein Label';
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should exist for input', () => {
		const element: HTMLElement = fixture.nativeElement.querySelector(input);

		expect(element).toBeInstanceOf(HTMLElement);
	});

	describe('max length attribute', () => {

		let inputElement: HTMLElement;

		beforeEach(() => {
			inputElement = fixture.nativeElement.querySelector(input);
		})

		it('should not be present if no length given', () => {
			expect(inputElement).not.toHaveAttribute('maxlength');
		});

		it('should be present if length is given', () => {
			component.maxlength = 5;
			fixture.detectChanges();

			expect(inputElement).toHaveAttribute('maxlength', '5');
		})
	})
});
