import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatTooltip } from '@angular/material/tooltip';
import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { MockComponent } from 'ng-mocks';
import { SpinnerComponent } from '../spinner/spinner.component';
import { IconButtonWithSpinnerComponent } from './icon-button-with-spinner.component';
import { configureTestSuite } from 'ng-bullet';

describe('IconButtonWithSpinnerComponent', () => {
	let component: IconButtonWithSpinnerComponent;
	let fixture: ComponentFixture<IconButtonWithSpinnerComponent>;

	const buttonSelector = '[data-test-class="icon-button"]'
	const iconSelector = '[data-test-class="icon"]'

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				IconButtonWithSpinnerComponent,
				MatIcon,
				MockComponent(MatTooltip),
				MockComponent(SpinnerComponent)
			],
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(IconButtonWithSpinnerComponent);
		component = fixture.componentInstance;
		component.icon = 'edit';

		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Button', () => {
		it('should NOT disabled initial', () => {
			const button = fixture.nativeElement.querySelector(buttonSelector);
			component.stateResource = createEmptyStateResource();
			fixture.detectChanges();

			expect(button).not.toBeDisabled()
		});

		it('should disabled if loading', () => {
			const button = fixture.nativeElement.querySelector(buttonSelector);
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			expect(button).toBeDisabled();
		});
	})

	describe('Icon', () => {
		it('should visible initial', () => {
			const icon = fixture.nativeElement.querySelector(iconSelector);
			component.stateResource = createEmptyStateResource();
			fixture.detectChanges();

			expect(icon).toBeVisible();
		});

		it('should not visible if loading', () => {
			const icon = fixture.nativeElement.querySelector(iconSelector);
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			expect(icon).not.toBeVisible();
		});
	})
});
