import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatProgressBar } from '@angular/material/progress-bar';
import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { configureTestSuite } from 'ng-bullet';
import { ProgressBarComponent } from './progress-bar.component';

describe('ProgressBarComponent', () => {
	let component: ProgressBarComponent;
	let fixture: ComponentFixture<ProgressBarComponent>;

	const overlaySelector = '[data-test-class="progressbar-overlay"]'

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				ProgressBarComponent,
				MatProgressBar
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(ProgressBarComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('progressbar overlay', () => {
		it('should NOT be visible if NOT loading and NOT loaded', () => {
			component.stateResource = { ...createEmptyStateResource(false), loaded: false };
			fixture.detectChanges();

			const overlay = fixture.nativeElement.querySelector(overlaySelector);

			expect(overlay).not.toBeInstanceOf(HTMLElement);
		});

		it('should NOT be visible if NOT loading and already loaded', () => {
			component.stateResource = { ...createEmptyStateResource(false), loaded: true };
			fixture.detectChanges();

			const overlay = fixture.nativeElement.querySelector(overlaySelector);

			expect(overlay).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible if loading', () => {
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			const overlay = fixture.nativeElement.querySelector(overlaySelector);

			expect(overlay).toBeInstanceOf(HTMLElement);
		});
	});
});
