import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatSpinner } from '@angular/material/progress-spinner';
import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { configureTestSuite } from 'ng-bullet';
import { SpinnerComponent } from './spinner.component';

describe('SpinnerComponent', () => {
	let component: SpinnerComponent;
	let fixture: ComponentFixture<SpinnerComponent>;
	const spinnerSelector = '[data-test-class="spinner"]'

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				SpinnerComponent,
				MatIcon,
				MatSpinner
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(SpinnerComponent);
		component = fixture.componentInstance;

		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('spinner', () => {
		it('should NOT be visible if NOT loading and already loaded', () => {
			component.stateResource = { ...createEmptyStateResource(false), loaded: true };
			fixture.detectChanges();

			const spinner = fixture.nativeElement.querySelector(spinnerSelector);

			expect(spinner).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if loading', () => {
			component.stateResource = createEmptyStateResource(true);
			fixture.detectChanges();

			const spinner = fixture.nativeElement.querySelector(spinnerSelector);

			expect(spinner).toBeInstanceOf(HTMLElement);
		});
	});

	describe('loaded spinner', () => {
		beforeEach(() => {
			component.show =  false;
			fixture.detectChanges();
		})

		it('should NOT be visible if showIf return false', () => {
			const spinner = fixture.nativeElement.querySelector(spinnerSelector);

			expect(spinner).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if loaded', () => {
			component.show = true;
			fixture.detectChanges();

			const spinner = fixture.nativeElement.querySelector(spinnerSelector);

			expect(spinner).toBeInstanceOf(HTMLElement);
		});
	});
});
