import { ApiRootLinkRel, ApiRootResource } from '@goofy-client/api-root-shared';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangHeaderLinkRel, VorgangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { createUserProfileResource } from '../../test/user-profile';
import { UserProfileResource } from './user-profile.model';
import { UserProfileRepository } from './user-profile.repository';

describe('OzgFileRepository', () => {
	const resourceFactory = mock(ResourceFactory);
	const resourceWrapper = { get: jest.fn() };
	let repository: UserProfileRepository;

	const vorgang: VorgangResource = createVorgangResource();
	const userProfile: UserProfileResource = createUserProfileResource();
	const apiRoot: ApiRootResource = createApiRootResource([ApiRootLinkRel.SEARCH_USER_PROFILES]);

	beforeEach(() => {
		repository = new UserProfileRepository(useFromMock(resourceFactory));

		resourceFactory.from.mockReturnValue(resourceWrapper);
		resourceFactory.fromId.mockReturnValue(resourceWrapper);
		resourceWrapper.get.mockReturnValue(cold('a', { a: userProfile }));
	})

	it('should be created', () => {
		expect(repository).toBeTruthy();
	})

	describe('get user profile', () => {

		const linkRel: string = VorgangHeaderLinkRel.ASSIGNED_TO;

		it('should call resource factory', () => {
			repository.getUserProfile(vorgang, linkRel);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
		})

		it('should get', () => {
			repository.getUserProfile(vorgang, linkRel);

			expect(resourceWrapper.get).toHaveBeenCalledWith(linkRel);
		})

		it('should return result', () => {
			const result = repository.getUserProfile(vorgang, linkRel);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(hot('a', { a: userProfile }));
		})
	})

	describe('search', () => {

		const searchBy: string = 'SearchMe';

		it('should call resource factory', () => {
			repository.search(apiRoot, searchBy);

			expect(resourceFactory.fromId).toHaveBeenCalled();
		})

		it('should build search-url', () => {
			repository.buildSearchByUrl = jest.fn();

			repository.search(apiRoot, searchBy);

			expect(repository.buildSearchByUrl).toHaveBeenCalledWith(apiRoot, searchBy);
		})
	})
})