import { ApiRootResource, ApiRootService } from '@goofy-client/api-root-shared';
import { createStateResource, NavigationService } from '@goofy-client/tech-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangHeaderLinkRel, VorgangResource } from '@goofy-client/vorgang-shared';
import { getLink } from '@ngxp/rest';
import { hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createUserProfileListResource, createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { UserProfileListResource, UserProfileResource } from './user-profile.model';
import { UserProfileRepository } from './user-profile.repository';
import { UserProfileService } from './user-profile.service';

describe('UserProfileService', () => {
	let service: UserProfileService;
	let repository: Mock<UserProfileRepository>;
	let apiRootService: Mock<ApiRootService>;
	let navigationService: Mock<NavigationService>;

	const vorgang: VorgangResource = createVorgangResource([VorgangHeaderLinkRel.ASSIGNED_TO]);
	const userProfile: UserProfileResource = createUserProfileResource();
	const userProfileList: UserProfileListResource = createUserProfileListResource();
	const apiRoot: ApiRootResource = createApiRootResource();

	beforeEach(() => {
		repository = mock(UserProfileRepository);
		apiRootService = mock(ApiRootService);
		navigationService = mock(NavigationService);
		navigationService.urlChanged = jest.fn();
		navigationService.urlChanged.mockReturnValue(of({}));

		service = new UserProfileService(useFromMock(repository), useFromMock(apiRootService), useFromMock(navigationService));
	})

	it('should created', () => {
		expect(service).toBeTruthy();
	})

	describe('load user profile', () => {

		const linkRel: string = VorgangHeaderLinkRel.ASSIGNED_TO;

		beforeEach(() => {
			repository.getUserProfile.mockReturnValue(of(userProfile));
			service.createIfNotExist(getLink(vorgang, linkRel).href);
			service.updateUserProfile = jest.fn();
		})

		it('should call repository', () => {
			service.loadUserProfile(vorgang, VorgangHeaderLinkRel.ASSIGNED_TO);

			expect(repository.getUserProfile).toHaveBeenCalledWith(vorgang, linkRel);
		})

		it('should update value', () => {
			service.loadUserProfile(vorgang, VorgangHeaderLinkRel.ASSIGNED_TO);

			expect(service.updateUserProfile).toHaveBeenCalledWith(vorgang, linkRel, userProfile);
		})
	})

	describe('search', () => {

		const searchBy: string = 'SearchMe';

		beforeEach(() => {
			repository.search.mockReturnValue(hot('-a', { a: userProfileList }))
			apiRootService.getApiRoot.mockReturnValue(of(createStateResource(apiRoot)));
		})

		it('should call repository', () => {
			service.search(searchBy);

			expect(repository.search).toHaveBeenCalledWith(apiRoot, searchBy);
		})

		it('should call api-root service', () => {
			service.search(searchBy);

			expect(apiRootService.getApiRoot).toHaveBeenCalled();
		})
	})
})