import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { UserProfileService } from '@goofy-client/user-profile-shared';
import { VorgangHeaderLinkRel } from '@goofy-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { AssignUserProfileButtonContainerComponent } from './assign-user-profile-button-container.component';

describe('AssignUserProfileButtonContainerComponent', () => {
	let component: AssignUserProfileButtonContainerComponent;
	let fixture: ComponentFixture<AssignUserProfileButtonContainerComponent>;

	const assignUserProfileButton: string = '[data-test-id="assign-user-profile-icon-button"]';

	const userProfileService = mock(UserProfileService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				AssignUserProfileButtonContainerComponent,
				HasLinkPipe,
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: UserProfileService,
					useValue: userProfileService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(AssignUserProfileButtonContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('button', () => {

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangHeaderLinkRel.ASSIGN]);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(assignUserProfileButton);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(assignUserProfileButton);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('showUserProfileSearch', () => {

		it('should call service', () => {
			component.showUserProfileSearch();

			expect(userProfileService.showUserProfileSearch).toHaveBeenCalled();
		})
	})
});
