import { Component, Input, SimpleChanges } from '@angular/core';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { UserProfileResource } from '@goofy-client/user-profile-shared';
import { isNil } from 'lodash';

@Component({
	selector: 'goofy-client-user-icon',
	templateUrl: './user-icon.component.html',
	styleUrls: ['./user-icon.component.scss']
})
export class UserIconComponent {

	@Input() userProfileStateResource: StateResource<UserProfileResource> = createEmptyStateResource();
	@Input() disableTooltip: boolean = false;

	readonly toolTipNoUser: string = 'kein Bearbeiter zugewiesen'

	toolTip: string;

	ngOnChanges(changes: SimpleChanges): void {
		if (changes.userProfileStateResource) {
			this.toolTip = this.userProfileStateResource.resource ? this.userProfileStateResource.resource.firstName + ' ' + this.userProfileStateResource.resource.lastName : this.toolTipNoUser;
		}
	}

	get initials(): string {
		return this.getFirstLetter(this.userProfileStateResource.resource.firstName) + this.getFirstLetter(this.userProfileStateResource.resource.lastName);
	}

	private getFirstLetter(value: string): string {
		return isNil(value) ? undefined : value.substr(0, 1).toUpperCase();
	}
}
