import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { UserProfileService } from '@goofy-client/user-profile-shared';
import { createKommentarResource } from 'libs/kommentar-shared/test/kommentar';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { UserIconComponent } from '../user-icon/user-icon.component';
import { UserProfileInKommentarContainerComponent } from './user-profile-in-kommentar-container.component';

describe('UserProfileInKommentarContainerComponent', () => {
	let component: UserProfileInKommentarContainerComponent;
	let fixture: ComponentFixture<UserProfileInKommentarContainerComponent>;

	const kommentarCreatedAt: string = '[data-test-id="user-profile-in-kommentar-created-at"]';

	const userProfileService = { ...mock(UserProfileService), getAssignedUserProfile: () => of(createStateResource(createUserProfileResource())) };

	const createdAt: Date = new Date('2021-06-29T11:50:00');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				UserProfileInKommentarContainerComponent,
				MockComponent(UserIconComponent)
			],
			providers: [
				{
					provide: UserProfileService,
					useValue: userProfileService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(UserProfileInKommentarContainerComponent);
		component = fixture.componentInstance;
		component.kommentar = { ...createKommentarResource(), createdAt };
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should show createdAt from kommentar', () => {
		const element = fixture.nativeElement.querySelector(kommentarCreatedAt);

		expect(element).toBeInstanceOf(HTMLElement);
		expect((<HTMLElement>element).innerHTML).toEqual('29.06.21 11:50');
	})
});
