import { Component, Input, ViewChild } from '@angular/core';
import { MatMenuTrigger } from '@angular/material/menu';
import { StateResource } from '@goofy-client/tech-shared';
import { UserProfileResource, UserProfileService } from '@goofy-client/user-profile-shared';
import { isNil } from 'lodash-es';
import { Observable } from 'rxjs';

@Component({
	selector: 'goofy-client-user-profile-button-container',
	templateUrl: './user-profile-button-container.component.html',
	styleUrls: ['./user-profile-button-container.component.scss']
})
export class UserProfileButtonContainerComponent {

	@Input() userProfile: StateResource<UserProfileResource>;

	@ViewChild(MatMenuTrigger) menuTrigger: MatMenuTrigger;

	showUserProfileSearch$: Observable<boolean>;

	constructor(public userProfileService: UserProfileService) {
		this.showUserProfileSearch$ = this.userProfileService.isUserProfileSearchVisible();
	}

	ngAfterContentInit(): void {
		this.subscribeToUserProfileSearchVisibility();
	}

	subscribeToUserProfileSearchVisibility(): void {
		this.userProfileService.isUserProfileSearchVisible().subscribe(isVisible => this.triggerMenu(isVisible))
	}

	triggerMenu(visible: boolean): void {
		if (!isNil(this.menuTrigger)) {
			visible ? this.menuTrigger.openMenu() : this.menuTrigger.closeMenu();
		}
	}

	showUserProfileSearch(): void {
		this.userProfileService.showUserProfileSearch();
	}

	hideUserProfileSearch(): void {
		this.userProfileService.hideUserProfileSearch();
	}
}