import { Component, Input, SimpleChanges } from '@angular/core';
import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@goofy-client/api-root-shared';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { UserProfileResource, UserProfileService } from '@goofy-client/user-profile-shared';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { hasLink } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { Observable, of } from 'rxjs';

@Component({
	selector: 'goofy-client-user-profile-in-vorgang-container',
	templateUrl: './user-profile-in-vorgang-container.component.html',
	styleUrls: ['./user-profile-in-vorgang-container.component.scss']
})
export class UserProfileInVorgangContainerComponent {

	@Input() vorgang: VorgangWithEingangResource;

	apiRoot$: Observable<StateResource<ApiRootResource>> = of(createEmptyStateResource());
	userProfile$: Observable<StateResource<UserProfileResource>> = of(createEmptyStateResource());

	readonly apiRootLinkRel = ApiRootLinkRel;

	constructor(private apiRootService: ApiRootService, public userProfileService: UserProfileService) {
		this.apiRoot$ = this.apiRootService.getApiRoot();
	}

	ngOnChanges(changes: SimpleChanges): void {
		if (changes.vorgang && !isNil(this.vorgang) && hasLink(this.vorgang, VorgangHeaderLinkRel.ASSIGNED_TO)) {
			this.userProfile$ = this.userProfileService.getAssignedUserProfile(this.vorgang, VorgangHeaderLinkRel.ASSIGNED_TO);
		}
	}
}