import { Component, EventEmitter, Output } from '@angular/core';
import { CommandResource, isDone } from '@goofy-client/command-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { Selectable } from '@goofy-client/ui';
import { UserProfileListLinkRel, UserProfileListResource, UserProfileResource, UserProfileService } from '@goofy-client/user-profile-shared';
import { VorgangService } from '@goofy-client/vorgang-shared';
import { getEmbeddedResource, getUrl } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { map, tap } from 'rxjs/operators';

@Component({
	selector: 'goofy-client-user-profile-search-container',
	templateUrl: './user-profile-search-container.component.html',
	styleUrls: ['./user-profile-search-container.component.scss']
})
export class UserProfileSearchContainerComponent {

	@Output() assigned: EventEmitter<void> = new EventEmitter();

	readonly EMPTY_LIST = [];

	assignTo$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource<CommandResource>());
	searchedUserProfiles$: Observable<StateResource<Selectable[]>> = of(createStateResource<Selectable[]>([]));

	assignedToName: string = '';

	constructor(private vorgangService: VorgangService, private service: UserProfileService) {
		this.searchedUserProfiles$ = this.service.getSearchedUserProfiles().pipe(map(userProfileListStateResource => {
			if (userProfileListStateResource.loaded) {
				return createStateResource(this.parseToSelectables(userProfileListStateResource.resource));
			}
			return { ...userProfileListStateResource, resource: this.EMPTY_LIST };
		}));
	}

	parseToSelectables(userProfileListResource: UserProfileListResource): Selectable[] {
		const resources: UserProfileResource[] = getEmbeddedResource(userProfileListResource, UserProfileListLinkRel.USER_PROFILE_LIST);
		return resources ? resources.map(userProfile => this.parseToSelectable(userProfile)) : [];
	}

	parseToSelectable(userProfile: UserProfileResource) {
		return <Selectable>{ label: userProfile.firstName + ' ' + userProfile.lastName, value: getUrl(userProfile) };
	}

	assign(userProfile: Selectable): void {
		this.assignedToName = userProfile.label;
		this.assignTo$ = this.vorgangService.assignUser(userProfile.value).pipe(tap((commandStateResource: StateResource<CommandResource>) => {
			if (isDone(commandStateResource.resource)) {
				this.assigned.emit();
			}
		}));
	}
}
