import { Component, EventEmitter, Input, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { createStateResource, StateResource } from '@goofy-client/tech-shared';
import { Selectable } from '@goofy-client/ui';
import { isNull } from 'lodash-es';
import { UserProfileSearchFormService } from './user-profile.search.formservice';

@Component({
	selector: 'goofy-client-user-profile-search',
	templateUrl: './user-profile-search.component.html',
	styleUrls: ['./user-profile-search.component.scss'],
	providers: [UserProfileSearchFormService]
})
export class UserProfileSearchComponent {

	@Input() searchedUserProfiles: StateResource<Selectable[]> = createStateResource<Selectable[]>([]);

	@Output() assignTo: EventEmitter<Selectable> = new EventEmitter();

	readonly formServiceClass = UserProfileSearchFormService;

	constructor(private formService: UserProfileSearchFormService) { }

	get form(): FormGroup {
		return this.formService.form;
	}

	assign(userProfile: Selectable): void {
		isNull(userProfile)
			? this.setNoUserProfileFoundError()
			: this.assignTo.emit(userProfile);
	}

	setEmptySelectionError(): void {
		this.formService.setEmptyUserProfileError();
	}

	setNoUserProfileFoundError(): void {
		this.formService.setNoUserProfileFoundError();
	}

	onKeyUp(): void {
		this.formService.submit();
	}
}