import { Injectable, OnDestroy } from "@angular/core";
import { FormBuilder, FormControl, FormGroup } from "@angular/forms";
import { AbstractFormService, StateResource } from "@goofy-client/tech-shared";
import { UserProfileListResource, UserProfileService } from "@goofy-client/user-profile-shared";
import { isNil } from "lodash-es";
import { Observable, Subscription } from "rxjs";

@Injectable()
export class UserProfileSearchFormService extends AbstractFormService implements OnDestroy {

	public static readonly SEARCH_FIELD = 'searchBy';

	private FIELD_PATH: string = 'only.fe';

	private valueSubscription: Subscription;

	constructor(formBuilder: FormBuilder, private userProfileService: UserProfileService) {
		super(formBuilder);
	}

	protected initForm(): FormGroup {
		return this.formBuilder.group({
			[UserProfileSearchFormService.SEARCH_FIELD]: new FormControl()
		})
	}

	protected doSubmit(): Observable<StateResource<UserProfileListResource>> {
		return this.userProfileService.search(this.getSearchString());
	}

	private getSearchString(): string {
		return this.getFormValue()[UserProfileSearchFormService.SEARCH_FIELD];
	}

	protected getPathPrefix(): string {
		return this.FIELD_PATH;
	}

	public setEmptyUserProfileError(): void {
		this.setError(emptyUserProfileError);
	}

	public setNoUserProfileFoundError(): void {
		this.setError(noUserProfileFoundError);
	}

	ngOnDestroy(): void {
		if (!isNil(this.valueSubscription)) this.valueSubscription.unsubscribe();
	}
}

const noUserProfileFoundError = { issues: [{ field: 'only.fe.searchBy', message: 'fe_only_validation_bearbeiter_not_exist', messageCode: 'fe_only_validation_bearbeiter_not_exist', parameters: [] }] };
const emptyUserProfileError = { issues: [{ field: 'only.fe.searchBy', message: 'validation_field_empty', messageCode: 'validation_field_empty', parameters: [] }] };