import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BearbeitenButtonComponent } from './bearbeiten-button.component';

describe('BearbeitenButtonComponent', () => {
	let component: BearbeitenButtonComponent;
	let fixture: ComponentFixture<BearbeitenButtonComponent>;

	const bearbeitenButton: string = '[data-test-id="bearbeiten-button"]';
	const bearbeitenIconButton: string = '[data-test-id="bearbeiten-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				BearbeitenButtonComponent,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent),
				HasLinkPipe
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(BearbeitenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('bearbeiten', () => {

		it('should call vorgang service', () => {
			component.bearbeiten();

			expect(vorgangCommandService.bearbeiten).toHaveBeenCalled();
		})
	})

	describe('bearbeiten button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bearbeitenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BEARBEITEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bearbeitenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('bearbeiten icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bearbeitenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BEARBEITEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bearbeitenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
