import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BescheidenButtonComponent } from './bescheiden-button.component';

describe('BescheidenButtonComponent', () => {
	let component: BescheidenButtonComponent;
	let fixture: ComponentFixture<BescheidenButtonComponent>;

	const bescheidenButton: string = '[data-test-id="bescheiden-button"]';
	const bescheidenIconButton: string = '[data-test-id="bescheiden-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				BescheidenButtonComponent,
				HasLinkPipe,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(BescheidenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('bescheiden', () => {

		it('should call vorgang service', () => {
			component.bescheiden();

			expect(vorgangCommandService.bescheiden).toHaveBeenCalled();
		})
	})

	describe('bescheiden button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bescheidenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEIDEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bescheidenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('bescheiden icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bescheidenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEIDEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(bescheidenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
