import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VerwerfenButtonComponent } from './verwerfen-button.component';

describe('VerwerfenButtonComponent', () => {
	let component: VerwerfenButtonComponent;
	let fixture: ComponentFixture<VerwerfenButtonComponent>;

	const verwerfenButton: string = '[data-test-id="verwerfen-button"]';
	const verwerfenIconButton: string = '[data-test-id="verwerfen-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VerwerfenButtonComponent,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent),
				HasLinkPipe
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(VerwerfenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('verwerfen', () => {

		it('should call vorgang service', () => {
			component.verwerfen();

			expect(vorgangCommandService.verwerfen).toHaveBeenCalled();
		})
	})

	describe('verwerfen button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(verwerfenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.VERWERFEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(verwerfenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('verwerfen icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(verwerfenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.VERWERFEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(verwerfenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
