import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { WiedereroeffnenButtonComponent } from './wiedereroeffnen-button.component';

describe('WiedereroeffnenButtonComponent', () => {
	let component: WiedereroeffnenButtonComponent;
	let fixture: ComponentFixture<WiedereroeffnenButtonComponent>;
	const wiedereroeffnenButton: string = '[data-test-id="wiedereroeffnen-button"]';
	const wiedereroeffnenIconButton: string = '[data-test-id="wiedereroeffnen-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedereroeffnenButtonComponent,
				HasLinkPipe,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(WiedereroeffnenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('wiedereroeffnen', () => {

		it('should call vorgang service', () => {
			component.wiedereroeffnen();

			expect(vorgangCommandService.wiedereroeffnen).toHaveBeenCalled();
		})
	})

	describe('wiedereroeffnen button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(wiedereroeffnenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.WIEDEREROEFFNEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(wiedereroeffnenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('wiedereroeffnen icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(wiedereroeffnenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.WIEDEREROEFFNEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(wiedereroeffnenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
