import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { ZurueckholenButtonComponent } from './zurueckholen-button.component';

describe('ZurueckholenButtonComponent', () => {
	let component: ZurueckholenButtonComponent;
	let fixture: ComponentFixture<ZurueckholenButtonComponent>;

	const zurueckholenButton: string = '[data-test-id="zurueckholen-button"]';
	const zurueckholenIconButton: string = '[data-test-id="zurueckholen-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				ZurueckholenButtonComponent,
				HasLinkPipe,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(ZurueckholenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('zurueckholen', () => {

		it('should call vorgang service', () => {
			component.zurueckholen();

			expect(vorgangCommandService.zurueckholen).toHaveBeenCalled();
		})
	})

	describe('zurueckholen button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckholenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ZURUECKHOLEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckholenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('zurueckholen icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckholenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ZURUECKHOLEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckholenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
