import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangWithEingangLinkRel } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { ZurueckstellenButtonComponent } from './zurueckstellen-button.component';

describe('ZurueckstellenButtonComponent', () => {
	let component: ZurueckstellenButtonComponent;
	let fixture: ComponentFixture<ZurueckstellenButtonComponent>;

	const zurueckstellenButton: string = '[data-test-id="zurueckstellen-button"]';
	const zurueckstellenIconButton: string = '[data-test-id="zurueckstellen-icon-button"]';

	const vorgangCommandService = { ...mock(VorgangCommandService), getCommandStateResources: () => of(createStateResource(createCommandResource())) };

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				ZurueckstellenButtonComponent,
				HasLinkPipe,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(ZurueckstellenButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('zurueckstellen', () => {

		it('should call vorgang service', () => {
			component.zurueckstellen();

			expect(vorgangCommandService.zurueckstellen).toHaveBeenCalled();
		})
	})

	describe('zurueckstellen button', () => {

		beforeEach(() => {
			component.showAsIconButton = false;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckstellenButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ZURUECKSTELLEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckstellenButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})

	describe('zurueckstellen icon button', () => {

		beforeEach(() => {
			component.showAsIconButton = true;
			fixture.detectChanges();
		})

		it('should be hidden', () => {
			component.vorgang = createVorgangWithEingangResource();

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckstellenIconButton);

			expect(buttonElement).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.ZURUECKSTELLEN]);

			fixture.detectChanges();
			const buttonElement = fixture.nativeElement.querySelector(zurueckstellenIconButton);

			expect(buttonElement).toBeInstanceOf(HTMLElement);
		});
	})
});
