import { ComponentFixture, TestBed } from '@angular/core/testing';
import { VorgangForwardingContainerComponent } from '@goofy-client/forwarding';
import { KommentarListInVorgangContainerComponent } from '@goofy-client/kommentar';
import { AnhangListContainerComponent } from '@goofy-client/ozg-file';
import { createEmptyStateResource, createStateResource, HasLinkPipe, ToEmbeddedResourcesPipe } from '@goofy-client/tech-shared';
import { ButtonWithSpinnerComponent, ExpansionPanelComponent } from '@goofy-client/ui';
import { VorgangHeaderLinkRel, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { WiedervorlageListInVorgangContainerComponent } from '@goofy-client/wiedervorlage';
import { SpinnerComponent } from 'libs/ui/src/lib/ui/spinner/spinner.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailAreaComponent } from './vorgang-detail-area.component';
import { VorgangDetailBodyComponent } from './vorgang-detail-body/vorgang-detail-body.component';
import { VorgangDetailFormularButtonsComponent } from './vorgang-detail-formular-buttons/vorgang-detail-formular-buttons.component';
import { VorgangDetailHeaderComponent } from './vorgang-detail-header/vorgang-detail-header.component';
import { MatDialog, MatDialogModule, MatDialogRef } from '@angular/material/dialog';

describe('VorgangDetailAreaComponent', () => {
	let component: VorgangDetailAreaComponent;
	let fixture: ComponentFixture<VorgangDetailAreaComponent>;

	const wiedervorlagenContainer: string = '[data-test-id="wiedervorlagen-container-in-vorgang"]';
	const kommentarContainer: string = '[data-test-id="kommentar-container-in-vorgang"]';
	const forwardingContainer: string = '[data-test-id="forwarding-container-in-vorgang"]';

	const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

	beforeEach(() => {
		TestBed.configureTestingModule({
			imports: [
				MatDialogModule
			],
			declarations: [
				VorgangDetailAreaComponent,
				HasLinkPipe,
				ToEmbeddedResourcesPipe,
				MockComponent(SpinnerComponent),
				MockComponent(VorgangDetailHeaderComponent),
				MockComponent(VorgangDetailBodyComponent),
				MockComponent(VorgangDetailFormularButtonsComponent),
				MockComponent(ExpansionPanelComponent),
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(WiedervorlageListInVorgangContainerComponent),
				MockComponent(VorgangForwardingContainerComponent),
				MockComponent(AnhangListContainerComponent),
				MockComponent(KommentarListInVorgangContainerComponent)
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailAreaComponent);
		component = fixture.componentInstance;
		component.vorgangStateResource = createEmptyStateResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('wiedervorlagen', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangHeaderLinkRel.WIEDERVORLAGEN]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(wiedervorlagenContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(wiedervorlagenContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('kommentare', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangHeaderLinkRel.KOMMENTARE]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(kommentarContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(kommentarContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('forwarding', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARDING]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(forwardingContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(forwardingContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})
});
