import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ExpansionPanelComponent } from 'libs/ui/src/lib/ui/expansion-panel/expansion-panel.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { AntragstellerComponent } from './antragsteller/antragsteller.component';
import { EingangHeaderComponent } from './eingang-header/eingang-header.component';
import { FormDataTableComponent } from './form-data-table/form-data-table.component';
import { VorgangDetailBodyComponent } from './vorgang-detail-body.component';

describe('VorgangDetailBodyComponent', () => {
	let component: VorgangDetailBodyComponent;
	let fixture: ComponentFixture<VorgangDetailBodyComponent>;

	const formulardatenPanelSelector = '[data-test-id="formulardaten-panel"]';
	const headerPanelSelector = '[data-test-id="header-panel"]';
	const antragstellerPanelSelector = '[data-test-id="antragsteller-panel"]';
	const formulardatenDataTableSelector = '[data-test-id="formulardaten-data-table"]';
	const antragstellerDataTableSelector = '[data-test-id="antragsteller-data-table"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangDetailBodyComponent,
				MockComponent(ExpansionPanelComponent),
				MockComponent(EingangHeaderComponent),
				MockComponent(AntragstellerComponent),
				MockComponent(FormDataTableComponent),
			],

		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailBodyComponent);
		component = fixture.componentInstance;

		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Formulardaten Panel ', () => {
		it('should NOT be visible', () => {

			const formulardatenPanel = fixture.nativeElement.querySelector(formulardatenPanelSelector);

			expect(formulardatenPanel).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if data has children', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const formulardatenPanel = fixture.nativeElement.querySelector(formulardatenPanelSelector);

			expect(formulardatenPanel).toBeInstanceOf(HTMLElement);
		});
	});

	describe('Eingang Header Panel', () => {
		it('should NOT be visible', () => {

			const headerPanel = fixture.nativeElement.querySelector(headerPanelSelector);

			expect(headerPanel).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if data has children', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const headerPanel = fixture.nativeElement.querySelector(headerPanelSelector);

			expect(headerPanel).toBeInstanceOf(HTMLElement);
		});
	});

	describe('Antragsteller Panel', () => {
		it('should NOT be visible', () => {

			const antragstellerPanel = fixture.nativeElement.querySelector(antragstellerPanelSelector);

			expect(antragstellerPanel).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if data has children', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const antragstellerPanel = fixture.nativeElement.querySelector(antragstellerPanelSelector);

			expect(antragstellerPanel).toBeInstanceOf(HTMLElement);
		});
	});

	describe('Formulardaten Data Table', () => {
		it('should NOT be visible', () => {

			const formulardatenDataTable = fixture.nativeElement.querySelector(formulardatenDataTableSelector);

			expect(formulardatenDataTable).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if data has children', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const formulardatenDataTable = fixture.nativeElement.querySelector(antragstellerPanelSelector);

			expect(formulardatenDataTable).toBeInstanceOf(HTMLElement);
		});
	});

	describe('Antragsteller Data Table', () => {
		it('should NOT be visible', () => {

			const antragstellerDataTable = fixture.nativeElement.querySelector(antragstellerDataTableSelector);

			expect(antragstellerDataTable).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible if data has children', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const antragstellerDataTable = fixture.nativeElement.querySelector(antragstellerPanelSelector);

			expect(antragstellerDataTable).toBeInstanceOf(HTMLElement);
		});
	});
});
