import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { createStateResource, StateResource } from '@goofy-client/tech-shared';
import { getElementFromFixtureByCss, mock } from '@goofy-client/test-utils';
import { VorgangCommandService, VorgangService, VorgangWithEingang, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ProgressBarComponent } from 'libs/ui/src/lib/ui/progress-bar/progress-bar.component';
import { SubnavigationComponent } from 'libs/ui/src/lib/ui/subnavigation/subnavigation.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { Subject } from 'rxjs';
import { VorgangDetailActionButtonsComponent } from './vorgang-detail-area/vorgang-detail-action-buttons/vorgang-detail-action-buttons.component';
import { VorgangDetailAreaComponent } from './vorgang-detail-area/vorgang-detail-area.component';
import { VorgangDetailFormularButtonsComponent } from './vorgang-detail-area/vorgang-detail-formular-buttons/vorgang-detail-formular-buttons.component';
import { VorgangDetailPageComponent } from './vorgang-detail-page.component';

describe('VorgangDetailPageComponent', () => {
	let component: VorgangDetailPageComponent;
	let fixture: ComponentFixture<VorgangDetailPageComponent>;

	const getVorgangWithEingangSubj: Subject<StateResource<VorgangWithEingang>> = new Subject();
	const vorgangService = { ...mock(VorgangService), getVorgangWithEingang: () => getVorgangWithEingangSubj };
	const vorgangCommandService = mock(VorgangCommandService);

	const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

	const subnavigation: string = 'goofy-client-subnavigation';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangDetailPageComponent,
				MatIcon,
				MockComponent(ProgressBarComponent),
				MockComponent(SubnavigationComponent),
				MockComponent(VorgangDetailAreaComponent),
				MockComponent(VorgangDetailActionButtonsComponent),
				MockComponent(VorgangDetailFormularButtonsComponent)
			],
			providers: [
				{
					provide: VorgangService,
					useValue: vorgangService
				},
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				}
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailPageComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('subnavigation', () => {

		it('should be hidden', () => {
			getVorgangWithEingangSubj.next(null);
			fixture.detectChanges();

			const subnavigationElement = getElementFromFixtureByCss(fixture, subnavigation);

			expect(subnavigationElement).toBeNull();
		})

		it('should be visible', () => {
			getVorgangWithEingangSubj.next(createStateResource(vorgangWithEingang));
			fixture.detectChanges();

			const subnavigationElement = getElementFromFixtureByCss(fixture, subnavigation);

			expect(subnavigationElement).toBeDefined();
		})
	})
});
