import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@goofy-client/api-root-shared';
import { createStateResource, HasLinkPipe, StateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { MockComponent } from 'ng-mocks';
import { BehaviorSubject } from 'rxjs';
import { VorgangSearchContainerComponent } from './vorgang-search-container.component';
import { VorgangSearchComponent } from './vorgang-search/vorgang-search.component';

describe('VorgangSearchContainerComponent', () => {
	let component: VorgangSearchContainerComponent;
	let fixture: ComponentFixture<VorgangSearchContainerComponent>;

	const apiRootSubj: BehaviorSubject<StateResource<ApiRootResource>> = new BehaviorSubject(createStateResource(createApiRootResource()));
	const apiRootService = { ...mock(ApiRootService), getApiRoot: () => apiRootSubj };

	const vorgangSearch: string = '[data-test-id="vorgang-search"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangSearchContainerComponent,
				HasLinkPipe,
				MockComponent(VorgangSearchComponent)
			],
			providers: [
				{
					provide: ApiRootService,
					useValue: apiRootService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangSearchContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('vorgang-search', () => {

		it('should hide on no link', () => {
			apiRootSubj.next(createStateResource(createApiRootResource()));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(vorgangSearch);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})

		it('should show if link exists', () => {
			apiRootSubj.next(createStateResource(createApiRootResource([ApiRootLinkRel.SEARCH])));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(vorgangSearch);

			expect(element).toBeInstanceOf(HTMLElement);
		})
	})
});
