import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, ReactiveFormsModule } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIcon } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterTestingModule } from '@angular/router/testing';
import { mock } from '@goofy-client/test-utils';
import { VorgangListService } from '@goofy-client/vorgang-shared';
import { BehaviorSubject, Subject } from 'rxjs';
import { VorgangSearchComponent } from './vorgang-search.component';
import { VorgangSearchFormService } from './vorgang-search.formservice';

describe('VorgangSearchComponent', () => {
	let component: VorgangSearchComponent;
	let fixture: ComponentFixture<VorgangSearchComponent>;

	const searchFormService = mock(VorgangSearchFormService);

	const searchStringSubj: Subject<boolean> = new BehaviorSubject(null);
	const listService = { ...mock(VorgangListService), getSearchString: () => searchStringSubj };

	const clearButton: string = '[data-test-id="clear-button"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangSearchComponent,
				MatIcon,
			],
			imports: [
				MatFormFieldModule,
				MatInputModule,
				ReactiveFormsModule,
				BrowserAnimationsModule,
				RouterTestingModule
			],
			providers: [
				FormBuilder,
				{
					provide: VorgangSearchFormService,
					useValue: searchFormService
				},
				{
					provide: VorgangListService,
					useValue: listService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangSearchComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('clear button', () => {

		it('should not show on empty search string', () => {
			searchStringSubj.next(false);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(clearButton);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})

		it('should shown when input is filled', () => {
			searchStringSubj.next(true);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(clearButton);

			expect(button).toBeInstanceOf(HTMLElement);
		})
	})
});