import { Injectable, OnDestroy } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { VorgangListService } from '@goofy-client/vorgang-shared';
import { isEmpty, isNil } from 'lodash-es';
import { Subscription } from 'rxjs';

@Injectable()
export class VorgangSearchFormService implements OnDestroy {

	form: FormGroup;

	readonly SEARCH_FIELD: string = VorgangListService.SEARCH;

	private subscription: Subscription;

	constructor(private service: VorgangListService, private formBuilder: FormBuilder, private router: Router) {
		this.initForm();

		this.subscribeToSearchString();
	}

	private initForm(): void {
		this.form = this.formBuilder.group({
			[this.SEARCH_FIELD]: new FormControl(null)
		})
	}

	private subscribeToSearchString(): void {
		this.subscription = this.service.getSearchString().subscribe(value => this.patch(value))
	}

	private patch(value: string): void {
		this.form.patchValue({ [this.SEARCH_FIELD]: value });
	}

	submit(): void {
		const search: string = this.getValue();

		if (!isEmpty(search)) {
			this.router.navigate([this.SEARCH_FIELD + '/', encodeURI(search)]);
		} else {
			this.router.navigateByUrl('/');
		}
	}

	clear(): void {
		this.form.controls[this.SEARCH_FIELD].patchValue('');
	}

	hasSearchString(): boolean {
		return !!this.form.controls[this.SEARCH_FIELD].value;
	}

	getValue(): string {
		const value: string = this.form.value.search;
		return isEmpty(value) ? null : value;
	}

	ngOnDestroy(): void {
		if (!isNil(this.subscription)) this.subscription.unsubscribe();
	}
}