import { CreateCommand } from '@goofy-client/command-shared';
import { ListResource } from '@goofy-client/tech-shared';
import { Resource, ResourceUri } from '@ngxp/rest';

export interface Vorgang {
	status: VorgangStatus;
	aktenzeichen: string;
	createdAt: Date;
	name: string;
	nextFrist: Date;
}

export interface VorgangWithEingang extends Vorgang {
	eingang: Eingang;
}

export interface Eingang {
	id: string,
	antragsteller: Antragsteller,
	zustaendigeStelle: ZustaendigeStelle,
	header: EingangHeader,
	formData: ReadonlyMap<number, string> | object
}

export interface Antragsteller {
	anrede: string,
	geburtsdatum: string,
	geburtsname: string,
	geburtsort: string,
	nachname: string,
	vorname: string,
	email: string,
	formData: ReadonlyMap<number, string> | object
}

export interface ZustaendigeStelle {
	email: string;
}

export interface EingangHeader {
	client: string,
	clientId: string,
	createdAt: string,
	customer: string,
	customerId: string,
	formId: string,
	formName: string,
	requestId: string,
	sender: string
}

export enum VorgangStatus {
	NEU = 'NEU',
	ANGENOMMEN = 'ANGENOMMEN',
	VERWORFEN = 'VERWORFEN',
	IN_BEARBEITUNG = 'IN_BEARBEITUNG',
	BESCHIEDEN = 'BESCHIEDEN',
	ABGESCHLOSSEN = 'ABGESCHLOSSEN',
	WEITERGELEITET = 'WEITERGELEITET'
}

export enum VorgangOrder {
	ANNEHMEN = 'VORGANG_ANNEHMEN',
	VERWERFEN = 'VORGANG_VERWERFEN',
	ZURUECKHOLEN = 'VORGANG_ZURUECKHOLEN',
	BEARBEITEN = 'VORGANG_BEARBEITEN',
	BESCHEIDEN = 'VORGANG_BESCHEIDEN',
	ZURUECKSTELLEN = 'VORGANG_ZURUECKSTELLEN',
	ABSCHLIESSEN = 'VORGANG_ABSCHLIESSEN',
	WIEDEREROEFFNEN = 'VORGANG_WIEDEREROEFFNEN',
	FORWARD = 'REDIRECT_VORGANG',
	FORWARD_SUCCESSFULL = 'FORWARD_SUCCESSFULL',
	FORWARD_FAILED = 'FORWARD_FAILED',
	ASSIGN_USER = 'ASSIGN_USER'
}

export interface VorgangResource extends Vorgang, Resource { }

export interface VorgangListResource extends ListResource { }

export interface VorgangWithEingangResource extends VorgangWithEingang, Resource { }

export interface VorgangDetailSnackBarData {
	message: string;
	showRevokeButton: boolean;
}

export interface CreateForwardCommand extends CreateCommand {
	order: VorgangOrder;
	redirectRequest: ForwardRequest;
}

export interface ForwardRequest {
	email: string;
	password: string;
}

export interface CreateAssignUserCommand extends CreateCommand {
	order: VorgangOrder;
	body: AssignUserBody;
}

export interface AssignUserBody {
	assignedTo: ResourceUri;
}
