import { Injectable } from '@angular/core';
import { ApiRootLinkRel, ApiRootResource } from '@goofy-client/api-root-shared';
import { OzgFileListResource } from '@goofy-client/ozg-file-shared';
import { getUrl, ResourceFactory, ResourceUri } from '@ngxp/rest';
import { Observable } from 'rxjs/internal/Observable';
import { VorgangListLinkRel, VorgangWithEingangLinkRel } from './vorgang.linkrel';
import { VorgangListResource, VorgangWithEingangResource } from './vorgang.model';

@Injectable({ providedIn: 'root' })
export class VorgangRepository {

	static SEARCH_PARAM: string = 'searchBy';

	constructor(private resourceFactory: ResourceFactory) { }

	public loadVorgangList(apiRootResource: ApiRootResource): Observable<VorgangListResource> {
		return this.resourceFactory.from(apiRootResource).get(ApiRootLinkRel.VORGAENGE);
	}

	public getNextVorgangListPage(vorgangListResource: VorgangListResource): Observable<VorgangListResource> {
		return this.resourceFactory.from(vorgangListResource).get(VorgangListLinkRel.NEXT);
	}

	public getVorgang(vorgangWithEingangUrl: ResourceUri): Observable<VorgangWithEingangResource> {
		return this.resourceFactory.fromId(vorgangWithEingangUrl).get();
	}

	public getAttachments(vorgang: VorgangWithEingangResource): Observable<OzgFileListResource> {
		return this.resourceFactory.from(vorgang).get(VorgangWithEingangLinkRel.ATTACHMENTS);
	}

	public searchVorgaengeBy(apiRootResource: ApiRootResource, searchBy: string): Observable<VorgangListResource> {
		return this.resourceFactory.fromId(this.buildSearchByUrl(apiRootResource, searchBy)).get();
	}

	buildSearchByUrl(apiRootResource: ApiRootResource, searchBy: string): ResourceUri {
		return getUrl(apiRootResource, ApiRootLinkRel.SEARCH).replace('{&' + VorgangRepository.SEARCH_PARAM + '}', '&' + VorgangRepository.SEARCH_PARAM + '=' + searchBy);
	}
}