import { CommandListResource, CommandResource, CreateCommand } from '@goofy-client/command-shared';
import { createCommandListResource, createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangForwardRequest } from 'libs/vorgang-shared/test/vorgang';
import { ForwardRequest, VorgangOrder } from './vorgang.model';
import { createAbschliessenCommand, createAnnehmenCommand, createBearbeitenCommand, createBescheidenCommand, createForwardCommand, createVerwerfenCommand, createZurueckholenCommand, createZurueckstellenCommand, getPendingForwardingCommand } from './vorgang.util';

describe('VorgangUtil', () => {

	describe('create ZurueckholenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createZurueckholenCommand();

			expect(command.order).toBe(VorgangOrder.ZURUECKHOLEN);
		})
	})

	describe('create AnnehmenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createAnnehmenCommand();

			expect(command.order).toBe(VorgangOrder.ANNEHMEN);
		})
	})

	describe('create VerwerfenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createVerwerfenCommand();

			expect(command.order).toBe(VorgangOrder.VERWERFEN);
		})
	})

	describe('create BearbeitenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createBearbeitenCommand();

			expect(command.order).toBe(VorgangOrder.BEARBEITEN);
		})
	})

	describe('create BescheidenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createBescheidenCommand();

			expect(command.order).toBe(VorgangOrder.BESCHEIDEN);
		})
	})

	describe('create ZurückstellenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createZurueckstellenCommand();

			expect(command.order).toBe(VorgangOrder.ZURUECKSTELLEN);
		})
	})

	describe('create AbschliessenCommand', () => {

		it('should return command', () => {
			const command: CreateCommand = createAbschliessenCommand();

			expect(command.order).toBe(VorgangOrder.ABSCHLIESSEN);
		})
	})

	describe('create ForwardCommand', () => {

		it('should return command', () => {
			const forwardRequest: ForwardRequest = createVorgangForwardRequest();

			const result: CreateCommand = createForwardCommand(forwardRequest);

			expect(result).toEqual({ order: VorgangOrder.FORWARD, redirectRequest: forwardRequest })
		})
	})

	describe('getPendingForwardCommand', () => {

		it('should return null on non existing pending command', () => {
			const listResource: CommandListResource = createCommandListResource();

			const pendingCommand: CommandResource = getPendingForwardingCommand(listResource);

			expect(pendingCommand).toBe(null);
		})

		it('should return existing pending command', () => {
			const pendingForwardCommand: CommandResource = { ...createCommandResource(), order: VorgangOrder.FORWARD };
			const listResource: CommandListResource = createCommandListResource([pendingForwardCommand]);

			const pendingCommand: CommandResource = getPendingForwardingCommand(listResource);

			expect(pendingCommand).toBe(pendingForwardCommand);
		})
	})
})