import { UrlSegment } from '@angular/router';
import { CommandListResource, CommandResource, CreateCommand } from '@goofy-client/command-shared';
import { getEmbeddedResource } from '@ngxp/rest';
import { CommandListLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { VorgangListLinkRel } from './vorgang.linkrel';
import { CreateForwardCommand, ForwardRequest, VorgangListResource, VorgangOrder, VorgangResource } from './vorgang.model';

export function createZurueckholenCommand(): CreateCommand {
	return { order: VorgangOrder.ZURUECKHOLEN };
}

export function createAnnehmenCommand(): CreateCommand {
	return { order: VorgangOrder.ANNEHMEN };
}

export function createVerwerfenCommand(): CreateCommand {
	return { order: VorgangOrder.VERWERFEN };
}

export function createBearbeitenCommand(): CreateCommand {
	return { order: VorgangOrder.BEARBEITEN };
}

export function createBescheidenCommand(): CreateCommand {
	return { order: VorgangOrder.BESCHEIDEN };
}

export function createZurueckstellenCommand(): CreateCommand {
	return { order: VorgangOrder.ZURUECKSTELLEN };
}

export function createAbschliessenCommand(): CreateCommand {
	return { order: VorgangOrder.ABSCHLIESSEN };
}

export function createWiedereroeffnenCommand(): CreateCommand {
	return { order: VorgangOrder.WIEDEREROEFFNEN };
}

export function createForwardCommand(redirectRequest: ForwardRequest): CreateForwardCommand {
	return { order: VorgangOrder.FORWARD, redirectRequest };
}

export function isNotVorgangDetailPage(urlSegements: UrlSegment[]): boolean {
	return (urlSegements.length < 2) || (urlSegements[0].path !== 'vorgang')
}

export function isNotVorgangListPage(urlSegements: UrlSegment[]): boolean {
	return urlSegements.length !== 0;
}

export function getVorgaengeFromList(vorgangList: VorgangListResource): VorgangResource[] {
	return getEmbeddedResource(vorgangList, VorgangListLinkRel.VORGANG_HEADER_LIST);
}

export function getPendingForwardingCommand(pendingCommands: CommandListResource): CommandResource {
	var forwardCommands: CommandResource[] = getEmbeddedResource<CommandResource[]>(pendingCommands, CommandListLinkRel.COMMAND_LIST)
		.filter(command => command.order == VorgangOrder.FORWARD);
	return forwardCommands.length > 0 ? forwardCommands[0] : null;
}