import { Antragsteller, Eingang, EingangHeader, ForwardRequest, Vorgang, VorgangListLinkRel, VorgangListResource, VorgangResource, VorgangWithEingang, VorgangWithEingangResource, ZustaendigeStelle } from '@goofy-client/vorgang-shared';
import * as faker from 'faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';

export function createVorgang(): Vorgang {
	return {
		createdAt: faker.date.past(),
		name: faker.lorem.word(),
		status: faker.lorem.word().toUpperCase(),
		aktenzeichen: faker.lorem.word(),
		nextFrist: faker.date.future()
	}
}

export function createVorgangWithEingang(): VorgangWithEingang {
	return { ...createVorgangResource(), eingang: createEingang() }
}

export function createEingang(): Eingang {
	return {
		id: faker.random.uuid,
		antragsteller: createAntragsteller(),
		zustaendigeStelle: createZustaendigeStelle(),
		header: createHeader(),
		formData: null
	}
}

export function createAntragsteller(): Antragsteller {
	return {
		anrede: faker.lorem.word(),
		geburtsdatum: faker.date.past(),
		geburtsname: faker.name.findName(),
		geburtsort: faker.address.city(),
		nachname: faker.name.lastName(),
		vorname: faker.name.firstName(),
		email: faker.internet.email(),
		formData: null
	}
}

export function createZustaendigeStelle(): ZustaendigeStelle {
	return {
		email: faker.internet.email()
	}
}

export function createHeader(): EingangHeader {
	return {
		client: faker.lorem.word(),
		clientId: faker.lorem.word(),
		createdAt: faker.date.past(),
		customer: faker.lorem.word(),
		customerId: faker.lorem.word(),
		formId: faker.lorem.word(),
		formName: faker.lorem.word(),
		requestId: faker.lorem.word(),
		sender: faker.lorem.word()
	}
}

export function createVorgangResource(linkRelations: string[] = []): VorgangResource {
	return toResource(createVorgang(), linkRelations);
}

export function createVorgangResources(linkRelations: string[] = []): VorgangResource[] {
	return times(10, () => toResource(createVorgangResource(), [...linkRelations]));
}

export function createVorgangListResource(linkRelations: string[] = []): VorgangListResource {
	return toResource({}, [...linkRelations], {
		[VorgangListLinkRel.VORGANG_HEADER_LIST]: createVorgangResources()
	});
}

export function createVorgangWithEingangResource(linkRelations: string[] = []): VorgangWithEingangResource {
	return toResource(createVorgangWithEingang(), linkRelations);
}


export function createVorgangForwardRequest(): ForwardRequest {
	return {
		email: faker.internet.email(),
		password: faker.internet.password()
	}
}
