import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { MatTooltipModule } from '@angular/material/tooltip';
import { RouterTestingModule } from '@angular/router/testing';
import { EnumToLabelPipe, FormatDateWithTimePipe, FormatListDatePipe, HasLinkPipe, ToResourceUriPipe } from '@goofy-client/tech-shared';
import { UserProfileInVorgangListItemContainerComponent } from '@goofy-client/user-profile';
import { VorgangHeaderLinkRel } from '@goofy-client/vorgang-shared';
import { AktenzeichenComponent, StatusDotComponent } from '@goofy-client/vorgang-shared-ui';
import { WiedervorlageListInVorgangListContainerComponent } from '@goofy-client/wiedervorlage';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { VorgangListItemComponent } from './vorgang-list-item.component';
import { VorgangNextFristButton } from './vorgang-next-frist-button/vorgang-next-frist-button.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangListItemComponent', () => {
	let component: VorgangListItemComponent;
	let fixture: ComponentFixture<VorgangListItemComponent>;

	const user: string = '[data-test-id="vorgang-user-icon"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				RouterTestingModule,
				MatIconTestingModule,
				MatTooltipModule
			],
			declarations: [
				VorgangListItemComponent,
				MatIcon,
				FormatDateWithTimePipe,
				FormatListDatePipe,
				EnumToLabelPipe,
				ToResourceUriPipe,
				HasLinkPipe,
				MockComponent(AktenzeichenComponent),
				MockComponent(StatusDotComponent),
				MockComponent(WiedervorlageListInVorgangListContainerComponent),
				MockComponent(VorgangNextFristButton),
				MockComponent(UserProfileInVorgangListItemContainerComponent)
			],
			providers: [
				{ provide: LOCALE_ID, useValue: 'de' },
				{ provide: MAT_DATE_LOCALE, useValue: 'de-DE' }
			],
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangListItemComponent);
		component = fixture.componentInstance;
		component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('bearbeiter/user-icon', () => {

		it('should be visible', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.ASSIGN]);
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).toBeInstanceOf(HTMLElement);
		})

		it('should NOT be visible', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).not.toBeInstanceOf(HTMLElement);
		})
	})
});
