import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory, ResourceUri } from '@ngxp/rest';
import * as faker from 'faker';
import { cold, hot } from 'jest-marbles';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { createWiedervorlageListResource, createWiedervorlageResource } from '../../test/wiedervorlage';
import { WiedervorlageListResource, WiedervorlageResource } from './wiedervorlage.model';
import { WiedervorlageRepository } from './wiedervorlage.repository';

describe('WiedervorlageRepository', () => {
	let repository: WiedervorlageRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		repository = new WiedervorlageRepository(useFromMock(resourceFactory));
	})

	describe('loadWiedervorlagenList', () => {
		const wiedervorlageListResource: WiedervorlageListResource = createWiedervorlageListResource();
		const vorgangResource: VorgangWithEingangResource = createVorgangWithEingangResource();

		beforeEach(() => {
			resourceFactory.from.mockReturnValue(resourceWrapper);
			resourceWrapper.get.mockReturnValue(cold('a', { a: wiedervorlageListResource }));
		})

		it('should call resourceFactory with resource', () => {
			repository.findByVorgang(vorgangResource);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgangResource);
		})

		it('should call resourceWrapper with link', () => {
			repository.findByVorgang(vorgangResource);

			expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangHeaderLinkRel.WIEDERVORLAGEN);
		})

		it('should return result', () => {
			let result = repository.findByVorgang(vorgangResource);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(hot('a', { a: wiedervorlageListResource }));
		})
	})

	describe('getWiedervorlage', () => {
		const wiedervorlageUri: ResourceUri = faker.internet.url();
		const wiedervorlageResource: WiedervorlageResource = createWiedervorlageResource();

		beforeEach(() => {
			resourceFactory.fromId.mockReturnValue(resourceWrapper);
			resourceWrapper.get.mockReturnValue(cold('a', { a: wiedervorlageResource }));
		})

		it('should call resourceFactory with resource', () => {
			repository.getWiedervorlage(wiedervorlageUri);

			expect(resourceFactory.fromId).toHaveBeenCalledWith(wiedervorlageUri);
		})

		it('should call resourceWrapper with link', () => {
			repository.getWiedervorlage(wiedervorlageUri);

			expect(resourceWrapper.get).toHaveBeenCalled();
		})

		it('should return result', () => {
			let result = repository.getWiedervorlage(wiedervorlageUri);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(hot('a', { a: wiedervorlageResource }));
		})
	})
})