import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ActivatedRoute } from '@angular/router';
import { CommandResource } from '@goofy-client/command-shared';
import { createStateResource, NavigationService, StateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { WiedervorlageFormService } from '../wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';
import { SubmitWiedervorlageButtonComponent } from './submit-wiedervorlage-button.component';

describe('SubmitWiedervorlageButtonComponent', () => {
	let component: SubmitWiedervorlageButtonComponent;
	let fixture: ComponentFixture<SubmitWiedervorlageButtonComponent>;

	const wiedervorlageService = mock(WiedervorlageService);
	const navigationService = mock(NavigationService);
	const wiedervorlageFormService = mock(WiedervorlageFormService);
	const activatedRoute = mock(ActivatedRoute);

	const commandWithError: StateResource<CommandResource> = createStateResource(createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]));

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				SubmitWiedervorlageButtonComponent,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: WiedervorlageService,
					useValue: wiedervorlageService
				},
				{
					provide: WiedervorlageFormService,
					useValue: wiedervorlageFormService
				},
				{
					provide: NavigationService,
					useValue: navigationService
				},
				{
					provide: ActivatedRoute,
					useValue: activatedRoute
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(SubmitWiedervorlageButtonComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('navigateIfCommandIsDone', () => {

		it('should navigate on success', () => {
			component.navigateIfCommandIsDone(commandWithError);

			expect(navigationService.navigateRelativeTo).toHaveBeenCalled();
		})

		it('should NOT navigate on loading', () => {
			component.navigateIfCommandIsDone({ ...commandWithError, loading: true });

			expect(navigationService.navigateRelativeTo).toHaveBeenCalledWith('../../', activatedRoute);
		})
	})
});
