import { Component, Input } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { CommandResource, isDone } from '@goofy-client/command-shared';
import { NavigationService, StateResource } from '@goofy-client/tech-shared';
import { WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { Observable } from 'rxjs';
import { filter, tap } from 'rxjs/operators';
import { WiedervorlageFormService } from '../wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';

@Component({
	selector: 'goofy-client-submit-wiedervorlage-button',
	templateUrl: './submit-wiedervorlage-button.component.html',
	styleUrls: ['./submit-wiedervorlage-button.component.scss']
})
export class SubmitWiedervorlageButtonComponent {

	@Input() showAsIconButton: boolean = false;

	submitInProgress$: Observable<StateResource<any>>;

	constructor(
		private wiedervorlageService: WiedervorlageService,
		private formService: WiedervorlageFormService,
		private navigationService: NavigationService,
		private route: ActivatedRoute
	) {
		this.submitInProgress$ = this.wiedervorlageService.getSubmitInProgress();
	}

	submit(): void {
		this.submitInProgress$ = this.formService.submit().pipe(
			filter(command => !command.loading),
			tap(commandWithError => this.navigateIfCommandIsDone(commandWithError)),
		);
	}

	navigateIfCommandIsDone(command: StateResource<CommandResource>): void {
		if (!command.loading && isDone(command.resource)) {
			this.navigateToVorgangDetailPage();
		}
	}

	private navigateToVorgangDetailPage(): void {
		this.navigationService.navigateRelativeTo('../../', this.route);
	}
}