import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { IconButtonWithSpinnerComponent } from '@goofy-client/ui';
import { WiedervorlageLinkRel, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { WiedereroeffnenButtonContainerComponent } from './wiedereroeffnen-button-container.component';

describe('WiedereroeffnenButtonContainerComponent', () => {
	let component: WiedereroeffnenButtonContainerComponent;
	let fixture: ComponentFixture<WiedereroeffnenButtonContainerComponent>;

	const wiedereroeffnenIconButton = '[data-test-id="wiedereroeffnen-icon-button"]'

	const wiedervorlageService = mock(WiedervorlageService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedereroeffnenButtonContainerComponent,
				HasLinkPipe,
				MockComponent(IconButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: WiedervorlageService,
					useValue: wiedervorlageService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(WiedereroeffnenButtonContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('Wiedereroeffnen Button', () => {

		it('should be hidden', () => {
			const button = fixture.nativeElement.querySelector(wiedereroeffnenIconButton);

			expect(button).not.toBeInstanceOf(HTMLElement);
		});

		it('should be visible', () => {
			component.wiedervorlage = createWiedervorlageResource([WiedervorlageLinkRel.WIEDEREROEFFNEN]);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(wiedereroeffnenIconButton);

			expect(button).toBeInstanceOf(HTMLElement);
		});
	})

	describe('wiedereroeffnen', () => {

		it('should call service', () => {
			component.wiedereroeffnen();

			expect(wiedervorlageService.wiedereroeffnen).toHaveBeenCalled();
		})
	});
});
