import { Component, EventEmitter, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { OzgFileListLinkRel, OzgFileResource } from '@goofy-client/ozg-file-shared';
import { WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { getEmbeddedResource } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { Observable, of } from 'rxjs';
import { map } from 'rxjs/operators';

@Component({
	selector: 'goofy-client-wiedervorlage-attachment-list',
	templateUrl: './wiedervorlage-attachment-list.component.html',
	styleUrls: ['./wiedervorlage-attachment-list.component.scss']
})
export class WiedervorlageAttachmentListComponent implements OnChanges {

	@Input() wiedervorlage: WiedervorlageResource;
	@Input() deletable: boolean;

	@Output() deleteAttachment: EventEmitter<OzgFileResource> = new EventEmitter();

	readonly fileListRel = OzgFileListLinkRel;

	attachments$: Observable<OzgFileResource[]> = of([]);

	constructor(public service: WiedervorlageService) { }

	ngOnChanges(changes: SimpleChanges): void {
		if (changes.wiedervorlage && !isNil(this.wiedervorlage)) {
			this.updateAttachments();
		}
	}

	updateAttachments() {
		this.attachments$ = this.service.getAttachmentList(this.wiedervorlage).pipe(map(list => {
			return list.resource
				? getEmbeddedResource<OzgFileResource[]>(list.resource, OzgFileListLinkRel.FILE_LIST)
				: [];
		}))
	}
}