import { ComponentFixture, TestBed } from '@angular/core/testing';
import { AnhaengeComponent, AnhangListInWiedervorlageComponent } from '@goofy-client/ozg-file';
import { OzgFileResource } from '@goofy-client/ozg-file-shared';
import { createEmptyStateResource, createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { FileUploadComponent, SpinnerComponent } from '@goofy-client/ui';
import { WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { createOzgFileResource } from 'libs/ozg-file-shared/test/ozg-file';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { WiedervorlageAttachmentListComponent } from '../wiedervorlage-attachment-list/wiedervorlage-attachment-list.component';
import { WiedervorlageFormService } from '../wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';
import { WiedervorlageAttachmentsComponent } from './wiedervorlage-attachments.component';

describe('WiedervorlageAttachmentsComponent', () => {
	let component: WiedervorlageAttachmentsComponent;
	let fixture: ComponentFixture<WiedervorlageAttachmentsComponent>;

	const formService = mock(WiedervorlageFormService);
	const service = mock(WiedervorlageService);

	const file: OzgFileResource = createOzgFileResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedervorlageAttachmentsComponent,
				HasLinkPipe,
				MockComponent(WiedervorlageAttachmentListComponent),
				MockComponent(SpinnerComponent),
				MockComponent(AnhaengeComponent),
				MockComponent(FileUploadComponent),
				MockComponent(AnhangListInWiedervorlageComponent)
			],
			providers: [
				{
					provide: WiedervorlageFormService,
					useValue: formService
				},
				{
					provide: WiedervorlageService,
					useValue: service
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(WiedervorlageAttachmentsComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('upload file', () => {

		const file: File = <any>{ name: 'TestDatei' };
		const ozgFile: OzgFileResource = createOzgFileResource();

		beforeEach(() => {
			service.uploadFile.mockReturnValue(of(createStateResource(ozgFile)));
		})

		it('should call service', () => {
			component.uploadFile(<any>file)

			expect(service.uploadFile).toHaveBeenCalledWith(file);
		})

		describe('do after upload', () => {

			beforeEach(() => {
				component.addAttachment = jest.fn();
			})

			it('should emit file after successfull upload', () => {
				component.doAfterFileUpload(createStateResource(ozgFile));

				expect(component.addAttachment).toHaveBeenCalledWith(ozgFile);
			})

			it('should not emit file while stateResource is null', () => {
				component.doAfterFileUpload(createEmptyStateResource());

				expect(component.addAttachment).not.toHaveBeenCalled();
			})
		})
	})

	describe('delete attachment', () => {

		it('should call form service', () => {
			component.deleteAttachment(file);

			expect(formService.deleteAttachment).toHaveBeenCalledWith(file);
		})
	})

	describe('add attachment', () => {

		it('should call form service', () => {
			component.addAttachment(file);

			expect(formService.addAttachment).toHaveBeenCalledWith(file);
		})
	})
});
