import { Component, EventEmitter, Input, Output } from '@angular/core';
import { OzgFileResource } from '@goofy-client/ozg-file-shared';
import { createEmptyStateResource, StateResource } from '@goofy-client/tech-shared';
import { WiedervorlageLinkRel, WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { isNull } from 'lodash-es';
import { Observable, of } from 'rxjs';
import { tap } from 'rxjs/operators';
import { WiedervorlageFormService } from '../wiedervorlage-page-container/wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';

@Component({
	selector: 'goofy-client-wiedervorlage-attachments',
	templateUrl: './wiedervorlage-attachments.component.html',
	styleUrls: ['./wiedervorlage-attachments.component.scss']
})
export class WiedervorlageAttachmentsComponent {

	@Input() wiedervorlage: WiedervorlageResource;

	@Output() uploadFinish: EventEmitter<void> = new EventEmitter();

	readonly linkRel = WiedervorlageLinkRel;

	attachments$: Observable<StateResource<OzgFileResource[]>> = of(createEmptyStateResource());
	uploadInProgress$: Observable<StateResource<any>> = of(createEmptyStateResource());

	constructor(public formService: WiedervorlageFormService, private wiedervorlageService: WiedervorlageService) {
		this.attachments$ = this.formService.getAttachments();
	}

	uploadFile(file: File): void {
		this.uploadInProgress$ = this.wiedervorlageService.uploadFile(file).pipe(
			tap((stateResource: StateResource<OzgFileResource>) => this.doAfterFileUpload(stateResource)));
	}

	doAfterFileUpload(stateResource: StateResource<OzgFileResource>): void {
		if (stateResource.loaded && !isNull(stateResource.resource)) {
			this.addAttachment(stateResource.resource);
		}
	}

	addAttachment(ozgFile: OzgFileResource) {
		this.formService.addAttachment(ozgFile);
	}

	deleteAttachment(ozgFile: OzgFileResource) {
		this.formService.deleteAttachment(ozgFile);
	}
}