import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { OzgFileListLinkRel, OzgFileListResource, OzgFileResource } from '@goofy-client/ozg-file-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { getEmbeddedResource } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { map } from 'rxjs/operators';

@Component({
	selector: 'goofy-client-wiedervorlage-attachment-list-in-vorgang',
	templateUrl: './wiedervorlage-attachment-list-in-vorgang.component.html',
	styleUrls: ['./wiedervorlage-attachment-list-in-vorgang.component.scss']
})
export class WiedervorlageAttachmentListInVorgangComponent implements OnChanges {

	@Input() wiedervorlage: WiedervorlageResource;

	readonly fileListRel = OzgFileListLinkRel;

	attachments$: Observable<StateResource<OzgFileResource[]>> = of(createEmptyStateResource());

	constructor(public wiedervorlageService: WiedervorlageService) { }

	ngOnChanges(changes: SimpleChanges): void {
		if (changes.wiedervorlage) {
			this.attachments$ = this.wiedervorlageService.getAttachmentList(this.wiedervorlage).pipe(
				map(attachmentList => this.mapAttachments(attachmentList))
			);
		}
	}

	private mapAttachments(attachmentListStateResource: StateResource<OzgFileListResource>): StateResource<OzgFileResource[]> {
		return attachmentListStateResource.resource
			? createStateResource(getEmbeddedResource<OzgFileResource[]>(attachmentListStateResource.resource, OzgFileListLinkRel.FILE_LIST))
			: { ...attachmentListStateResource, resource: [] };
	}
}