import { DatePipe, registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatTooltipModule } from '@angular/material/tooltip';
import { RouterTestingModule } from '@angular/router/testing';
import { ConvertForDataTestPipe, FormatListDatePipe, HasLinkPipe, ToResourceUriPipe, ToTrafficLightTooltipPipe } from '@goofy-client/tech-shared';
import { ExpansionPanelComponent } from '@goofy-client/ui';
import { WiedervorlageResource } from '@goofy-client/wiedervorlage-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { WiedervorlageStatusComponent } from '../../../wiedervorlage-status/wiedervorlage-status.component';
import { WiedervorlageAttachmentListInVorgangComponent } from './wiedervorlage-attachment-list-in-vorgang/wiedervorlage-attachment-list-in-vorgang.component';
import { WiedervorlageInVorgangComponent } from './wiedervorlage-in-vorgang.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('WiedervorlageInVorgangComponent', () => {
	let component: WiedervorlageInVorgangComponent;
	let fixture: ComponentFixture<WiedervorlageInVorgangComponent>;

	const expandable: string = '[data-test-class="expandable"]';
	const expandButton: string = '[data-test-class="expand-button"]';
	const wiedervorlage: WiedervorlageResource = createWiedervorlageResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedervorlageInVorgangComponent,
				FormatListDatePipe,
				ToResourceUriPipe,
				ToTrafficLightTooltipPipe,
				ConvertForDataTestPipe,
				MatIcon,
				HasLinkPipe,
				MockComponent(ExpansionPanelComponent),
				MockComponent(WiedervorlageStatusComponent),
				MockComponent(WiedervorlageAttachmentListInVorgangComponent)
			],
			imports: [
				MatTooltipModule,
				RouterTestingModule
			],
			providers: [
				DatePipe,
				{ provide: LOCALE_ID, useValue: 'de' },
				{ provide: MAT_DATE_LOCALE, useValue: 'de-DE' }
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(WiedervorlageInVorgangComponent);
		component = fixture.componentInstance;
		component.wiedervorlageResource = wiedervorlage;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('tooltip should contain betreff', () => {
		expect(component.tooltip).toContain(`${wiedervorlage.betreff}, erstellt am`);
	});

	it('tooltip should contain creator', () => {
		expect(component.tooltip).toContain(`von ${wiedervorlage.createdByName}`);
	});

	it('should have data test id', () => {
		component.wiedervorlageResource = { ...createWiedervorlageResource(), betreff: 'Hier steht ein Betreff' };
		fixture.detectChanges();

		const element = fixture.nativeElement.querySelector('[data-test-id="Hier_steht_ein_Betreff"]');

		expect(element).toBeInstanceOf(HTMLElement);
	})

	describe('beschreibung', () => {
		it('should be hidden initially', () => {
			const betreff = fixture.nativeElement.querySelector(expandable);

			expect(betreff).not.toBeInstanceOf(HTMLElement);
		})

		it('should be visible on click', () => {
			const button = fixture.nativeElement.querySelector(expandButton);
			button.click();
			fixture.detectChanges();

			const betreff = fixture.nativeElement.querySelector(expandable);

			expect(betreff).toBeInstanceOf(HTMLElement);
			expect(betreff).toHaveTextContent(wiedervorlage.beschreibung);
		})
	})
});
