import { AfterViewInit, Component, ElementRef, Input, OnChanges } from '@angular/core';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { VorgangResource } from '@goofy-client/vorgang-shared';
import { WiedervorlageLinkRel, WiedervorlageListLinkRel, WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { getEmbeddedResource, hasLink } from '@ngxp/rest';
import { isNil } from 'lodash';
import { isNull } from 'lodash-es';
import { Observable } from 'rxjs';
import { filter, map, startWith } from 'rxjs/operators';

@Component({
	selector: 'goofy-client-wiedervorlage-list-in-vorgang-list-container',
	templateUrl: './wiedervorlage-list-in-vorgang-list-container.component.html',
	styleUrls: ['./wiedervorlage-list-in-vorgang-list-container.component.scss']
})
export class WiedervorlageListInVorgangListContainerComponent implements OnChanges, AfterViewInit {

	@Input() vorgang: VorgangResource;

	wiedervorlagenStateResource$: Observable<StateResource<WiedervorlageResource[]>>;

	public readonly wiedervorlageListLinkRel = WiedervorlageListLinkRel;

	constructor(
		private wiedervorlageService: WiedervorlageService,
		private elementRef: ElementRef
	) { }

	ngOnChanges(): void {
		if (!isNil(this.vorgang)) {
			this.loadWiedervorlagen();
		}
	}

	ngAfterViewInit(): void {
		if (this.elementRef.nativeElement.getBoundingClientRect().bottom >= window.innerHeight) {
			this.elementRef.nativeElement.classList.add('up');
		}
	}

	loadWiedervorlagen(): void {
		this.wiedervorlagenStateResource$ = this.wiedervorlageService.loadStrictWiedervorlagenNotDone(this.vorgang).pipe(
			filter(listStateResource => !isNull(listStateResource.resource)),
			map(listStateResource => getEmbeddedResource<WiedervorlageResource[]>(listStateResource.resource, this.wiedervorlageListLinkRel.WIEDERVORLAGE_LIST)),
			map(wiedervorlagen => createStateResource(wiedervorlagen ? wiedervorlagen.filter(wiedervorlage => hasLink(wiedervorlage, WiedervorlageLinkRel.ERLEDIGEN)) : [])),
			startWith(createEmptyStateResource<WiedervorlageResource[]>())
		);
	}
}
