import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, ReactiveFormsModule } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { ButtonWithSpinnerComponent, DateEditorComponent, TextAreaEditorComponent, TextEditorComponent } from '@goofy-client/ui';
import { WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { configureTestSuite } from 'ng-bullet';
import { MockComponent } from 'ng-mocks';
import { SubmitWiedervorlageButtonComponent } from '../../../submit-wiedervorlage-button/submit-wiedervorlage-button.component';
import { WiedervorlageAttachmentsComponent } from '../../../wiedervorlage-attachments/wiedervorlage-attachments.component';
import { WiedervorlageFormComponent } from './wiedervorlage-form.component';
import { WiedervorlageFormService } from './wiedervorlage.formservice';

describe('WiedervorlageFormComponent', () => {
	let component: WiedervorlageFormComponent;
	let fixture: ComponentFixture<WiedervorlageFormComponent>;

	const formService = new WiedervorlageFormService(new FormBuilder(), useFromMock(mock(WiedervorlageService)));
	const wiedervorlage: WiedervorlageResource = createWiedervorlageResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedervorlageFormComponent,
				MockComponent(ButtonWithSpinnerComponent),
				MockComponent(DateEditorComponent),
				MockComponent(TextEditorComponent),
				MockComponent(TextAreaEditorComponent),
				MockComponent(SubmitWiedervorlageButtonComponent),
				MockComponent(WiedervorlageAttachmentsComponent)
			],
			imports: [
				MatFormFieldModule,
				ReactiveFormsModule,
				BrowserAnimationsModule
			],
			providers: [
				{
					provide: WiedervorlageFormService,
					useValue: formService
				}
			]
		})
	});

	beforeEach(() => {
		formService.patch = jest.fn();

		fixture = TestBed.createComponent(WiedervorlageFormComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('patch wiedervorlage', () => {

		it('should NOT patch by formservice', () => {
			component.wiedervorlage = null;

			component.patchWiedervorlage();

			expect(formService.patch).not.toHaveBeenCalled();
		})

		it('should patch by formservice', () => {
			component.wiedervorlage = wiedervorlage;

			component.patchWiedervorlage();

			expect(formService.patch).toHaveBeenCalledWith(wiedervorlage);
		})
	})
});
