import { FormBuilder } from '@angular/forms';
import { createEmptyStateResource } from '@goofy-client/tech-shared';
import { mock, Mock, useFromMock } from '@goofy-client/test-utils';
import { WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { of } from 'rxjs';
import { WiedervorlageFormService } from './wiedervorlage.formservice';

describe('WiedervorlageFormService', () => {
	let formService: WiedervorlageFormService;
	let wiedervorlageService: Mock<WiedervorlageService>;

	const formBuilder: FormBuilder = new FormBuilder();
	const wiedervorlageResource: WiedervorlageResource = createWiedervorlageResource();

	beforeEach(() => {
		wiedervorlageService = mock(WiedervorlageService);

		formService = new WiedervorlageFormService(formBuilder, useFromMock(wiedervorlageService));
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	it('should have default date in 14 days', () => {
		const date = new Date();
		date.setDate(date.getDate() + 14);

		expect(formService.form.controls[WiedervorlageFormService.FIELD_FRIST].value.getDate()).toBe(date.getDate());
	})

	describe('submit', () => {

		beforeEach(() => {
			wiedervorlageService.createWiedervorlage.mockReturnValue(of(createEmptyStateResource()));
			wiedervorlageService.saveWiedervorlage.mockReturnValue(of(createEmptyStateResource()));
		})

		it('should call createWiedervorlage', () => {
			formService.source = null;

			formService.submit();

			expect(wiedervorlageService.createWiedervorlage).toHaveBeenCalled();
		})

		it('should call saveWiedervorlage', () => {
			formService.source = wiedervorlageResource;

			formService.submit();

			expect(wiedervorlageService.saveWiedervorlage).toHaveBeenCalled();
		})
	})
})
