import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ToTrafficLightPipe } from '@goofy-client/tech-shared';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { WiedervorlageStatusComponent } from './wiedervorlage-status.component';
import { MockComponent } from 'ng-mocks';
import { MatTooltip } from '@angular/material/tooltip';

const doneIcon: string = '[data-test-class="done-icon"]';

describe('WiedervorlageStatusComponent', () => {
	let component: WiedervorlageStatusComponent;
	let fixture: ComponentFixture<WiedervorlageStatusComponent>;

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedervorlageStatusComponent,
				ToTrafficLightPipe,
				MockComponent(MatTooltip),
				MatIcon
			]
		})
			.compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(WiedervorlageStatusComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('status dot', () => {
		it('should have no icon initially', () => {
			const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

			expect(doneIconElement).not.toBeInstanceOf(HTMLElement);
		})

		it('should have no icon if wiedervorlage is not done', () => {
			component.wiedervorlageResource = { ...createWiedervorlageResource(), done: false };
			fixture.detectChanges();

			const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

			expect(doneIconElement).not.toBeInstanceOf(HTMLElement);
		})

		it('should have icon if wiedervorlage is done', () => {
			component.wiedervorlageResource = { ...createWiedervorlageResource(), done: true };
			fixture.detectChanges();

			const doneIconElement = fixture.nativeElement.querySelector(doneIcon);

			expect(doneIconElement).toBeInstanceOf(HTMLElement);
		})
	})
});
