package de.itvsh.goofy;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.time.Instant;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.info.BuildProperties;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.common.user.CurrentUserService;
import de.itvsh.goofy.common.user.UserProfileController;
import de.itvsh.goofy.common.user.UserRole;
import de.itvsh.goofy.vorgang.VorgangController;

@RestController
@RequestMapping("/api")
public class RootController {

	static final String REL_VORGAENGE = "vorgaenge";
	static final String REL_SEARCH = "search";
	static final String REL_SEARCH_USER = "search-user-profiles";

	@Autowired(required = false)
	public BuildProperties buildProperties;

	@Autowired
	private CurrentUserService userService;

	@GetMapping
	public EntityModel<RootResource> getRootResource() {
		return ModelBuilder.fromEntity(new RootResource())//
				.addLink(linkTo(RootController.class).withSelfRel()) //
				.addLink(linkTo(methodOn(VorgangController.class).getVorgangListByPage(0)).withRel(REL_VORGAENGE))//
				.ifMatch(this::isEinheitlicherAnsprechpartner)
				.addLink(() -> linkTo(methodOn(VorgangController.class).searchVorgangList(0, null)).withRel(REL_SEARCH))
				.ifMatch(this::hasRoleVerwaltungUser)
				.addLink(() -> linkTo(methodOn(UserProfileController.class).findUsers(null)).withRel(REL_SEARCH_USER))
				.buildModel();
	}

	private boolean isEinheitlicherAnsprechpartner() {
		return userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
	}

	private boolean hasRoleVerwaltungUser() {
		return userService.hasRole(UserRole.VERWALTUNG_USER);
	}

	class RootResource {

		public String getVersion() {
			return buildProperties == null ? "--" : buildProperties.getVersion();
		}

		public Instant getBuildTime() {
			return buildProperties == null ? null : buildProperties.getTime();
		}

		public String getJavaVersion() {
			return System.getProperty("java.version");
		}
	}
}