package de.itvsh.goofy;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.http.CacheControl;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.resource.PathResourceResolver;

@Configuration
public class WebConfig implements WebMvcConfigurer {

	private static final String RESOURCE_LOCATION = "classpath:/META-INF/resources/";

	@Override
	public void addResourceHandlers(ResourceHandlerRegistry registry) {

		registry.addResourceHandler("/*.js", "/*.css", "/*.ttf", "/*.woff", "/*.woff2", "/*.eot",
				"/**/*.svg", "/*.svf", "/*.otf", "/*.ico", "/**/*.png")
				.addResourceLocations(RESOURCE_LOCATION)
				.setCacheControl(CacheControl.maxAge(30, TimeUnit.DAYS).cachePublic())
				.resourceChain(true)
				.addResolver(new PathResourceResolver());

		registry.addResourceHandler("/**/*")
				.addResourceLocations(RESOURCE_LOCATION)
				.setCacheControl(CacheControl.noStore())
				.setUseLastModified(false)
				.resourceChain(true)
				.addResolver(new PathResourceResolver() {
					@Override
					protected Resource getResource(String resourcePath, Resource location) throws IOException {
						Resource requestedResource = location.createRelative(resourcePath);
						return requestedResource.exists() && requestedResource.isReadable() ? requestedResource
								: super.getResource("index.html", location);
					}
				});

	}

}
