package de.itvsh.goofy.binary;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.common.file.OzgFile;

@Component
class BinaryFileModelAssembler implements RepresentationModelAssembler<OzgFile, EntityModel<OzgFile>> {

	static final String REL_DOWNLOAD = "download";

	@Override
	public EntityModel<OzgFile> toModel(OzgFile file) {
		var selfLink = linkTo(BinaryFileController.class).slash(file.getId());

		return ModelBuilder.fromEntity(file).addLink(selfLink.withSelfRel())
				.addLink(linkTo(BinaryFileController.class).slash(file.getId()).withRel(REL_DOWNLOAD))
				.buildModel();
	}

	public CollectionModel<EntityModel<OzgFile>> toCollectionModel(Stream<OzgFile> entities) {
		return CollectionModel.of(entities.map(this::toModel).collect(Collectors.toList()),
				linkTo(BinaryFileController.class).withSelfRel());
	}
}