package de.itvsh.goofy.binary;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.goofy.common.ContextService;
import de.itvsh.goofy.common.file.OzgFile;
import de.itvsh.goofy.common.file.OzgFileData;
import de.itvsh.goofy.common.file.OzgFileDataMapper;
import de.itvsh.goofy.common.file.OzgFileMapper;
import de.itvsh.ozg.pluto.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcBinaryFileDataRequest;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcBinaryFileDataResponse;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcFindFilesResponse;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcUploadRequest;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcUploadResponse;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class BinaryFileRemoteService {

	@GrpcClient("pluto")
	private BinaryFileServiceBlockingStub serviceStub;
	@Autowired
	private OzgFileMapper mapper;
	@Autowired
	private OzgFileDataMapper dataMapper;
	@Autowired
	private FileIdMapper fileIdMapper;

	@Autowired
	private ContextService contextService;

	public OzgFile getFile(FileId fileId) {
		GrpcFindFilesResponse response = serviceStub.findBinaryFiles(buildGrpcBinaryFileRequest(fileId));

		return mapper.toFile(response.getFile(0));
	}

	GrpcBinaryFilesRequest buildGrpcBinaryFileRequest(FileId fileId) {
		return GrpcBinaryFilesRequest.newBuilder()
				.addFileId(fileId.toString())
				.setContext(contextService.createCallContext())
				.build();
	}

	public FileId uploadFile(String vorgangId, String fieldName, OzgFileData file) {
		GrpcUploadRequest request = buildRequest(vorgangId, fieldName, file);

		GrpcUploadResponse response = serviceStub.uploadBinaryFile(request);

		return FileId.from(response.getFileId());
	}

	private GrpcUploadRequest buildRequest(String vorgangId, String fieldName, OzgFileData file) {
		return GrpcUploadRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setFieldName(fieldName)
				.setFile(dataMapper.toGrpcOzgFileData(file))
				.setContext(contextService.createCallContext())
				.build();
	}

	public Stream<OzgFile> getFiles(List<FileId> fileIds) {
		GrpcFindFilesResponse response = serviceStub.findBinaryFiles(buildBinaryFilesRequest(fileIds));

		return response.getFileList().stream().map(mapper::toFile);
	}

	public GrpcBinaryFilesRequest buildBinaryFilesRequest(List<FileId> fileIds) {
		return GrpcBinaryFilesRequest.newBuilder()
				.addAllFileId(mapFileIds(fileIds))
				.setContext(contextService.createCallContext())
				.build();
	}

	private List<String> mapFileIds(List<FileId> fileIds) {
		return fileIds.stream().map(fileIdMapper::toString).collect(Collectors.toList());
	}

	public OzgFileData getFileData(FileId fileId) {
		GrpcBinaryFileDataResponse response = serviceStub.getBinaryFileData(buildGrpcBinaryFileDataRequest(fileId));

		return dataMapper.toOzgFileData(response.getData());
	}

	private GrpcBinaryFileDataRequest buildGrpcBinaryFileDataRequest(FileId fileId) {
		return GrpcBinaryFileDataRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setFileId(fileIdMapper.toString(fileId))
				.build();
	}
}