package de.itvsh.goofy.common;

import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.fasterxml.jackson.annotation.JacksonAnnotationsInside;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import lombok.NoArgsConstructor;

@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.FIELD)
@Inherited

@JacksonAnnotationsInside
@JsonSerialize(using = LinkedResourceSerializer.class)
@JsonDeserialize(using = LinkedResourceDesiralizer.class)
public @interface LinkedResource {

	Class<?> controllerClass();

	Class<? extends IdExtractor<Object>> extractor() default ToStringExtractor.class;

	Class<? extends ObjectBuilder<Object>> builder() default JustIdBuilder.class;
}

@NoArgsConstructor
class ToStringExtractor implements IdExtractor<Object> {

	@Override
	public String extractId(Object object) {
		return object.toString();
	}
}

@NoArgsConstructor
class JustIdBuilder implements ObjectBuilder<Object> {

	@Override
	public Object build(Object id) {
		return id;
	}

	@Override
	public ObjectBuilder<Object> constructContextAware(BeanProperty property) {
		return new JustIdBuilder();
	}
}