package de.itvsh.goofy.common;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.lang3.reflect.ConstructorUtils;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;

import de.itvsh.goofy.common.errorhandling.TechnicalException;
import lombok.NoArgsConstructor;

@NoArgsConstructor
public class LinkedResourceSerializer extends JsonSerializer<Object> implements ContextualSerializer {

	private LinkedResource annotation;

	private LinkedResourceSerializer(LinkedResource annotation) {
		this.annotation = annotation;
	}

	@Override
	public void serialize(Object value, JsonGenerator gen, SerializerProvider serializers) throws IOException {

		if (value instanceof Collection) {
			gen.writeStartArray();
			((Collection<?>) value).forEach(val -> writeObject(gen, buildLink(val)));
			gen.writeEndArray();
		} else {
			writeObject(gen, buildLink(value));
		}
	}

	@Override
	public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
		return new LinkedResourceSerializer(property.getAnnotation(LinkedResource.class));
	}

	String buildLink(Object id) {
		return linkTo(annotation.controllerClass()).slash(getExtractor().extractId(id)).toString();
	}

	private IdExtractor<Object> getExtractor() {
		try {
			return ConstructorUtils.invokeConstructor(annotation.extractor());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating IdExtractor", e);
		}
	}

	private void writeObject(JsonGenerator gen, Object value) {
		try {
			gen.writeObject(value);
		} catch (IOException e) {
			throw new TechnicalException("Error writing String to json", e);
		}
	}

}
