package de.itvsh.goofy.common.command;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.mapstruct.Mapper;

import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBody;
import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBodyField;

@Mapper
public interface CommandBodyMapper {

	default GrpcCommandBody mapToBody(Map<String, String> bodyMap) {
		if (Objects.isNull(bodyMap))
			return GrpcCommandBody.getDefaultInstance();

		var builder = GrpcCommandBody.newBuilder();

		bodyMap.entrySet().stream()
				.map(entry -> GrpcCommandBodyField.newBuilder().setName(entry.getKey()).setValue(entry.getValue()).build())
				.forEach(builder::addField);

		return builder.build();
	}

	default List<GrpcCommandBodyField> mapToBodyFields(Map<String, String> bodyMap) {
		if (Objects.isNull(bodyMap))
			return Collections.emptyList();

		return bodyMap.entrySet().stream()
				.map(entry -> GrpcCommandBodyField.newBuilder().setName(entry.getKey()).setValue(entry.getValue()).build())
				.collect(Collectors.toList());
	}

	default Map<String, String> map(GrpcCommandBody body) {
		return body.getFieldList().stream().collect(Collectors.toMap(GrpcCommandBodyField::getName, GrpcCommandBodyField::getValue));
	}
}
