package de.itvsh.goofy.common.command;

import java.util.Optional;
import java.util.stream.Stream;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import lombok.NonNull;

@Service
@Validated
class CommandService {

	@Autowired
	private CommandRemoteService commandRemoteService;

	public Command createCommand(String vorgangId, String relationId, long version, @Valid CreateCommand command) {
		command = command.toBuilder().vorgangId(vorgangId).relationId(relationId).build();
		return commandRemoteService.createCommand(command, version);
	}

	public Command getById(String commandId) {
		return commandRemoteService.getCommand(commandId);
	}

	public Command revoke(String commandId) {
		return commandRemoteService.revokeCommand(commandId);
	}

	public boolean existsPendingCommands(String relationId) {
		return commandRemoteService.existsPendingCommands(relationId);
	}

	public Stream<Command> getPendingCommands(String vorgangId) {
		return commandRemoteService.getPendingCommands(vorgangId);
	}

	public Stream<Command> findFinishedCommandsByOrder(@NonNull String vorgangId, @NonNull CommandOrder order) {
		return commandRemoteService.findCommands(vorgangId, Optional.of(CommandStatus.FINISHED), Optional.of(order));
	}

	public Stream<Command> findErrorCommandsByOrder(@NonNull String vorgangId, @NonNull CommandOrder order) {
		return commandRemoteService.findCommands(vorgangId, Optional.of(CommandStatus.ERROR), Optional.of(order));
	}
}