package de.itvsh.goofy.common.file;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.goofy.binary.FileId;
import de.itvsh.goofy.common.ContextService;
import de.itvsh.ozg.pluto.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetAttachmentsRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetAttachmentsResponse;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetOzgFileDataRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetOzgFileDataResponse;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetRepresentationsRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetRepresentationsResponse;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class OzgFileRemoteService {

	@GrpcClient("pluto")
	private FileServiceBlockingStub fileServiceStub;

	@Autowired
	private ContextService contextService;

	@Autowired
	private OzgFileMapper fileMapper;

	@Autowired
	private OzgFileDataMapper ozgFileDataMapper;

	public Stream<OzgFile> getAttachmentsByEingang(String eingangId) {
		GrpcGetAttachmentsResponse response = fileServiceStub.getAttachments(buildGrpcGetAttachmentsRequest(eingangId));

		return response.getFileList().stream().map(fileMapper::toFile);
	}

	private GrpcGetAttachmentsRequest buildGrpcGetAttachmentsRequest(String eingangId) {
		return GrpcGetAttachmentsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setEingangId(eingangId)
				.build();
	}

	public Stream<OzgFile> getRepresentationsByEingang(String eingangId) {
		GrpcGetRepresentationsResponse response = fileServiceStub.getRepresentations(buildGrpcGetRepresentationsRequest(eingangId));

		return response.getFileList().stream().map(fileMapper::toFile);
	}

	private GrpcGetRepresentationsRequest buildGrpcGetRepresentationsRequest(String eingangId) {
		return GrpcGetRepresentationsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setEingangId(eingangId)
				.build();
	}

	public OzgFileData getOzgFileData(FileId fileId) {
		GrpcGetOzgFileDataResponse response = fileServiceStub.getFileData(buildGetOzgFileDataRequest(fileId));

		return ozgFileDataMapper.toOzgFileData(response.getFileData());
	}

	private GrpcGetOzgFileDataRequest buildGetOzgFileDataRequest(FileId fileId) {
		return GrpcGetOzgFileDataRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setFileId(fileId.toString())
				.build();
	}
}