package de.itvsh.goofy.common.user;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import org.keycloak.KeycloakPrincipal;
import org.keycloak.representations.AccessToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Service;

@Service
public class CurrentUserService {

	public boolean hasRole(String role) {
		return CurrentUserHelper.hasRole(role);
	}

	public Collection<GrantedAuthority> getAuthorities() {
		return Collections.unmodifiableCollection(new HashSet<GrantedAuthority>(CurrentUserHelper.getAuthentication().getAuthorities()));
	}

	public GoofyUser getUser() {
		AccessToken token = getCurrentSecurityToken();

		return GoofyUser.builder()
				.id(getUserId())
				.userName(token.getPreferredUsername())
				.firstName(token.getGivenName())
				.lastName(token.getFamilyName())
				.authorities(getAuthorities())
				.build();
	}

	public UserId getUserId() {
		return CurrentUserHelper.getCurrentUserId();
	}

	private AccessToken getCurrentSecurityToken() {
		KeycloakPrincipal<?> principal = (KeycloakPrincipal<?>) CurrentUserHelper.getAuthentication().getPrincipal();
		return principal.getKeycloakSecurityContext().getToken();
	}
}