package de.itvsh.goofy.kommentar;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.kommentar.KommentarCommandController.KommentarCommandByVorgangController;

@Component
class KommentarModelAssembler implements RepresentationModelAssembler<Kommentar, EntityModel<Kommentar>> {

	static final String REL_CREATE = "create-kommentar";
	static final String REL_EDIT = "edit";

	@Override
	public EntityModel<Kommentar> toModel(Kommentar kommentar) {
		var selfLink = linkTo(KommentarController.class).slash(kommentar.getId());

		var commandLink = linkTo(methodOn(KommentarCommandController.class).createCommand(null, kommentar.getId()));

		return ModelBuilder.fromEntity(kommentar).addLink(selfLink.withSelfRel())
				.addLink(commandLink.withRel(REL_EDIT))
				.buildModel();
	}

	public CollectionModel<EntityModel<Kommentar>> toCollectionModel(Stream<Kommentar> entities, String vorgangId) {
		return CollectionModel.of(entities.map(this::toModel).collect(Collectors.toList()),
				linkTo(KommentarController.class).withSelfRel(),
				linkTo(methodOn(KommentarCommandByVorgangController.class).createCommand(null, vorgangId)).withRel(REL_CREATE));
	}
}