package de.itvsh.goofy.kommentar;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.goofy.common.ContextService;
import de.itvsh.ozg.pluto.grpc.command.GrpcCreateCommandRequest;
import de.itvsh.ozg.pluto.grpc.command.GrpcOrder;
import de.itvsh.ozg.pluto.kommentar.GrpcFindKommentarsByVorgangIdRequest;
import de.itvsh.ozg.pluto.kommentar.GrpcFindKommentarsByVorgangIdResponse;
import de.itvsh.ozg.pluto.kommentar.GrpcGetKommentarRequest;
import de.itvsh.ozg.pluto.kommentar.GrpcGetKommentarResponse;
import de.itvsh.ozg.pluto.kommentar.KommentarServiceGrpc.KommentarServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class KommentarRemoteService {

	@GrpcClient("pluto")
	private KommentarServiceBlockingStub serviceStub;
	@Autowired
	private ContextService contextService;
	@Autowired
	private KommentarMapper mapper;

	public Stream<Kommentar> findByVorgangId(String vorgangId) {
		GrpcFindKommentarsByVorgangIdResponse response = serviceStub
				.findKommentarsByVorgangId(buildFindKommentarsByVorgangRequest(vorgangId));

		return response.getKommentarsList().stream().map(mapper::toKommentar);
	}

	private GrpcFindKommentarsByVorgangIdRequest buildFindKommentarsByVorgangRequest(String vorgangId) {
		return GrpcFindKommentarsByVorgangIdRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	public Kommentar getById(String kommentarId) {
		GrpcGetKommentarResponse response = serviceStub.getKommentar(buildGetKommentarRequest(kommentarId));

		return mapper.toKommentar(response.getKommentar());
	}

	private GrpcGetKommentarRequest buildGetKommentarRequest(String kommentarId) {
		return GrpcGetKommentarRequest.newBuilder().setId(kommentarId).build();
	}

	public KommentarCommand sendKommentarCommand(String relationId, KommentarCommand command) {
		var request = buildCreateCommandRequest(relationId, command);
		var response = serviceStub.createCommand(request);

		return mapper.toKommentarCommand(response.getCommand());
	}

	GrpcCreateCommandRequest buildCreateCommandRequest(String relationId, KommentarCommand command) {
		return GrpcCreateCommandRequest.newBuilder()
				.setCallContext(contextService.createCallContext())
				.setRelationId(relationId)
				.setOrder(GrpcOrder.valueOf(command.getOrder().name()))
				.setKommentar(mapper.toGrpcKommentar(command.getKommentar()))
				.build();
	}
}
