package de.itvsh.goofy.vorgang;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.tuple.Pair;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.itvsh.ozg.pluto.vorgang.GrpcEingang;
import de.itvsh.ozg.pluto.vorgang.GrpcFormData;
import de.itvsh.ozg.pluto.vorgang.GrpcFormField;
import de.itvsh.ozg.pluto.vorgang.GrpcSubForm;

@Mapper(uses = { AntragstellerMapper.class,
		EingangHeaderMapper.class, ZustaendigeStelleMapper.class }, collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
interface EingangMapper {

	@Mapping(source = "formData", target = "formData")
	Eingang fromGrpc(GrpcEingang eingang);

	default Map<String, Object> mapFormData(GrpcFormData formData) {
		Map<String, Object> result = new HashMap<>();
		result.putAll(mapSubFormFields(formData.getFieldList()));
		result.putAll(mapFormData(formData.getFormList()));
		return result;
	}

	default Map<String, Object> mapFormData(List<GrpcSubForm> subForms) {
		return subForms.stream().map(subForm -> Pair.of(subForm.getTitle(), mapSubForm(subForm)))
				.collect(Collectors.toMap(Pair::getLeft, Pair::getRight));
	}

	default Map<String, Object> mapSubForm(GrpcSubForm subForm) {
		Map<String, Object> result = new HashMap<>();
		result.putAll(mapSubFormFields(subForm.getFieldList()));
		result.putAll(mapFormData(subForm.getSubFormList()));
		return result;
	}

	default Map<String, Object> mapSubFormFields(List<GrpcFormField> fields) {
		return fields.stream().collect(Collectors.toMap(GrpcFormField::getName, GrpcFormField::getValue));
	}

}
