package de.itvsh.goofy.vorgang;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import de.itvsh.goofy.common.user.UserRole;

public interface Vorgang {

	enum VorgangStatus {
		NEU, ANGENOMMEN, VERWORFEN, IN_BEARBEITUNG, BESCHIEDEN, ABGESCHLOSSEN, WEITERGELEITET;

		private Map<String, Set<VorgangStatus>> allowedFollowStatusByRole = new HashMap<>();

		static {
			NEU.allowedFollowStatusByRole.put(UserRole.EINHEITLICHER_ANSPRECHPARTNER, Set.of(WEITERGELEITET, VERWORFEN));
			IN_BEARBEITUNG.allowedFollowStatusByRole.put(UserRole.EINHEITLICHER_ANSPRECHPARTNER, Set.of(ABGESCHLOSSEN, WEITERGELEITET));
			VERWORFEN.allowedFollowStatusByRole.put(UserRole.EINHEITLICHER_ANSPRECHPARTNER, Set.of(NEU));
			ABGESCHLOSSEN.allowedFollowStatusByRole.put(UserRole.EINHEITLICHER_ANSPRECHPARTNER, Set.of(IN_BEARBEITUNG));

			NEU.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(ANGENOMMEN, VERWORFEN));
			VERWORFEN.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(NEU));
			ANGENOMMEN.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(IN_BEARBEITUNG));
			IN_BEARBEITUNG.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(BESCHIEDEN, ANGENOMMEN));
			BESCHIEDEN.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(ABGESCHLOSSEN, IN_BEARBEITUNG));
			ABGESCHLOSSEN.allowedFollowStatusByRole.put(UserRole.VERWALTUNG_USER, Set.of(IN_BEARBEITUNG));
		}

		public boolean isTransactionAllowed(String userRole, VorgangStatus newStatus) {
			if (Objects.isNull(allowedFollowStatusByRole.get(userRole))) {
				return false;
			}
			return allowedFollowStatusByRole.get(userRole).contains(newStatus);
		}
	}

	String getId();

	long getVersion();

	VorgangStatus getStatus();

}