package de.itvsh.goofy.vorgang;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.itvsh.goofy.wiedervorlage.Wiedervorlage;
import de.itvsh.goofy.wiedervorlage.WiedervorlageController;

@RestController
@RequestMapping(VorgangController.PATH)
public class VorgangController {

	static final String PATH = "/api/vorgangs"; // NOSONAR
	static final String PARAM_SEARCH = "searchBy";
	static final String PARAM_PAGE = "page";

	static final int PAGE_SIZE = 100;

	@Autowired
	private VorgangService vorgangService;
	@Autowired
	private VorgangModelAssembler modelAssembler;
	@Autowired
	private WiedervorlageController wiedervorlageController;

	@GetMapping
	public CollectionModel<EntityModel<Vorgang>> getVorgangListByPage(@RequestParam(defaultValue = "0") int page) {
		return searchVorgangList(page, Optional.empty());
	}

	@GetMapping(params = PARAM_SEARCH)
	public CollectionModel<EntityModel<Vorgang>> searchVorgangList(@RequestParam(defaultValue = "0") int page,
			@RequestParam(required = true) Optional<String> searchBy) {
		var vorgangListResponse = vorgangService.findVorgangHeader(PAGE_SIZE, page * PAGE_SIZE, searchBy);

		return modelAssembler.toCollectionModel(vorgangListResponse.getVorgaengeHeader().stream().map(this::addNextFrist),
				hasPreviousPage(page),
				hasNextPage(page, vorgangListResponse, PAGE_SIZE));
	}

	VorgangHeader addNextFrist(VorgangHeader vorgangHeader) {
		return vorgangHeader.withNextFrist(
				wiedervorlageController.findWithNextFrist(vorgangHeader.getId()).map(Wiedervorlage::getFrist));
	}

	private Optional<Integer> hasPreviousPage(int page) {
		return page == 0 ? Optional.empty() : Optional.of(page - 1);
	}

	Optional<Integer> hasNextPage(int page, VorgangListResponse vorgangListResponse, int pageSize) {
		int lastVorgangIndex = getLastVorgangIndex(page, vorgangListResponse, pageSize);

		return lastVorgangIndex >= vorgangListResponse.getTotal() ? Optional.empty() : Optional.of(page + 1);
	}

	private int getLastVorgangIndex(int page, VorgangListResponse vorgangListResponse, int pageSize) {
		return page == 0
				? vorgangListResponse.getVorgaengeHeader().size()
				: pageSize + (page * pageSize);
	}

	@GetMapping("/{vorgangId}")
	public EntityModel<Vorgang> getVorgangWithEingang(@PathVariable String vorgangId) {
		VorgangWithEingang vorgangWithEingang = vorgangService.findVorgangWithEingang(vorgangId);

		return modelAssembler.toModel(vorgangWithEingang);
	}
}