package de.itvsh.goofy.vorgang;

import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.ozg.pluto.vorgang.GrpcFindVorgangRequest;
import de.itvsh.ozg.pluto.vorgang.GrpcFindVorgangResponse;
import de.itvsh.ozg.pluto.vorgang.GrpcFindVorgangWithEingangRequest;
import de.itvsh.ozg.pluto.vorgang.GrpcFindVorgangWithEingangResponse;
import de.itvsh.ozg.pluto.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class VorgangRemoteService {

	@GrpcClient("pluto")
	private VorgangServiceBlockingStub vorgangServiceStub;
	@Autowired
	private VorgangHeaderMapper vorgangHeaderMapper;
	@Autowired
	private VorgangWithEingangMapper vorgangWithEingangMapper;

	public VorgangListResponse findVorgaengeHeader(int limit, int offset, Optional<String> searchBy) {
		GrpcFindVorgangResponse response = vorgangServiceStub.findVorgang(buildFindVorgangRequest(limit, offset, searchBy));

		return buildVorgangHeaderListResponse(response);
	}

	private GrpcFindVorgangRequest buildFindVorgangRequest(int limit, int offset, Optional<String> searchBy) {
		var builder = GrpcFindVorgangRequest.newBuilder().setLimit(limit).setOffset(offset);

		setSearchBy(builder, searchBy);

		return builder.build();
	}

	private void setSearchBy(GrpcFindVorgangRequest.Builder builder, Optional<String> searchBy) {
		if (searchBy.isPresent()) {
			builder.setSearchBy(searchBy.get());
		}
	}

	private VorgangListResponse buildVorgangHeaderListResponse(GrpcFindVorgangResponse response) {
		return VorgangListResponse.builder()
				.total(response.getTotal())
				.vorgaengeHeader(response.getVorgangList().stream()
						.map(vorgangHeaderMapper::toVorgangHeader)
						.collect(Collectors.toList()))
				.build();
	}

	public VorgangWithEingang findVorgangWithEingang(String vorgangId) {
		GrpcFindVorgangWithEingangResponse response = vorgangServiceStub.findVorgangWithEingang(buildFindVorgangWithEingangRequest(vorgangId));

		return buildVorgangWithEingangResponse(response);
	}

	private GrpcFindVorgangWithEingangRequest buildFindVorgangWithEingangRequest(String vorgangId) {
		return GrpcFindVorgangWithEingangRequest.newBuilder().setId(vorgangId).build();
	}

	private VorgangWithEingang buildVorgangWithEingangResponse(GrpcFindVorgangWithEingangResponse response) {
		var responseEingang = response.getVorgangWithEingang();

		return vorgangWithEingangMapper.toVorgangWithEingang(responseEingang);
	}
}