package de.itvsh.goofy.vorgang;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.function.Predicate;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.attachment.AttachmentController;

@Component
class VorgangWithEingangProzessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	private static final Predicate<VorgangWithEingang> HAS_ATTACHMENTS = vorgangWithEingang -> //
	vorgangWithEingang.getEingang().getNumberOfAttachments() > 0 || vorgangWithEingang.getEingang().getNumberOfRepresentations() > 0;

	static final LinkRelation REL_ATTACHMENTS = LinkRelation.of("attachments");

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		VorgangWithEingang vorgangWithEingang = model.getContent();
		if (Objects.isNull(vorgangWithEingang)) {
			return model;
		}

		model = addAttachmentLink(model, vorgangWithEingang);

		return model;
	}

	private EntityModel<VorgangWithEingang> addAttachmentLink(EntityModel<VorgangWithEingang> model, VorgangWithEingang vorgangWithEingang) {
		if (HAS_ATTACHMENTS.test(vorgangWithEingang)) {
			return model.add(linkTo(methodOn(AttachmentController.class).getAllByEingang(vorgangWithEingang.getEingang().getId()))//
					.withRel(REL_ATTACHMENTS));
		}
		return model;
	}

}