package de.itvsh.goofy.vorgang.command;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.LinkBuilder;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.common.command.CommandController.CommandByRelationController;
import de.itvsh.goofy.common.user.CurrentUserService;
import de.itvsh.goofy.common.user.UserRole;
import de.itvsh.goofy.vorgang.Vorgang;
import de.itvsh.goofy.vorgang.Vorgang.VorgangStatus;
import de.itvsh.goofy.vorgang.VorgangWithEingang;
import de.itvsh.goofy.vorgang.forwarding.ForwardingController;

@Component
public class VorgangWithEingangCommandProzessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_VORGANG_FORWARD = LinkRelation.of("forward");
	static final LinkRelation REL_VORGANG_FORWARDING = LinkRelation.of("forwarding");

	@Autowired
	private CurrentUserService userService;
	@Autowired
	private ForwardingController forwardingController;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		return ModelBuilder.fromModel(model)
				.ifMatch(this::addForwardLink).addLink(vorgang -> buildCreateCommandLink(vorgang).withRel(REL_VORGANG_FORWARD))
				.ifMatch(this::isEinheitlicherAnsprechpartner).addLink(vorgang -> buildForwardingLink(vorgang).withRel(REL_VORGANG_FORWARDING))
				.buildModel();
	}

	boolean addForwardLink(VorgangWithEingang vorgang) {
		return userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER) && (vorgang.getStatus() == VorgangStatus.NEU ||
				(vorgang.getStatus() == VorgangStatus.IN_BEARBEITUNG && (hasForwardingFailed(vorgang))));
	}

	private boolean hasForwardingFailed(VorgangWithEingang vorgang) {
		return !forwardingController.findFailedForwardings(vorgang.getId()).isEmpty();
	}

	private LinkBuilder buildCreateCommandLink(Vorgang vorgang) {
		return linkTo(
				methodOn(CommandByRelationController.class).createCommandByRelation(vorgang.getId(), vorgang.getId(), vorgang.getVersion(), null));
	}

	private boolean isEinheitlicherAnsprechpartner(VorgangWithEingang vorgang) {
		return userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
	}

	private Link buildForwardingLink(Vorgang vorgang) {
		return linkTo(methodOn(ForwardingController.class).findByVorgangId(vorgang.getId())).withRel(REL_VORGANG_FORWARDING);
	}
}