package de.itvsh.goofy.vorgang.forwarding;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.itvsh.goofy.vorgang.forwarding.Forwarding.Status;

@RestController
@RequestMapping(ForwardingController.LIST_PATH)
public class ForwardingController {

	static final String LIST_PATH = "/api/forwardings"; // NOSONAR

	static final String PARAM_VORGANG_ID = "vorgangId";

	@Autowired
	private ForwardingService service;
	@Autowired
	private ForwardingModelAssembler modelAssembler;

	@GetMapping(params = { PARAM_VORGANG_ID })
	public CollectionModel<EntityModel<Forwarding>> findByVorgangId(@RequestParam String vorgangId) {
		return modelAssembler.toCollectionModel(findByVorgang(vorgangId));
	}

	public List<Forwarding> findFailedForwardings(String vorgangId) {
		return findByVorgang(vorgangId)
				.filter(forwarding -> forwarding.getStatus() == Status.FAILED || forwarding.getStatus() == Status.SEND_ERROR)
				.collect(Collectors.toList());
	}

	public Stream<Forwarding> findByVorgang(String vorgangId) {
		return service.findByVorgang(vorgangId);
	}
}