package de.itvsh.goofy.vorgang.forwarding;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.common.command.CommandController.CommandByRelationController;
import de.itvsh.goofy.vorgang.forwarding.Forwarding.Status;

@Component
class ForwardingModelAssembler implements RepresentationModelAssembler<Forwarding, EntityModel<Forwarding>> {

	static final LinkRelation REL_MARK_AS_SUCCESS = LinkRelation.of("mark-as-success");
	static final LinkRelation REL_MARK_AS_FAIL = LinkRelation.of("mark-as-fail");

	static final LinkRelation REL_SUCCESSFULL = LinkRelation.of("successfull");
	static final LinkRelation REL_FAILED = LinkRelation.of("failed");

	static final LinkRelation REL_ERROR = LinkRelation.of("error");

	@Override
	public EntityModel<Forwarding> toModel(Forwarding entity) {
		var selfLink = linkTo(ForwardingController.class).slash(entity.getId());

		return ModelBuilder.fromEntity(entity)
				.addLink(linkTo(ForwardingController.class).slash(entity.getId()).withSelfRel())
				.ifMatch(forwarding -> forwarding.getStatus() == Status.SENT)
				.addLinks(buildMarkAsSuccessLink(entity))
				.ifMatch(forwarding -> forwarding.getStatus() == Status.SENT || forwarding.getStatus() == Status.SUCCESSFULL)
				.addLinks(buildMarkAsFailLink(entity))
				.ifMatch(forwarding -> forwarding.getStatus() == Status.FAILED)
				.addLink(selfLink.withRel(REL_FAILED))
				.ifMatch(forwarding -> forwarding.getStatus() == Status.SUCCESSFULL)
				.addLink(selfLink.withRel(REL_SUCCESSFULL))
				.ifMatch(forwarding -> forwarding.getStatus() == Status.SEND_ERROR)
				.addLink(selfLink.withRel(REL_ERROR))
				.buildModel();
	}

	public CollectionModel<EntityModel<Forwarding>> toCollectionModel(Stream<Forwarding> entities) {
		return CollectionModel.of(entities.map(this::toModel).collect(Collectors.toList()),
				linkTo(ForwardingController.class).withSelfRel());
	}

	public Link buildMarkAsSuccessLink(Forwarding forwarding) {
		return buildLinkWithLinkRel(forwarding, REL_MARK_AS_SUCCESS);
	}

	public Link buildMarkAsFailLink(Forwarding forwarding) {
		return buildLinkWithLinkRel(forwarding, REL_MARK_AS_FAIL);
	}

	private Link buildLinkWithLinkRel(Forwarding forwarding, LinkRelation linkRel) {
		return linkTo(methodOn(CommandByRelationController.class).createCommandByRelation(forwarding.getVorgangId(), forwarding.getId(), -1, null))
				.withRel(linkRel);
	}
}