package de.itvsh.goofy.wiedervorlage;

import static de.itvsh.goofy.common.ValidationMessageCodes.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.*;

import javax.validation.constraints.FutureOrPresent;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.Singular;

import de.itvsh.goofy.binary.FileId;

@Getter
@Builder(toBuilder = true)
@NoArgsConstructor(access = AccessLevel.PRIVATE)
@AllArgsConstructor(access = AccessLevel.PACKAGE)
@ToString
public class Wiedervorlage {

	@JsonIgnore
	private String id;

	@JsonProperty(access = Access.READ_ONLY)
	private boolean done;
	@JsonIgnore
	private String createdBy;
	@JsonProperty(access = Access.READ_ONLY)
	private String createdByName;
	@JsonProperty(access = Access.READ_ONLY)
	private ZonedDateTime createdAt;

	@NotNull(message = FIELD_IS_EMPTY)
	@Size(min = 3, max = 40, message = FIELD_SIZE)
	private String betreff;
	private String beschreibung;

	@NotNull(message = FIELD_IS_EMPTY)
	@FutureOrPresent(message = FIELD_DATE_PAST)
	private LocalDate frist;
	
	@Singular
	private List<FileId> attachments;
}