package de.itvsh.goofy.wiedervorlage;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.itvsh.goofy.common.command.CommandController;

@RestController
@RequestMapping(WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS)
public class WiedervorlageCommandController {

	static final String WIEDERVORLAGE_COMMANDS = "/api/wiedervorlages/{wiedervorlageId}/commands";

	@Autowired
	private WiedervorlageService service;

	@PostMapping
	public ResponseEntity<Void> createCommand(@RequestBody WiedervorlageCommand command, @PathVariable String wiedervorlageId) {
		return buildResponseLink(service.createCommand(wiedervorlageId, command));
	}

	private ResponseEntity<Void> buildResponseLink(WiedervorlageCommand created) {
		return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
	}

	@RestController
	@RequestMapping(WiedervorlageCommandByVorgangController.WIEDERVORLAGE_COMMANDS_BY_VORGANG)
	public static class WiedervorlageCommandByVorgangController {

		static final String WIEDERVORLAGE_COMMANDS_BY_VORGANG = "/api/vorgangs/{vorgangId}/wiedervorlageCommands";

		@Autowired
		private WiedervorlageService service;

		@PostMapping
		public ResponseEntity<Void> createCommand(@RequestBody WiedervorlageCommand command, @PathVariable String vorgangId) {
			var created = service.createCreateCommand(vorgangId, command);

			return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
		}
	}
}