package de.itvsh.goofy.wiedervorlage;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.itvsh.goofy.binary.BinaryFileController;
import de.itvsh.goofy.binary.FileId;
import de.itvsh.goofy.common.file.OzgFile;

@RestController
@RequestMapping(WiedervorlageController.WIEDERVORLAGE_PATH)
public class WiedervorlageController {

	static final String WIEDERVORLAGE_PATH = "/api/wiedervorlages"; // NOSONAR
	static final String PARAM_VORGANG_ID = "vorgangId";

	@Autowired
	private WiedervorlageService service;
	@Autowired
	private WiedervorlageModelAssembler modelAssembler;

	@GetMapping("/{wiedervorlageId}")
	public EntityModel<Wiedervorlage> getById(@PathVariable String wiedervorlageId) {
		return modelAssembler.toModel(service.getById(wiedervorlageId));
	}

	@GetMapping(params = PARAM_VORGANG_ID)
	public CollectionModel<EntityModel<Wiedervorlage>> findByVorgang(@RequestParam String vorgangId) {
		return modelAssembler.toCollectionModel(service.findByVorgangId(vorgangId), vorgangId);
	}

	public Optional<Wiedervorlage> findWithNextFrist(String vorgangId) {
		return service.findWithNextFrist(vorgangId);
	}

	@RestController
	@RequestMapping(AttachmentsByWiedervorlageController.PATH)
	static class AttachmentsByWiedervorlageController {

		static final String PATH = "/api/wiedervorlages/{wiedervorlageId}"; // NOSONAR

		@Autowired
		private WiedervorlageService service;
		@Autowired
		private BinaryFileController binaryFileController;

		@GetMapping("/attachments")
		public CollectionModel<EntityModel<OzgFile>> getAttachments(@PathVariable String wiedervorlageId) {
			var wiedervorlage = service.getById(wiedervorlageId);

			return binaryFileController.getFiles(mapToId(wiedervorlage.getAttachments()));
		}

		List<FileId> mapToId(List<FileId> fileIds) {
			var fileIdsAsStringStream = fileIds.stream().map(FileId::toString);
			return fileIdsAsStringStream.map(idString -> FileId.from(idString.substring(idString.lastIndexOf("/") + 1, idString.length())))
					.collect(Collectors.toList());
		}
	}
}