package de.itvsh.goofy.wiedervorlage;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingConstants;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;
import org.mapstruct.ValueMapping;

import de.itvsh.goofy.binary.FileIdMapper;
import de.itvsh.goofy.common.IgnoreGrpcFields;
import de.itvsh.ozg.pluto.grpc.command.GrpcCommand;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcWiedervorlage;

@Mapper(uses = { FileIdMapper.class }, unmappedTargetPolicy = ReportingPolicy.WARN, //
		unmappedSourcePolicy = ReportingPolicy.WARN, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
public interface WiedervorlageMapper {

	@Mapping(target = "beschreibungBytes", ignore = true)
	@Mapping(target = "betreffBytes", ignore = true)
	@Mapping(target = "fristBytes", ignore = true)
	@Mapping(target = "version", ignore = true)
	@Mapping(target = "attachmentsList", source = "attachments")
	@IgnoreGrpcFields
	GrpcWiedervorlage toGrpcWiedervorlage(Wiedervorlage wiedervorlage);

	@Mapping(target = "attachments", source = "attachmentsList")
	Wiedervorlage toWiedervorlage(GrpcWiedervorlage grpcWiedervorlage);

	@Mapping(target = "wiedervorlage", ignore = true)
	@ValueMapping(source = "UNRECOGNIZED", target = MappingConstants.NULL)
	@ValueMapping(source = "UNDEFINED", target = MappingConstants.NULL)
	WiedervorlageCommand toWiedervorlageCommand(GrpcCommand command);
}
