package de.itvsh.goofy.wiedervorlage;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.itvsh.goofy.binary.BinaryFileController;
import de.itvsh.goofy.common.ModelBuilder;
import de.itvsh.goofy.wiedervorlage.WiedervorlageCommandController.WiedervorlageCommandByVorgangController;
import de.itvsh.goofy.wiedervorlage.WiedervorlageController.AttachmentsByWiedervorlageController;

@Component
class WiedervorlageModelAssembler implements RepresentationModelAssembler<Wiedervorlage, EntityModel<Wiedervorlage>> {

	static final String REL_EDIT = "edit";
	static final String REL_ERLEDIGEN = "erledigen";
	static final String REL_WIEDEREROEFFNEN = "wiedereroeffnen";
	static final String REL_ATTACHMENTS = "attachments";

	static final String REL_CREATE = "create-wiedervorlage";
	static final String REL_UPLOAD_FILE = "uploadFile";

	private static final Predicate<Wiedervorlage> ALLOW_ERLEDIGEN = wiedervorlage -> !wiedervorlage.isDone();
	private static final Predicate<Wiedervorlage> ALLOW_WIEDEREROEFFNEN = Wiedervorlage::isDone;
	private static final Predicate<Wiedervorlage> HAS_ATTACHMENTS = wiedervorlage -> !wiedervorlage.getAttachments().isEmpty();

	@Override
	public EntityModel<Wiedervorlage> toModel(Wiedervorlage wiedervorlage) {
		var selfLink = linkTo(WiedervorlageController.class).slash(wiedervorlage.getId());
		var commandLink = linkTo(methodOn(WiedervorlageCommandController.class).createCommand(null, wiedervorlage.getId()));

		return ModelBuilder.fromEntity(wiedervorlage).addLink(selfLink.withSelfRel())
				.addLink(commandLink.withRel(REL_EDIT))
				.ifMatch(ALLOW_ERLEDIGEN).addLink(commandLink.withRel(REL_ERLEDIGEN))
				.ifMatch(ALLOW_WIEDEREROEFFNEN).addLink(commandLink.withRel(REL_WIEDEREROEFFNEN))
				.ifMatch(HAS_ATTACHMENTS)
				.addLink(linkTo(methodOn(AttachmentsByWiedervorlageController.class).getAttachments(wiedervorlage.getId())).withRel(REL_ATTACHMENTS))
				.buildModel();
	}

	public CollectionModel<EntityModel<Wiedervorlage>> toCollectionModel(Stream<Wiedervorlage> entities, String vorgangId) {
		return CollectionModel.of(entities.map(this::toModel).collect(Collectors.toList()),
				linkTo(WiedervorlageController.class).withSelfRel(),
				linkTo(methodOn(WiedervorlageCommandByVorgangController.class).createCommand(null, vorgangId)).withRel(REL_CREATE),
				linkTo(BinaryFileController.class).slash(vorgangId).slash("wiedervorlageAttachment").slash("file").withRel(REL_UPLOAD_FILE));
	}
}