package de.itvsh.goofy.wiedervorlage;

import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.goofy.common.ContextService;
import de.itvsh.ozg.pluto.grpc.command.GrpcCreateCommandRequest;
import de.itvsh.ozg.pluto.grpc.command.GrpcOrder;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcFindWiedervorlagenByVorgangRequest;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcFindWiedervorlagenByVorgangResponse;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcGetWiedervorlageRequest;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcGetWiedervorlageResponse;
import de.itvsh.ozg.pluto.wiedervorlage.GrpcWiedervorlage;
import de.itvsh.ozg.pluto.wiedervorlage.WiedervorlageServiceGrpc.WiedervorlageServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class WiedervorlageRemoteService {

	@GrpcClient("pluto")
	private WiedervorlageServiceBlockingStub serviceStub;

	@Autowired
	private ContextService contextService;

	@Autowired
	private WiedervorlageMapper mapper;

	public Stream<Wiedervorlage> findByVorgangId(String vorgangId) {
		GrpcFindWiedervorlagenByVorgangResponse response = serviceStub
				.findWiedervorlagenByVorgangId(buildFindWiedervorlagenByVorgangRequest(vorgangId));
		List<GrpcWiedervorlage> wiedervorlagenList = response.getWiedervorlagenList();

		return wiedervorlagenList.stream().map(mapper::toWiedervorlage);
	}

	private GrpcFindWiedervorlagenByVorgangRequest buildFindWiedervorlagenByVorgangRequest(String vorgangId) {
		return GrpcFindWiedervorlagenByVorgangRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	public Wiedervorlage getById(String wiedervorlageId) {
		GrpcGetWiedervorlageResponse response = serviceStub.getWiedervorlage(buildGetWiedervorlageRequest(wiedervorlageId));

		return mapper.toWiedervorlage(response.getWiedervorlage());
	}

	private GrpcGetWiedervorlageRequest buildGetWiedervorlageRequest(String id) {
		return GrpcGetWiedervorlageRequest.newBuilder().setId(id).build();
	}

	public WiedervorlageCommand sendCreateWiedervorlageCommand(String vorgangId, WiedervorlageCommand command) {
		var result = serviceStub.createCommand(buildCreateCommandRequest(vorgangId, command));

		return mapper.toWiedervorlageCommand(result.getCommand());
	}

	public WiedervorlageCommand sendWiedervorlageCommand(String wiedervorlageId, WiedervorlageCommand command) {
		var request = buildCreateCommandRequest(wiedervorlageId, command);
		var result = serviceStub.createCommand(request);

		return mapper.toWiedervorlageCommand(result.getCommand());
	}

	GrpcCreateCommandRequest buildCreateCommandRequest(String relationId, WiedervorlageCommand command) {
		var builder = createOrderRequestBuilder(relationId, command);

		return addWiedervorlage(builder, command).build();
	}

	GrpcCreateCommandRequest.Builder addWiedervorlage(GrpcCreateCommandRequest.Builder builder, WiedervorlageCommand command) {
		if (Objects.nonNull(command.getWiedervorlage())) {
			builder.setWiedervorlage(mapper.toGrpcWiedervorlage(command.getWiedervorlage()));
		}
		return builder;
	}

	private GrpcCreateCommandRequest.Builder createOrderRequestBuilder(String relationId, WiedervorlageCommand command) {
		return GrpcCreateCommandRequest.newBuilder()
				.setCallContext(contextService.createCallContext())
				.setRelationId(relationId)
				.setOrder(GrpcOrder.valueOf(command.getOrder().name()));
	}
}