package de.itvsh.goofy.binary;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.common.ContextService;
import de.itvsh.goofy.common.GrpcCallContextTestFactory;
import de.itvsh.goofy.common.file.GrpcOzgFileDataTestFactory;
import de.itvsh.goofy.common.file.GrpcOzgFileTestFactory;
import de.itvsh.goofy.common.file.OzgFile;
import de.itvsh.goofy.common.file.OzgFileData;
import de.itvsh.goofy.common.file.OzgFileDataMapper;
import de.itvsh.goofy.common.file.OzgFileDataTestFactory;
import de.itvsh.goofy.common.file.OzgFileMapper;
import de.itvsh.goofy.common.file.OzgFileTestFactory;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;
import de.itvsh.ozg.pluto.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcBinaryFileDataResponse;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcFindFilesResponse;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcUploadResponse;
import de.itvsh.ozg.pluto.grpc.command.GrpcCallContext;
import de.itvsh.ozg.pluto.grpc.file.GrpcOzgFile;

class BinaryFileRemoteServiceTest {

	@InjectMocks
	private BinaryFileRemoteService remoteService;
	@Mock
	private BinaryFileServiceBlockingStub serviceStub;
	@Mock
	private ContextService contextService;
	@Mock
	private OzgFileMapper ozgFileMapper;
	@Mock
	private OzgFileDataMapper ozgFileDataMapper;
	@Mock
	private FileIdMapper fileIdMapper;

	private GrpcCallContext callContext = GrpcCallContextTestFactory.create();

	@BeforeEach
	void mockContextService() {
		when(contextService.createCallContext()).thenReturn(callContext);
	}

	@Nested
	class TestUploadFile {

		private final GrpcUploadResponse response = BinaryFileTestFactory.createFileUploadReponse();

		@BeforeEach
		void mock() {
			when(serviceStub.uploadBinaryFile(any())).thenReturn(response);
			when(ozgFileDataMapper.toGrpcOzgFileData(any())).thenReturn(GrpcOzgFileDataTestFactory.create());
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		private FileId doServiceCall() {
			return remoteService.uploadFile(VorgangHeaderTestFactory.ID, BinaryFileTestFactory.FIELD_NAME, OzgFileDataTestFactory.create());
		}
	}

	@Nested
	class TestGetFile {

		private final GrpcFindFilesResponse response = BinaryFileTestFactory.createFindBinaryFilesResponse();

		@BeforeEach
		void mock() {
			when(serviceStub.findBinaryFiles(any())).thenReturn(response);
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallMapper() {
			doServiceCall();

			verify(ozgFileMapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private OzgFile doServiceCall() {
			return remoteService.getFile(BinaryFileTestFactory.FILE_ID);
		}
	}

	@Nested
	class TestGetFiles {

		private final GrpcFindFilesResponse response = BinaryFileTestFactory.createFindBinaryFilesResponse();

		@BeforeEach
		void mock() {
			when(serviceStub.findBinaryFiles(any())).thenReturn(response);
			when(fileIdMapper.toString(any())).thenReturn(OzgFileTestFactory.ID.toString());
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileIdMapper() {
			doServiceCall();

			verify(fileIdMapper).toString(any());
		}

		@Test
		void shouldCallOzgFileMapper() {
			var result = doServiceCall();

			result = result.collect(Collectors.toList()).stream();

			verify(ozgFileMapper).toFile(any(GrpcOzgFile.class));
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getFiles(List.of(OzgFileTestFactory.ID));
		}
	}

	@Nested
	class TestOzgFileData {

		private final GrpcBinaryFileDataResponse response = BinaryFileTestFactory.createBinaryFileDataResponse();

		@BeforeEach
		void mock() {
			when(serviceStub.getBinaryFileData(any())).thenReturn(response);
			when(fileIdMapper.toString(any())).thenReturn(GrpcOzgFileDataTestFactory.FILE_ID);
		}

		@Test
		void shouldCallServiceStub() {
			doServiceCall();

			verify(serviceStub).getBinaryFileData(any());
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallMapper() {
			doServiceCall();

			verify(ozgFileDataMapper).toOzgFileData(GrpcOzgFileDataTestFactory.create());
		}

		private OzgFileData doServiceCall() {
			return remoteService.getFileData(BinaryFileTestFactory.FILE_ID);
		}
	}
}