package de.itvsh.goofy.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.context.ApplicationContext;

import com.thedeanda.lorem.LoremIpsum;

import de.itvsh.goofy.common.user.CurrentUserService;
import de.itvsh.goofy.common.user.UserTestFactory;
import de.itvsh.ozg.pluto.grpc.command.GrpcUser;

class ContextServiceTest {

	final String APPLICATION_NAME = LoremIpsum.getInstance().getName();

	@InjectMocks
	private ContextService service;

	@Mock
	private CurrentUserService userService;
	@Mock
	private ApplicationContext context;

	@Nested
	class TestCreateCallContest {

		@BeforeEach
		void mockContext() {
			when(context.getApplicationName()).thenReturn(APPLICATION_NAME);
		}

		@BeforeEach
		void mockUserService() {
			when(userService.getUser()).thenReturn(UserTestFactory.create());
		}

		@Test
		void shouldCallContextGetApplicationName() {
			service.createCallContext();

			verify(context).getApplicationName();
		}

		@Test
		void shouldContainClient() {
			var callContext = service.createCallContext();

			assertThat(callContext.getClient()).isEqualTo(APPLICATION_NAME);
		}

		@Test
		void shouldCallUserService() {
			service.createCallContext();

			verify(userService).getUser();
		}

		@Test
		void shoultHaveUserId() {
			var context = service.createCallContext();

			assertThat(context.getUser()).extracting(GrpcUser::getId).isEqualTo(UserTestFactory.ID.toString());
		}

		@Test
		void shouldHaveFullName() {
			var context = service.createCallContext();

			assertThat(context.getUser()).extracting(GrpcUser::getName).isEqualTo(UserTestFactory.FULLNAME);
		}

		@Test
		void shouldHaveRoles() {
			var context = service.createCallContext();

			assertThat(context.getUser().getRolesList()).hasSize(1).contains(UserTestFactory.ROLE);
		}
	}
}