package de.itvsh.goofy.common;

import static org.assertj.core.api.Assertions.*;

import java.util.regex.Pattern;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

public class RegexUtilTest {

	@Nested
	class TestValidationEMail {

		private final String REGEX = RegexUtil.VALIDATION_EMAIL;

		@Nested
		class TestEmailAt {

			@Test
			void shouldMatchOnOneAtChar() {
				shouldMatch("localpart@domain.com");
			}

			@Test
			void shouldNotMatchOnNoneAtChar() {
				shouldNotMatch("localpartdomain.com");
			}

			@Test
			void shouldNotMatchOnMultipleAtChar() {
				shouldNotMatch("localpart@@domain.com");
			}
		}

		@Nested
		class TestDomain {

			@Nested
			class TestDomainDot {

				@Test
				void shouldMatchOnOneDot() {
					shouldMatch("localpart@domain.com");
				}

				@Test
				void shouldNotMatchOnNoneDot() {
					shouldNotMatch("localpart@domaincom");
				}

				@Test
				void shouldMatchOnMultipleDots() {
					shouldNotMatch("localpart@domain...com");
				}
			}

			@Nested
			class TestDomainCharCaseSensitive {

				@Test
				void shouldMatchOnLowerCaseChar() {
					shouldMatch("localpart@domain.com");
				}

				@Test
				void shouldMatchOnUpperCaseChar() {
					shouldMatch("localpart@DOMAIN.COM");
				}
			}
		}

		@Nested
		class TestLocalPart {

			@Nested
			class TestLocalPartChar {

				@Test
				void shouldMatchOnAtLeastOneChar() {
					shouldMatch("l@domain.com");
				}

				@Test
				void shouldMatchOnNoneChar() {
					shouldNotMatch("@domain.com");
				}
			}

			@Nested
			class TestLocalPartCharCaseSensitive {

				@Test
				void shouldAllowUpperCaseChar() {
					shouldMatch("UPPER_CHAR@domain.com");
				}

				@Test
				void shouldAllowLowerCaseChar() {
					shouldMatch("lower_char@domain.com");
				}
			}
		}

		@Test
		void shouldAcceptCharInLocalPart() {
			shouldMatch("local-part-1234567890!#$%&'*+/=?^_`{|}~-.'/&%*@domain.com");
		}

		private void shouldMatch(String eMail) {
			boolean match = matchPattern(eMail);

			assertThat(match).isTrue();
		}

		private void shouldNotMatch(String eMail) {
			boolean match = matchPattern(eMail);

			assertThat(match).isFalse();
		}

		private boolean matchPattern(String eMail) {
			return Pattern.matches(REGEX, eMail);
		}
	}
}