package de.itvsh.goofy.common.command;

import static org.assertj.core.api.Assertions.*;

import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBody;
import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBodyField;

class CommandBodyMapperTest {

	private static final String FIELD = "FIELD";
	private static final String VALUE = "VALUE";

	private CommandBodyMapper mapper = Mappers.getMapper(CommandBodyMapper.class);

	@Nested
	class TestMapToBody {
		@Test
		void shouldReturnEmptyBodyForNull() {
			var result = mapper.mapToBody(null);

			assertThat(result).isNotNull();
			assertThat(result.getFieldList()).isEmpty();
		}

		@Test
		void shouldReturnBodyWithField() {
			var result = mapper.mapToBody(buildBodyMap());

			assertThat(result.getFieldList()).hasSize(1);
			assertThat(result.getFieldList().get(0).getName()).isEqualTo("FIELD");
			assertThat(result.getFieldList().get(0).getValue()).isEqualTo("VALUE");
		}

	}

	@Nested
	class TestMapToBodyFields {
		@Test
		void shouldReturnEmptyListOnNull() {
			var result = mapper.mapToBodyFields(null);

			assertThat(result).isEmpty();
		}

		@Test
		void shouldReturnField() {
			var result = mapper.mapToBodyFields(buildBodyMap());

			assertThat(result).hasSize(1);
			assertThat(result.get(0).getName()).isEqualTo(FIELD);
			assertThat(result.get(0).getValue()).isEqualTo(VALUE);
		}
	}

	private Map<String, String> buildBodyMap() {
		return Map.of(FIELD, VALUE);
	}

	@Nested
	class TestMap {
		@Test
		void shouldReturnMap() {
			var result = mapper
					.map(GrpcCommandBody.newBuilder().addField(GrpcCommandBodyField.newBuilder().setName(FIELD).setValue(VALUE).build()).build());

			assertThat(result).hasSize(1).containsEntry(FIELD, VALUE);
		}
	}
}
