package de.itvsh.goofy.common.command;

import static de.itvsh.goofy.TestUtils.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import de.itvsh.goofy.common.user.UserTestFactory;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

@AutoConfigureMockMvc
@SpringBootTest

public class CommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private CommandRemoteService commandRemoteService;

	@Nested
	@WithMockUser
	class TestAssignUser {

		@Captor
		private ArgumentCaptor<CreateCommand> commandCaptor;

		@BeforeEach
		void initTests() {
			when(commandRemoteService.createCommand(any(), anyLong())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldExtractUserId() throws Exception {
			createCommand();

			verify(commandRemoteService).createCommand(commandCaptor.capture(), anyLong());
			assertThat(commandCaptor.getValue().getBody()).containsEntry("assignedTo", UserTestFactory.ID.toString());
		}

		private void createCommand() throws Exception {
			String content = loadTextFile("jsonTemplates/command/createCommandWithBody.json.tmpl", CommandOrder.ASSIGN_USER.name(),
					loadTextFile("jsonTemplates/command/commandAssignedToBody", addQuote("/api/users/" + UserTestFactory.ID.toString())));
			String url = String.format("/api/vorgangs/%s/relations/%s/%d/commands", VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.ID,
					VorgangHeaderTestFactory.VERSION);

			mockMvc.perform(post(url).content(content).contentType(MediaType.APPLICATION_JSON))
					.andExpect(status().isCreated());

		}
	}
}
