package de.itvsh.goofy.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class CommandServiceTest {

	@InjectMocks
	private CommandService service;

	@Mock
	private CommandRemoteService remoteService;

	@Captor
	private ArgumentCaptor<CreateCommand> vorgangCommandCaptor;

	@Nested
	class TestCreateVorgangCommand {

		@Test
		void shouldCallRemoteServiceWithVersion() {
			service.createCommand(CommandTestFactory.VORGANG_ID, CommandTestFactory.RELATION_ID, CommandTestFactory.VERSION,
					CommandTestFactory.createCreateCommand());

			verify(remoteService).createCommand(any(), eq(CommandTestFactory.VERSION));
		}

		@Test
		void shouldHaveVorgangId() {
			CreateCommand command = CommandTestFactory.createCreateCommand();

			service.createCommand(CommandTestFactory.VORGANG_ID, CommandTestFactory.RELATION_ID, CommandTestFactory.VERSION, command);

			verify(remoteService).createCommand(vorgangCommandCaptor.capture(), anyLong());
			assertThat(vorgangCommandCaptor.getValue().getVorgangId()).isEqualTo(CommandTestFactory.VORGANG_ID);
		}

		@Test
		void shouldHaveRelationId() {
			CreateCommand command = CommandTestFactory.createCreateCommand();

			service.createCommand(CommandTestFactory.VORGANG_ID, CommandTestFactory.RELATION_ID, CommandTestFactory.VERSION, command);

			verify(remoteService).createCommand(vorgangCommandCaptor.capture(), anyLong());
			assertThat(vorgangCommandCaptor.getValue().getRelationId()).isEqualTo(CommandTestFactory.RELATION_ID);
		}
	}

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(CommandTestFactory.RELATION_ID);

			verify(remoteService).getCommand(any());
		}
	}

	@Nested
	class TestRevoke {

		@Test
		void shouldCallRemoteService() {
			service.revoke(CommandTestFactory.RELATION_ID);

			verify(remoteService).revokeCommand(CommandTestFactory.RELATION_ID);
		}
	}

	@Nested
	class TestExistsPendingCommands {

		@Test
		void shouldCallRemoteService() {
			service.existsPendingCommands(VorgangHeaderTestFactory.ID);

			verify(remoteService).existsPendingCommands(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestGetPendingCommands {

		@Test
		void shouldCallRemoteService() {
			service.getPendingCommands(VorgangHeaderTestFactory.ID);

			verify(remoteService).getPendingCommands(VorgangHeaderTestFactory.ID);
		}
	}
}