package de.itvsh.goofy.common.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;

import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Path;

import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;

class ExceptionControllerTest {

	@InjectMocks
	private ExceptionController exceptionController;

	@Test
	void handleResourceNotFoundException() {
		ApiError response = exceptionController.handleResourceNotFoundException(new ResourceNotFoundException(String.class, 42L));

		assertThat(response.getIssues()).hasSize(1);
		assertThat(response.getIssues().get(0).getMessageCode()).isEqualTo("resource.not_found");
		assertThat(response.getIssues().get(0).getMessage()).isEqualTo("EntityModel 'String' with id '42' not found.");
	}

	@Test
	void handleFieldInValidationException() {
		ApiError error = exceptionController
				.handleConstaintViolationException(new ConstraintViolationException(Collections.singleton(mockViolation())));

		assertThat(error.getIssues()).hasSize(1);
		Issue issue = error.getIssues().get(0);
		assertThat(issue.getField()).isEqualTo("command.wiedervorlage.betreff");
	}

	@Test
	void handleMessageCodeInValidationException() {
		ApiError error = exceptionController
				.handleConstaintViolationException(new ConstraintViolationException(Collections.singleton(mockViolation())));

		assertThat(error.getIssues()).hasSize(1);
		Issue issue = error.getIssues().get(0);
		assertThat(issue.getMessageCode()).isEqualTo("huhu.code");
	}

	private ConstraintViolation<?> mockViolation() {
		ConstraintViolation<?> violation = mock(ConstraintViolation.class);
		Path path = mock(Path.class);
		when(violation.getPropertyPath()).thenReturn(path);
		when(path.toString()).thenReturn("createCommandByRelation.command.wiedervorlage.betreff");
		when(violation.getMessageTemplate()).thenReturn("{huhu.code}");

		return violation;
	}
}