package de.itvsh.goofy.common.file;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.http.HttpHeaders;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

class OzgFileDataControllerTest {

	private final String PATH = OzgFileDataController.OZG_FILE_PATH + "/";

	@InjectMocks
	private OzgFileDataController controller;

	@Mock
	private OzgFileService service;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetFileData {

		@BeforeEach
		void mockServiceReturnValue() {
			when(service.getOzgFileData(any())).thenReturn(OzgFileDataTestFactory.create());
		}

		@Test
		void shouldCallEndpoint() throws Exception {
			callEndpoint();
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getOzgFileData(OzgFileTestFactory.ID);
		}

		@Nested
		class TestHeader {

			ResultActions result = null;

			@BeforeEach
			void doCall() throws Exception {
				result = callEndpoint();
			}

			@Test
			void shouldHaveContentDisposition() throws Exception {
				result.andExpect(header().string(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + OzgFileDataTestFactory.NAME));
			}

			@Test
			void shouldHaveContentType() throws Exception {
				result.andExpect(header().string(HttpHeaders.CONTENT_TYPE, OzgFileDataTestFactory.CONTENT_TYPE_STR));
			}

			@Test
			void shouldHaveContentLength() throws Exception {
				result.andExpect(header().longValue(HttpHeaders.CONTENT_LENGTH, OzgFileDataTestFactory.SIZE));
			}
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(PATH + OzgFileTestFactory.ID)).andExpect(status().isOk());
		}
	}
}