package de.itvsh.goofy.common.file;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.attachment.GrpcGetAttachmentsResponseTestFactory;
import de.itvsh.goofy.common.ContextService;
import de.itvsh.goofy.common.GrpcCallContextTestFactory;
import de.itvsh.goofy.representation.GrpcGetRepresentationsResponseTestFactory;
import de.itvsh.goofy.vorgang.EingangTestFactory;
import de.itvsh.ozg.pluto.grpc.command.GrpcCallContext;
import de.itvsh.ozg.pluto.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetAttachmentsRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetAttachmentsResponse;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetOzgFileDataRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetOzgFileDataResponse;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetRepresentationsRequest;
import de.itvsh.ozg.pluto.grpc.file.GrpcGetRepresentationsResponse;

class OzgFileRemoteServiceTest {

	@InjectMocks
	private OzgFileRemoteService remoteService;
	@Mock
	private FileServiceBlockingStub serviceStub;
	@Mock
	private OzgFileMapper mapper;
	@Mock
	private OzgFileDataMapper ozgFileDataMapper;
	@Mock
	private ContextService contextService;

	private GrpcCallContext callContext = GrpcCallContextTestFactory.create();

	@BeforeEach
	void mockContextService() {
		when(contextService.createCallContext()).thenReturn(callContext);
	}

	@Nested
	class TestGetAttachmentsByEingang {

		private final GrpcGetAttachmentsResponse response = GrpcGetAttachmentsResponseTestFactory.create();

		@BeforeEach
		void mock() {
			when(serviceStub.getAttachments(any())).thenReturn(response);
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileStub() {
			doServiceCall();

			verify(serviceStub).getAttachments(any(GrpcGetAttachmentsRequest.class));
		}

		@Test
		void shouldCallMapper() {
			var fileStream = doServiceCall();
			fileStream = forceActionsDone(fileStream);

			verify(mapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private Stream<OzgFile> forceActionsDone(Stream<OzgFile> stream) {
			return stream.collect(Collectors.toList()).stream();
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getAttachmentsByEingang(EingangTestFactory.ID);
		}
	}

	@Nested
	class TestGetRepresentationsByEingang {

		private final GrpcGetRepresentationsResponse response = GrpcGetRepresentationsResponseTestFactory.create();

		@BeforeEach
		void mock() {
			when(serviceStub.getRepresentations(any())).thenReturn(response);
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileStub() {
			doServiceCall();

			verify(serviceStub).getRepresentations(any(GrpcGetRepresentationsRequest.class));
		}

		@Test
		void shouldCallMapper() {
			var fileStream = doServiceCall();
			fileStream = forceActionsDone(fileStream);

			verify(mapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private Stream<OzgFile> forceActionsDone(Stream<OzgFile> stream) {
			return stream.collect(Collectors.toList()).stream();
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getRepresentationsByEingang(EingangTestFactory.ID);
		}
	}

	@Nested
	class TestGetOzgFileData {

		private final GrpcGetOzgFileDataResponse response = GrpcOzgFileDataTestFactory.createReponse();

		@BeforeEach
		void mock() {
			when(serviceStub.getFileData(any())).thenReturn(response);
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileStub() {
			doServiceCall();

			verify(serviceStub).getFileData(any(GrpcGetOzgFileDataRequest.class));
		}

		@Test
		void shouldCallMapper() {
			doServiceCall();

			verify(ozgFileDataMapper).toOzgFileData(GrpcOzgFileDataTestFactory.create());
		}

		private OzgFileData doServiceCall() {
			return remoteService.getOzgFileData(OzgFileTestFactory.ID);
		}
	}
}