package de.itvsh.goofy.common.user;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

class UserProfileControllerTest {

	@InjectMocks
	private UserProfileController controller;

	@Mock
	private UserProfileService service;
	@Mock
	private UserProfileModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class findUsersTest {

		private static final String SEARCH_BY = "theo";

		private Stream<UserProfile> profiles = Stream.of(UserProfileTestFactory.create());

		@SuppressWarnings("unchecked")
		@BeforeEach
		void init() {
			when(service.findUsers(anyString())).thenReturn(profiles);

			when(modelAssembler.toCollectionModel(Mockito.<Stream<UserProfile>>any()))
					.then(i -> CollectionModel.of(
							((Stream<UserProfile>) i.getArgument(0)).map(profile -> EntityModel.of(profile)).collect(Collectors.toList())));
		}

		@Test
		void shouldReturnOk() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).findUsers(anyString());
		}

		@Test
		void shouldCallModellAssembler() throws Exception {
			doRequest();

			verify(modelAssembler).toCollectionModel(profiles);
		}

		@Test
		void shouldReturnProfiles() throws Exception {
			doRequest()
					.andExpect(jsonPath("content.length()").value(1));
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(UserProfileController.BASE_PATH).param(UserProfileController.PARAM_SEARCH_BY, SEARCH_BY))
					.andExpect(status().is2xxSuccessful());
		}
	}

	@Nested
	class getUserTest {

		static final String SINGLE_RESOURCE_URL = UserProfileController.BASE_PATH + "/{id}";

		private UserProfile userProfile = UserProfileTestFactory.create();

		@BeforeEach
		void init() {
			when(service.getUser(any())).thenReturn(userProfile);

			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));
		}

		@Test
		void shouldReturnOk() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).getUser(UserProfileTestFactory.ID);
		}

		@Test
		void shouldCallModellAssembler() throws Exception {
			doRequest();

			verify(modelAssembler).toModel(userProfile);
		}

		@Test
		void shouldReturnProfile() throws Exception {
			doRequest()
					.andExpect(jsonPath("firstName").value(UserProfileTestFactory.FIRST_NAME));

		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(SINGLE_RESOURCE_URL, UserProfileTestFactory.ID)).andExpect(status().is2xxSuccessful());
		}
	}

}
