package de.itvsh.goofy.common.user;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.TestUtils;
import de.itvsh.goofy.common.errorhandling.ResourceNotFoundException;

class UserProfileServiceTest {

	@InjectMocks
	private UserProfileService service;
	@Mock
	private UserRemoteService userRemoteService;
	@Mock
	private UserProfileMapper mapper;

	@Nested
	class TestFindUserProfiles {

		private final String SEARCH_BY = "TEST";
		private final GoofyUser user = GoofyUserTestFactory.create();

		@BeforeEach
		void initTest() {
			when(userRemoteService.findUserProfile(anyString(), anyInt())).thenReturn(Stream.of(user));
		}

		@Test
		void shouldCallRemoteService() {
			service.findUsers(SEARCH_BY);

			verify(userRemoteService).findUserProfile(SEARCH_BY, UserProfileService.DEFAULT_LIMIT);
		}

		@Test
		void shouldCallMapper() {
			service.findUsers(SEARCH_BY).forEach(TestUtils.DUMMY_CONSUMER);

			verify(mapper).from(user);
		}
	}

	@Nested
	class TestGetUser {

		private final GoofyUser user = GoofyUserTestFactory.create();

		@BeforeEach
		void initTest() {
			when(userRemoteService.findUser(any())).thenReturn(Optional.of(user));
		}

		@Test
		void shouldCallRemoteService() {
			when(mapper.from(any())).thenReturn(UserProfileTestFactory.create());

			service.getUser(UserTestFactory.ID);

			verify(userRemoteService).findUser(UserTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			when(mapper.from(any())).thenReturn(UserProfileTestFactory.create());

			service.getUser(UserTestFactory.ID);

			verify(mapper).from(user);
		}

		@Test
		void shouldThrowExceptionIfNotFound() {
			when(userRemoteService.findUser(any())).thenReturn(Optional.empty());

			assertThrows(ResourceNotFoundException.class, () -> service.getUser(UserTestFactory.ID));
		}
	}

}
