package de.itvsh.goofy.kommentar;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.goofy.kommentar.KommentarController.KommentarByVorgangController;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class KommentarByVorgangControllerTest {

	@InjectMocks
	private KommentarByVorgangController controller;

	@Mock
	private KommentarService service;

	@Mock
	private KommentarModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initMockMvc() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetAll {

		private final Stream<Kommentar> kommentars = Stream.of(KommentarTestFactory.create());

		@BeforeEach
		void mockService() {
			when(service.findByVorgangId(any())).thenReturn(kommentars);
		}

		@Test
		void shouldCallEndpoint() throws Exception {
			ResultActions callEndpoint = callEndpoint();

			callEndpoint.andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(kommentars, VorgangHeaderTestFactory.ID);
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(KommentarByVorgangController.KOMMENTAR_BY_VORGANG_PATH, VorgangHeaderTestFactory.ID));
		}
	}
}