package de.itvsh.goofy.kommentar;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

class KommentarControllerTest {

	private final String PATH = KommentarController.KOMMENTAR_PATH + "/{id}";

	@InjectMocks
	private KommentarController controller;

	@Mock
	private KommentarService service;

	@Mock
	private KommentarModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetById {

		private Kommentar kommentar = KommentarTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.getById(any())).thenReturn(kommentar);
		}

		@Test
		void shouldReturnValue() throws Exception {
			ResultActions callEndpoint = callEndpoint();

			callEndpoint.andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getById(KommentarTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toModel(kommentar);
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(PATH, KommentarTestFactory.ID));
		}
	}
}