package de.itvsh.goofy.kommentar;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

class KommentarMapperTest {

	private KommentarMapper mapper = Mappers.getMapper(KommentarMapper.class);

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMap() {
			var kommentar = mapper.toKommentar(GrpcKommentarTestFactory.create());

			assertThat(kommentar).isNotNull().usingRecursiveComparison().isEqualTo(KommentarTestFactory.create());
		}

		@Test
		void shouldHandleNull() {
			var grpcKommentar = mapper.toKommentar(null);

			assertThat(grpcKommentar).isNull();
		}
	}

	@Nested
	class TestToGrpc {

		@Test
		void shouldMap() {
			var grpcKommentar = mapper.toGrpcKommentar(KommentarTestFactory.createBuilder().id(null).build());

			assertThat(grpcKommentar).isNotNull().usingRecursiveComparison().ignoringFields("id_", "memoizedHashCode")
					.isEqualTo(GrpcKommentarTestFactory.create());
		}

		@Test
		void shouldHandleNull() {
			var grpcKommentar = mapper.toGrpcKommentar(null);

			assertThat(grpcKommentar).isNull();
		}
	}
}
