package de.itvsh.goofy.kommentar;

import static org.assertj.core.api.Assertions.*;

import java.util.Collections;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.server.EntityLinks;

import de.itvsh.goofy.common.user.UserProfileTestFactory;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class KommentarModelAssemblerTest {

	private final String PATH = KommentarController.KOMMENTAR_PATH + "/";
	private static final String REL_CREATED_BY = "createdBy";

	@InjectMocks
	private KommentarModelAssembler modelAssembler;

	@Mock
	private EntityLinks entityLinks;

	@Nested
	class TestLinksOnModel {

		final String COMMAND_BY_KOMMENTAR_PATH = //
				KommentarCommandController.KOMMENTAR_COMMANDS.replace("{kommentarId}", KommentarTestFactory.ID);

		@Test
		void shouldHaveSelfLink() {
			var link = toModel().getLink(IanaLinkRelations.SELF);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(PATH + KommentarTestFactory.ID);
		}

		@Test
		void shouldHaveEditLink() {
			var link = toModel();

			assertThat(link.getLink(KommentarModelAssembler.REL_EDIT)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(COMMAND_BY_KOMMENTAR_PATH);
		}

		@Test
		void shouldHaveCreatedByLink() {
			var link = toModel();

			assertThat(link.getLink(REL_CREATED_BY)).isPresent().get().extracting(Link::getHref)
					.isEqualTo("/api/userProfiles/" + UserProfileTestFactory.ID);
		}

		EntityModel<Kommentar> toModel() {
			return modelAssembler.toModel(KommentarTestFactory.create());
		}
	}

	@Nested
	class TestLinksOnCollectionModel {

		@Test
		void shouldHaveCreateKommentarLink() {
			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(KommentarTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);

			var link = collectionModel.getLink(KommentarModelAssembler.REL_CREATE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo("/api/vorgangs/" + VorgangHeaderTestFactory.ID + "/kommentarCommands");
		}
	}
}