package de.itvsh.goofy.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class KommentarServiceTest {

	@InjectMocks
	private KommentarService service;

	@Mock
	private KommentarRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(KommentarTestFactory.ID);

			verify(remoteService).getById(KommentarTestFactory.ID);
		}
	}

	@Nested
	class TestFindByVorgangId {

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(remoteService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveOrderByCreatedAtReverse() {
			Kommentar newestKommentar = KommentarTestFactory.createBuilder().id(UUID.randomUUID().toString()).createdAt(ZonedDateTime.now()).build();
			List<Kommentar> list = List.of(KommentarTestFactory.create(), newestKommentar);

			when(service.findByVorgangId(any())).thenReturn(list.stream());

			var kommentarList = service.findByVorgangId(VorgangHeaderTestFactory.ID).collect(Collectors.toList());
			assertThat(kommentarList.get(0).getId()).isEqualTo(newestKommentar.getId());
			assertThat(kommentarList.get(1).getId()).isEqualTo(KommentarTestFactory.ID);
		}
	}

	@Nested
	class TestCreateCommand {

		@Test
		void shouldCallRemoteService() {
			KommentarCommand command = KommentarCommandTestFactory.create();

			service.createCommand(VorgangHeaderTestFactory.ID, command);

			verify(remoteService).sendKommentarCommand(VorgangHeaderTestFactory.ID, command);
		}

		@Test
		void shouldReturnCorrectResult() {
			KommentarCommand resultFromRemoteService = KommentarCommandTestFactory.create();
			when(remoteService.sendKommentarCommand(anyString(), any())).thenReturn(resultFromRemoteService);

			KommentarCommand resultFromService = service.createCommand(VorgangHeaderTestFactory.ID, KommentarCommandTestFactory.create());

			assertThat(resultFromService).isSameAs(resultFromRemoteService);
		}
	}
}
