package de.itvsh.goofy.representation;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.goofy.common.file.OzgFile;
import de.itvsh.goofy.common.file.OzgFileModelAssembler;
import de.itvsh.goofy.common.file.OzgFileService;
import de.itvsh.goofy.common.file.OzgFileTestFactory;

class RepresentationControllerTest {

	private final String PATH = RepresentationController.REPRESENTATIONS_PATH;

	@InjectMocks
	private RepresentationController controller;
	@Mock
	private OzgFileModelAssembler modelAssembler;
	@Mock
	private OzgFileService fileService;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetRepresentationsByEingang {

		@BeforeEach
		void mockFileService() {
			when(fileService.getRepresentationsByEingang(anyString())).thenReturn(Stream.of(OzgFileTestFactory.create()));
		}

		@Test
		void shouldCallEndpoint() throws Exception {
			callEndpoint();
		}

		@Test
		void shoudlCallFileService() throws Exception {
			callEndpoint();

			verify(fileService).getRepresentationsByEingang(any());
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(ArgumentMatchers.<Stream<OzgFile>>any());
		}

		private void callEndpoint() throws Exception {
			mockMvc.perform(get(PATH).param(RepresentationController.PARAM_EINGANG_ID, "1")).andExpect(status().isOk());
		}
	}
}