package de.itvsh.goofy.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.ozg.pluto.vorgang.GrpcFindVorgangResponse;
import de.itvsh.ozg.pluto.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;

class VorgangRemoteServiceTest {

	@InjectMocks // NOSONAR
	private VorgangRemoteService service;

	@Mock
	private VorgangServiceBlockingStub serviceStub;
	@Mock
	private VorgangHeaderMapper vorgangHeaderMapper;
	@Mock
	private VorgangWithEingangMapper vorgangWithEingangMapper;

	@Nested
	class TestFindVorgaengeHeader {

		final GrpcFindVorgangResponse findVorgangRespone = GrpcVorgangTestFactory.createVorgangResponse();
		final VorgangHeader vorgangHeader = VorgangHeaderTestFactory.create();

		@BeforeEach
		void initMockReturnValues() {
			when(serviceStub.findVorgang(any())).thenReturn(findVorgangRespone);
			when(vorgangHeaderMapper.toVorgangHeader(any())).thenReturn(vorgangHeader);
		}

		@Test
		void shouldCallStub() {
			callService();

			verify(serviceStub).findVorgang(any());
		}

		@Test
		void shouldCallMapper() {
			callService();

			verify(vorgangHeaderMapper).toVorgangHeader(any());
		}

		@Test
		void shouldReturnMappedVorgangHeader() {
			var vorgangResponse = callService();

			assertThat(vorgangResponse.getVorgaengeHeader()).hasSize(1);
			assertThat(vorgangResponse.getVorgaengeHeader().get(0)).isEqualTo(vorgangHeader);
		}

		@Test
		void shouldReturnTotal() {
			var vorgangResponse = callService();

			assertThat(vorgangResponse.getTotal()).isEqualTo(findVorgangRespone.getTotal());
		}

		private VorgangListResponse callService() {
			return service.findVorgaengeHeader(2, 0, Optional.empty());
		}
	}

	@Nested
	class TestFindVorgangWithEingang {

		private final VorgangWithEingang vorgangWithEingang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void mockReturnValue() {
			when(serviceStub.findVorgangWithEingang(any())).thenReturn(GrpcVorgangWthEingangTestFactory.createVorgangWithEingangResponse());
			when(vorgangWithEingangMapper.toVorgangWithEingang(any())).thenReturn(vorgangWithEingang);
		}

		@Test
		void shouldCallStub() {
			callService();

			verify(serviceStub).findVorgangWithEingang(any());
		}

		@Test
		void shouldCallMapper() {
			callService();

			verify(vorgangWithEingangMapper).toVorgangWithEingang(any());
		}

		@Test
		void shouldReturn() {
			var result = callService();

			assertThat(result).isEqualTo(vorgangWithEingang);
		}

		private VorgangWithEingang callService() {
			return service.findVorgangWithEingang(VorgangHeaderTestFactory.ID);
		}
	}
}