package de.itvsh.goofy.vorgang;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;

//TODO refactor to unit test or rename to itcase
@SpringBootTest
@AutoConfigureMockMvc
class VorgangWithEingangProzessorTest {

	@InjectMocks
	private VorgangWithEingangProzessor processor;

	@Nested
	@DisplayName("add 'attachments' link")
	class TestAttachmentsLink {

		private final LinkRelation linkRel = VorgangWithEingangProzessor.REL_ATTACHMENTS;
		private final String BASE_PATH = "http://localhost/api/attachments";

		@DisplayName("should be present on numberOfAttachments > 0")
		@Test
		void shouldBePresentByAttachments() {
			var processed = processor.process(buildModelWithAttachments(1));

			verifyLinkHref(processed, linkRel, BASE_PATH + "?eingangId=" + EingangTestFactory.ID);
		}

		@DisplayName("should be present on numberOfRepresentations > 0")
		@Test
		void shouldBePresentByRepresentations() {
			var processed = processor.process(buildModelWithRepresentation(1));

			verifyLinkHref(processed, linkRel, BASE_PATH + "?eingangId=" + EingangTestFactory.ID);
		}

		@Test
		void shouldNotBePresent() {
			var processed = processor.process(buildModelWithAttachments(0));

			assertThat(processed.getLink(linkRel)).isNotPresent();
		}

		private EntityModel<VorgangWithEingang> buildModelWithRepresentation(int numberOfRepresentations) {
			return buildModelWithEingang(EingangTestFactory.createBuilder()
					.numberOfRepresentations(numberOfRepresentations)
					.build());
		}

		private EntityModel<VorgangWithEingang> buildModelWithAttachments(int numberOfAttachments) {
			return buildModelWithEingang(EingangTestFactory.createBuilder()
					.numberOfAttachments(numberOfAttachments)
					.build());
		}
	}

	private EntityModel<VorgangWithEingang> buildModelWithEingang(Eingang eingang) {
		return EntityModel.of(VorgangWithEingangTestFactory.createBuilder().eingang(eingang).build());
	}

	private void verifyLinkHref(EntityModel<VorgangWithEingang> processedEntity, LinkRelation linkRel, String expectedPath) {
		assertThat(processedEntity.getLink(linkRel)).isPresent().get().extracting(Link::getHref).isEqualTo(expectedPath);
	}
}