package de.itvsh.goofy.vorgang.forwarding;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class ForwardingControllerTest {

	@InjectMocks
	private ForwardingController controller;
	@Mock
	private ForwardingService service;
	@Mock
	private ForwardingModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestFindByVorgang {
		@Test
		void shouldReturn200() throws Exception {
			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).findByVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			doRequest();

			verify(modelAssembler).toCollectionModel(Mockito.<Stream<Forwarding>>any());
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(ForwardingController.LIST_PATH).param(ForwardingController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID));
		}
	}

}
