package de.itvsh.goofy.vorgang.forwarding;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.common.ContextService;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;
import de.itvsh.ozg.pluto.forwarding.ForwardingServiceGrpc.ForwardingServiceBlockingStub;
import de.itvsh.ozg.pluto.forwarding.GrpcFindForwardingsRequest;
import de.itvsh.ozg.pluto.forwarding.GrpcForwarding;
import de.itvsh.ozg.pluto.forwarding.GrpcForwardingsResponse;
import de.itvsh.ozg.pluto.grpc.command.GrpcCallContext;

class ForwardingRemoteServiceTest {

	@InjectMocks // NOSONAR
	private ForwardingRemoteService service;

	@Mock
	private ForwardingServiceBlockingStub serviceStub;
	@Mock
	private ForwardingMapper mapper;

	@Mock
	private ContextService contextService;

	@Captor
	private ArgumentCaptor<GrpcFindForwardingsRequest> requestCaptor;

	private GrpcCallContext callContext = GrpcCallContext.newBuilder().build();

	@BeforeEach
	void initContextService() {
		when(contextService.createCallContext()).thenReturn(callContext);
	}

	@Nested
	class TestFindForwardings {
		private GrpcForwarding grpcForwarding = GrpcForwarding.newBuilder().build();
		private GrpcForwardingsResponse response = GrpcForwardingsResponse.newBuilder().addForwardings(grpcForwarding).build();

		@BeforeEach
		void initStub() {
			when(serviceStub.findForwardings(any())).thenReturn(response);
		}

		@Test
		void shouldCallStub() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(any());
		}

		@Test
		void shouldFillCallContext() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getContext()).isSameAs(callContext);
		}

		@Test
		void shouldHaveVorgangId() {
			service.findForwardings(VorgangHeaderTestFactory.ID);

			verify(serviceStub).findForwardings(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallMapper() {
			service.findForwardings(VorgangHeaderTestFactory.ID).forEach(v -> {
			});

			verify(mapper).fromGrpc(grpcForwarding);
		}
	}

}
