package de.itvsh.goofy.wiedervorlage;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.goofy.binary.BinaryFileController;
import de.itvsh.goofy.binary.FileId;
import de.itvsh.goofy.wiedervorlage.WiedervorlageController.AttachmentsByWiedervorlageController;

class AttachmentsByWiedervorlageControllerTest {

	private final String PATH = AttachmentsByWiedervorlageController.PATH;

	@Spy
	@InjectMocks
	private AttachmentsByWiedervorlageController controller;
	@Mock
	private WiedervorlageService service;
	@Mock
	private BinaryFileController binaryFileController;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetAttachments {

		private List<FileId> fileIds = List.of(FileId.from("42"));

		@BeforeEach
		void initMocks() {
			when(service.getById(any())).thenReturn(WiedervorlageTestFactory.create());
			doReturn(fileIds).when(controller).mapToId(any());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getById(anyString());
		}

		@Test
		void shouldCallBinaryFileController() throws Exception {
			callEndpoint();

			verify(binaryFileController).getFiles(fileIds);
		}

		private void callEndpoint() throws Exception {
			mockMvc.perform(get(PATH.replace("{wiedervorlageId}", WiedervorlageTestFactory.ID) + "/attachments")).andExpect(status().isOk());
		}
	}
}