package de.itvsh.goofy.wiedervorlage;

import java.util.Objects;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.itvsh.goofy.TestUtils;
import de.itvsh.goofy.common.command.CommandOrder;
import de.itvsh.goofy.common.command.CommandTestFactory;

public class WiedervorlageCommandTestFactory {

	public static final String ID = CommandTestFactory.ID;
	public static final CommandOrder ORDER = CommandOrder.CREATE_WIEDERVORLAGE;

	public static WiedervorlageCommand create() {
		return createBuilder().build();
	}

	public static WiedervorlageCommand createWithWiedervorlage(Wiedervorlage wiedervorlage) {
		return createBuilder().wiedervorlage(wiedervorlage).build();
	}

	public static WiedervorlageCommand.WiedervorlageCommandBuilder createBuilder() {
		return WiedervorlageCommand.builder()
				.id(ID)
				.order(ORDER)
				.wiedervorlage(WiedervorlageTestFactory.create());
	}

	public static String createValidRequestContent() {
		return createRequestContent(create());
	}

	public static String createRequestContent(WiedervorlageCommand command) {
		return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithWiedervorlage.json.tmpl",
				command.getOrder().name(),
				addTuedelchen(command.getWiedervorlage().getBetreff()),
				command.getWiedervorlage().getBeschreibung(),
				Optional.ofNullable(command.getWiedervorlage().getFrist()).map(Object::toString).orElse(StringUtils.EMPTY));
	}

	private static String addTuedelchen(String str) {
		return Objects.isNull(str) ? null : String.format("\"%s\"", str);
	}

	public static String createRequestContent(CommandOrder order) {
		return TestUtils.loadTextFile("jsonTemplates/command/createWiedervorlageOrderCommand.json.tmpl", order.name());
	}
}
