package de.itvsh.goofy.wiedervorlage;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.itvsh.goofy.binary.BinaryFileController;
import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class WiedervorlageControllerTest {

	private final String PATH = WiedervorlageController.WIEDERVORLAGE_PATH + "/";

	@Spy
	@InjectMocks
	private WiedervorlageController controller;
	@Mock
	private WiedervorlageService service;
	@Mock
	private WiedervorlageModelAssembler modelAssembler;
	@Mock
	private BinaryFileController binaryFileController;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetById {

		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockService() {
			when(service.getById(any())).thenReturn(wiedervorlage);
		}

		@Test
		void shouldReturnValue() throws Exception {
			callEndpoint();
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).getById(WiedervorlageTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toModel(wiedervorlage);
		}

		private void callEndpoint() throws Exception {
			mockMvc.perform(get(PATH + WiedervorlageTestFactory.ID)).andExpect(status().isOk());
		}
	}

	@Nested
	class TestFindByVorgang {
		private final Stream<Wiedervorlage> wiedervorlagen = Stream.of(WiedervorlageTestFactory.create());

		@BeforeEach
		void mockService() {
			when(service.findByVorgangId(any())).thenReturn(wiedervorlagen);
		}

		@Test
		void shouldCallEndpoint() throws Exception {
			callEndpoint();
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(wiedervorlagen, VorgangHeaderTestFactory.ID);
		}

		private void callEndpoint() throws Exception {
			mockMvc.perform(get(PATH).param(WiedervorlageController.PARAM_VORGANG_ID, VorgangHeaderTestFactory.ID))
					.andExpect(status().isOk());
		}
	}

	@Nested
	class TestFindNextFrist {
		@Test
		void shouldCallService() throws Exception {
			controller.findWithNextFrist(VorgangHeaderTestFactory.ID);

			verify(service).findWithNextFrist(VorgangHeaderTestFactory.ID);
		}
	}
}