package de.itvsh.goofy.wiedervorlage;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

class WiedervorlageMapperTest {

	private WiedervorlageMapper mapper = Mappers.getMapper(WiedervorlageMapper.class);

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMap() {
			var wiedervorlage = mapper.toWiedervorlage(GrpcWiedervorlageTestFactory.create());

			assertThat(wiedervorlage).isNotNull().usingRecursiveComparison().isEqualTo(WiedervorlageTestFactory.create());
		}

		@Test
		void shouldHandleNull() {
			var grpcWiedervorlage = mapper.toWiedervorlage(null);

			assertThat(grpcWiedervorlage).isNull();
		}
	}

	@Nested
	class TestToGrpc {

		@Test
		void shouldMap() {
			var grpcWiedervorlage = mapper.toGrpcWiedervorlage(WiedervorlageTestFactory.createBuilder().id(null).build());

			assertThat(grpcWiedervorlage).isNotNull().usingRecursiveComparison().ignoringFields("id_", "memoizedHashCode")
					.isEqualTo(GrpcWiedervorlageTestFactory.create());
		}

		@Test
		void shouldHandleNull() {
			var grpcWiedervorlage = mapper.toGrpcWiedervorlage(null);

			assertThat(grpcWiedervorlage).isNull();
		}
	}
}