package de.itvsh.goofy.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.LocalDate;
import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.goofy.vorgang.VorgangHeaderTestFactory;

class WiedervorlageServiceTest {

	@InjectMocks
	private WiedervorlageService service;

	@Mock
	private WiedervorlageRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(WiedervorlageTestFactory.ID);

			verify(remoteService).getById(WiedervorlageTestFactory.ID);
		}
	}

	@Nested
	class TestFindByVorgangId {

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(remoteService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestCreateCreateCommand {

		@Test
		void shouldCallRemoteService() {
			var wiedervorlageCommand = WiedervorlageCommandTestFactory.create();

			service.createCreateCommand(VorgangHeaderTestFactory.ID, wiedervorlageCommand);

			verify(remoteService).sendCreateWiedervorlageCommand(VorgangHeaderTestFactory.ID, wiedervorlageCommand);
		}
	}

	@Nested
	class TestCreateCommand {

		@Test
		void shouldCallRemoteService() {
			var wiedervorlageCommand = WiedervorlageCommandTestFactory.create();

			service.createCommand(WiedervorlageTestFactory.ID, wiedervorlageCommand);

			verify(remoteService).sendWiedervorlageCommand(WiedervorlageTestFactory.ID, wiedervorlageCommand);
		}
	}

	@Nested
	class TestFindWithNextFrist {

		@Test
		void shouldLoadAll() {
			service.findWithNextFrist(VorgangHeaderTestFactory.ID);

			verify(remoteService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnNextFrist() {
			when(remoteService.findByVorgangId(anyString())).thenReturn(Stream.of(
					WiedervorlageTestFactory.createBuilder().id("2").frist(LocalDate.parse("2021-04-02")).build(),
					WiedervorlageTestFactory.createBuilder().id("1").frist(LocalDate.parse("2021-04-01")).build()));

			var nextWiedervorlage = service.findWithNextFrist(VorgangHeaderTestFactory.ID);

			assertThat(nextWiedervorlage).isPresent().get().extracting(Wiedervorlage::getId).isEqualTo("1");
		}

		@Test
		void shouldHandleEmptyResult() {
			when(remoteService.findByVorgangId(anyString())).thenReturn(Stream.empty());

			var nextWiedervorlage = service.findWithNextFrist(VorgangHeaderTestFactory.ID);

			assertThat(nextWiedervorlage).isEmpty();
		}
	}
}